package com.dragonsoft.dcuc.approve.enumresources;

import com.dragonsoft.duceap.base.enums.ICodeEnum;
import lombok.Getter;

/**
 * 权限申请类型枚举
 *
 * @author mazq
 * @date 2021/2/22
 */
public enum FlowTypeEnum implements ICodeEnum {

    /**
     * 应用功能权限
     */
    APP_FUN_AUTH("APP_FUN_AUTH", "应用功能授权工作单", "应用功能资源"),

    /**
     * 服务权限
     */
    SERVICE_AUTH("SERVICE_AUTH", "服务授权工作单", "服务资源"),

    /**
     * 服务权限撤销
     */
    SERVICE_AUTH_CANCEL("SERVICE_AUTH_CANCEL", "权限撤销工作单", "服务资源"),

    /**
     * 数据授权
     */
    DATA_AUTH("DATA_AUTH", "数据授权工作单", "资源资源"),

    /**
     * 业务任务
     */
    TASK("TASK", "业务任务", "任务"),
    /**
     * 其他
     */
    OTHER("OTHER", "其他", "其他");


    private final String value;

    private final String label;

    @Getter
    private final String resourceName;

    FlowTypeEnum(String value, String label, String resourceName) {
        this.value = value;
        this.label = label;
        this.resourceName = resourceName;
    }

    @Override
    public String getValue() {
        return value;
    }


    @Override
    public String getLabel() {
        return label;
    }

    public ListResourceTypeEnum toListResourceTypeEnum() {
        switch (this) {
            case DATA_AUTH:
                return ListResourceTypeEnum.DATA;
            case APP_FUN_AUTH:
                return ListResourceTypeEnum.FUNCTION;
            case SERVICE_AUTH:
            case SERVICE_AUTH_CANCEL:
                return ListResourceTypeEnum.SERVICE;
            case OTHER:
            default:
                throw new IllegalArgumentException(String.format("%s isn't parse to ListResourceTypeEnum", this));
        }
    }

    public ApproveTypeEnum toApproveTypeEnum() {
        switch (this) {
            case DATA_AUTH:
                return ApproveTypeEnum.DATA;
            case SERVICE_AUTH:
            case SERVICE_AUTH_CANCEL:
                return ApproveTypeEnum.SERVICE;
            case APP_FUN_AUTH:
                return ApproveTypeEnum.APP_FUNCTION;
            case TASK:
                return ApproveTypeEnum.TASK;
            case OTHER:
                return ApproveTypeEnum.OTHER;
            default:
                throw new IllegalArgumentException(String.format("%s is'nt parse to ApproveTypeEnum", this));
        }
    }

}
