package com.dragonsoft.dcuc.approve.facade;

import com.dragonsoft.dcuc.approve.model.req.ApproveTaskClassAddReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApproveTaskClassUpdateReqVO;
import com.dragonsoft.dcuc.approve.model.resp.ApproveTaskClassDetailRespVO;
import com.dragonsoft.duceap.base.annotations.parameter.QueryObject;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.core.search.Searchable;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.data.domain.Page;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

/**
 * @author huangzqa
 * @date 2023/06/09
 */
@Api(tags = "任务类型")
@FeignClient(name = "approve")
public interface IApproveTaskClassFacade {

    /**
     * 任务类型列表
     *
     * @param searchable 查询条件
     * @return 分页
     */
    @ApiOperation(value = "任务类型列表", notes = "任务类型列表")
    @RequestMapping(value = "v1/task-class/search", method = RequestMethod.POST)
    Page<ApproveTaskClassDetailRespVO> page(@QueryObject Searchable searchable);

    /**
     * 删除任务类型
     *
     * @param id id
     * @return 状态
     */
    @ApiOperation(value = "删除任务类型", notes = "删除任务类型")
    @RequestMapping(value = "v1/task-class/{id}", method = RequestMethod.DELETE)
    ResponseStatus delete(@PathVariable(value = "id") String id);

    /**
     * 任务类型
     *
     * @param approveTaskClassAddReqVO 任务类型
     * @return 状态
     */
    @ApiOperation(value = "任务类型", notes = "任务类型")
    @RequestMapping(value = "v1/task-class", method = RequestMethod.POST)
    ResponseDTO<String> save(@Validated @RequestBody ApproveTaskClassAddReqVO approveTaskClassAddReqVO);

    /**
     * 修改任务类型
     *
     * @param approveTaskClassUpdateReqVO 修改任务类型
     * @return 状态
     */
    @ApiOperation(value = "修改任务类型", notes = "修改任务类型")
    @RequestMapping(value = "v1/task-class", method = RequestMethod.PUT)
    ResponseDTO<String> update(@Validated @RequestBody ApproveTaskClassUpdateReqVO approveTaskClassUpdateReqVO);

    /**
     * 获取任务类型详情
     *
     * @param id 任务类型ID
     * @return 状态
     */
    @ApiOperation(value = "任务类型详情", notes = "任务类型详情")
    @RequestMapping(value = "v1/task-class/{id}", method = RequestMethod.GET)
    ResponseDTO<ApproveTaskClassDetailRespVO> detail(@PathVariable("id") String id);

}