package com.dragonsoft.dcuc.approve.facade.api;

import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApprovalInfoUpdateReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApproveOperateReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApproveRecordReportReqVO;
import com.dragonsoft.dcuc.approve.model.resp.*;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.duwf.api.model.TraceProcessAndOverVo;
import com.dragonsoft.duceap.duwf.api.model.WorkflowProcesstypeVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author : wang.rs huangzqa
 * @date : 2020-12-15
 */
@Api(tags = "审批服务接口")
@FeignClient(name = "approve")
public interface IApiV1ApproveFacade {
    /**
     * 审批请求推送接口
     *
     * @param approvalInfoReqVO 审批请求信息
     * @return 状态
     */
    @ApiOperation(value = "审批请求推送接口", notes = "审批请求推送接口")
    @RequestMapping(value = "api/v1/apply", method = RequestMethod.POST)
    ResponseDTO<String> save(@Validated @RequestBody ApprovalInfoReqVO approvalInfoReqVO);

    /**
     * 修改
     *
     * @param approvalInfoDTO 审批信息
     * @return 状态
     */
    @ApiOperation(value = "修改", notes = "修改")
    @RequestMapping(value = "api/v1/update", method = RequestMethod.PUT)
    ResponseDTO<String> update(@Validated @RequestBody ApprovalInfoUpdateReqVO approvalInfoDTO);

    /**
     * 查询审批流程详情
     *
     * @param id 审批ID
     * @return 审批流程信息
     */
    @ApiOperation(value = "查询审批流程详情", notes = "查询审批流程详情")
    @RequestMapping(value = "api/v1/apply/{id}", method = RequestMethod.GET)
    ResponseDTO<ApprovalInfoRespVO> applyDetail(@PathVariable(value = "id") String id);

    /**
     * 审批操作
     *
     * @param approveOperateReqVO 审批操作
     * @return 结果
     */
    @ApiOperation(value = "审批操作", notes = "审批操作")
    @RequestMapping(value = "api/v1/approve-operate", method = RequestMethod.POST)
    ResponseDTO<OperateRespVO> approveOperate(@Validated @RequestBody ApproveOperateReqVO approveOperateReqVO);

    /**
     * 获取流程图
     *
     * @param processInstanceId 流程实例ID
     * @return 流程图
     */
    @ApiOperation(value = "获取流程图", notes = "获取流程图")
    @RequestMapping(value = "api/v1/apply-info/{processInstanceId}/process-pic", method = RequestMethod.GET)
    ResponseDTO<TraceProcessAndOverVo> processPic(@PathVariable(value = "processInstanceId") String processInstanceId);

    /**
     * 删除审批流程
     *
     * @param id 审批ID
     * @return 状态
     */
    @ApiOperation(value = "删除审批流程", notes = "删除审批流程")
    @RequestMapping(value = "api/v1/delApply/{id}", method = RequestMethod.DELETE)
    ResponseStatus delApply(@PathVariable(value = "id") String id);

    /**
     * 获取审批意见列表
     *
     * @param processInstanceId 流程实例ID
     * @return 审批意见列表
     */
    @ApiOperation(value = "审批意见", notes = "审批意见")
    @RequestMapping(value = "api/v1/apply-info/{processInstanceId}/approve-comments", method = RequestMethod.GET)
    ResponseStatus approveComments(@PathVariable(value = "processInstanceId") String processInstanceId);

    /**
     * 流程图状态
     *
     * @param processInstanceId 流程实例ID
     * @return 流程图状态
     */
    @ApiOperation(value = "流程图状态", notes = "流程图状态")
    @RequestMapping(value = "api/v1/apply-info/{processInstanceId}/processStatus", method = RequestMethod.GET)
    ResponseDTO<List<ProcessStatusVo>> processStatus(@PathVariable(value = "processInstanceId") String processInstanceId);

    /**
     * 获取流程定义列表
     *
     * @return 流程定义列表
     */
    @ApiOperation(value = "获取流程定义列表", notes = "获取流程定义列表")
    @RequestMapping(value = "api/v1/process-types", method = RequestMethod.GET)
    ResponseDTO<List<ApproveWorkflowProcessTypeVo>> queryProcessTypeList();

    /**
     * 获取流程定义详情
     *
     * @param processTypeCode 流程定义
     * @return 流程定义详情
     */
    @ApiOperation(value = "获取流程定义详情", notes = "获取流程定义详情")
    @RequestMapping(value = "api/v1/process-types/{processTypeCode}", method = RequestMethod.GET)
    ResponseDTO<WorkflowProcesstypeVo> queryProcessType(@PathVariable(value = "processTypeCode") String processTypeCode);

    /**
     * 同步应用，服务资源
     *
     * @return 状态
     */
    @ApiOperation(value = "同步应用，服务资源", notes = "同步应用，服务资源")
    @RequestMapping(value = "api/v1/sync-resource", method = RequestMethod.GET)
    ResponseStatus syncResource();

    /**
     * 根据任务ID查询任务分支（运行时任务）
     *
     * @param taskId 任务ID
     * @return 任务分支信息
     */
    @ApiOperation(value = "根据任务ID查询任务分支（运行时任务）", notes = "根据任务ID查询任务分支（运行时任务）")
    @RequestMapping(value = "api/v1/apply/tasks/task-out/{taskId}", method = RequestMethod.GET)
    ResponseDTO<List<TaskOutgoingVo>> taskOut(@PathVariable(value = "taskId") String taskId);

    /**
     * 获取任务支持的操作对象信息
     *
     * @param taskId   任务ID
     * @param pageType 页面类型
     * @return 支持的操作对象
     * @see PageTypeEnum pageType
     */
    @ApiOperation(value = "获取任务支持的操作对象信息", notes = "获取任务支持的操作对象信息")
    @RequestMapping(value = "api/v1/task-operate/{taskId}/{pageType}", method = RequestMethod.GET)
    ResponseDTO<TaskOperateRespVO> taskOperate(@PathVariable(value = "taskId") String taskId, @PathVariable(value = "pageType") String pageType);

    /**
     * 审批信息报送
     *
     * @param approveRecordList 报送信息
     * @return 状态
     */
    @ApiOperation(value = "审批信息报送", notes = "审批信息报送")
    @RequestMapping(value = "api/v1/approve-upload", method = RequestMethod.POST)
    ResponseStatus approveUpload(@RequestBody List<ApproveRecordReportReqVO> approveRecordList);

    /**
     * 流程信息
     *
     * @param processNo 流程代码
     * @return 状态
     */
    @ApiOperation(value = "流程信息", notes = "流程信息")
    @GetMapping(value = "api/v1/processes/{processNo}")
    ResponseDTO<ProcessDetailRespVO> processDetail(@PathVariable String processNo);

    /**
     * 批量获取任务支持的操作对象信息
     *
     * @param apiSearchReq 搜索条件
     * @return 支持的操作对象
     * @see PageTypeEnum pageType
     */
    @ApiOperation(value = "获取任务支持的操作对象信息", notes = "获取任务支持的操作对象信息")
    @PostMapping(value = "api/v1/operate-types/search")
    ResponseDTO<List<BatchTaskOperateRespVO>> batchApproveOperateType(@RequestBody ApiSearchReq apiSearchReq);

}