package com.dragonsoft.dcuc.approve.model.req;

import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.Date;

/**
 * 权限审批申请信息
 *
 * @author lidr
 * @date 2021/2/3
 */
@Data
@ApiModel(description = "权限审批申请信息")
public class ApprovalInfoAuthReqVO implements Serializable {

    /**
     * 流程的标题
     */
    @ApiModelProperty(value = "流程标题")
    private String processTitle;

    @ApiModelProperty(value = "流程标识码,从审批中心获取")
    @NotBlank
    private String businessCode;

    /**
     * @see FlowTypeEnum
     */
    @NotBlank
    @ApiModelProperty(value = "申请类型")
    private String flowType;

    @ApiModelProperty(value = "申请理由")
    @NotBlank
    private String applyReason;

    @ApiModelProperty(value = "申请人身份证号")
    @NotBlank
    @Length(max = 18, message = "请输入正确的身份证号")
    private String applicantIdcard;

    @ApiModelProperty(value = "申请人姓名")
    @NotBlank
    private String applicantName;

    @ApiModelProperty(value = "申请人单位编码")
    @NotBlank
    private String applicantOrgCode;

    @ApiModelProperty(value = "申请人单位名称")
    @NotBlank
    private String applicantOrgName;

    @ApiModelProperty(value = "申请人联系方式")
    @NotBlank
    private String applicantPhoneNo;

    /**
     * @see com.dragonsoft.dcuc.approve.enumresources.PermissionValidTypeEnum
     */
    @NotBlank
    @ApiModelProperty(value = "有效时间类别")
    private String permissionValidType;

    @ApiModelProperty(value = "开始时间")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date startTime;

    @ApiModelProperty(value = "结束时间")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    private Date endTime;

    @Length(max = 128, message = "请输入正确的消息ID")
    @ApiModelProperty(value = "第三方消息ID")
    private String messageId;

    @ApiModelProperty(value = "访问资源信息")
    @Valid
    @NotNull
    private ResourceInfoReqVO resourceInfo;


}
