package com.dragonsoft.dcuc.approve.model.req.v3;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 创建审批请求
 * </p>
 *
 * @author huangzqa
 * @date 2023/5/22
 */
@ApiModel("创建审批请求")
@Setter
@Getter
@ToString
public class ZeroTrustApproveCreateReqV3VO implements Serializable {

    /**
     * 应用令牌
     */
    @NotBlank
    @ApiModelProperty("应用令牌")
    private String appTokenId;

    /**
     * 流程标识
     * 调用审批服务需先在审批中心注册流程，每个注册流程有唯一流程定义ID
     */
    @NotBlank
    @ApiModelProperty("流程标识")
    private String processDefId;

    /**
     * 任务名称
     */
    @NotBlank
    @ApiModelProperty("任务名称")
    private String title;

    /**
     * 任务类型
     * “01”表示落地调查 “02”表示立线侦察 “03” 表示立案侦察 “04”表示任务初查 “05”表示刑事任务 “06” 表示重点对象管控 “07” 表示行政任务 “08” 表示权限变更
     * “98” 表示平台演示 “99” 表示系統巡检
     */
    @NotBlank
    @ApiModelProperty("任务类型")
    private String taskClass;

    /**
     * 任务编号
     * 审批流程所关联的任务唯一标识，任务id编码规则（共32位）：RWBH+公安机关组织机构代码（符合GA/T 380－2012的要求）+日期（格式YYYYMMDD）+8位流水号。如：RWBH0100000000002022030100000001
     */
    @NotBlank
    @ApiModelProperty("任务编号")
    private String taskId;

    /**
     * 表单
     * “表名”：[{"字段 1":"值","字段 2":"值"}，{"字段 1":"值","字段2":"值"}]}
     */
    @ApiModelProperty("表单")
    private Map<String, List<Map<String, Object>>> bo;

    /**
     * 应用接收审批结果地址（应用开发的接收审批结果变动的地址）
     */
    @NotBlank
    @ApiModelProperty("应用接收审批结果地址")
    private String callbackUrl;

    /**
     * 业务数据
     * 扩展字段
     */
    @ApiModelProperty("业务数据")
    private Map<String, Object> bizData;

    /**
     * 签名
     * 应采用国产密码算法对接口请求参数进行完整性保护
     */
    @NotBlank
    @ApiModelProperty("签名")
    private String callerSign;

}
