package com.dragonsoft.dcuc.approve.business;

import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.ApproveActRunIdentityLink;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.req.ResourceInfoReqVO;
import com.dragonsoft.dcuc.approve.model.resp.ApprovalInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.BatchTaskOperateRespVO;
import com.dragonsoft.dcuc.approve.model.dto.ContentRespDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.duwf.api.model.ProcessInstanceVO;
import com.dragonsoft.duceap.duwf.api.model.TaskHistoryDetailVo;

import java.util.List;

/**
 * <p>
 * 审批业务类
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/19
 */
public interface IApproveBusiness {


    /**
     * 删除待办数据，新增已办数据
     *
     * @param taskId                任务id
     * @param workFlowOperationEnum 操作类型
     * @param operateUserIdcard     操作者身份证号
     * @param operateUserName       操作者身份姓名
     * @param taskRunDetailVos      流程实例对象
     */
    void afterTaskCommitTask(String taskId, ApproveWorkFlowOperationEnum workFlowOperationEnum, String operateUserIdcard, String operateUserName, List<TaskHistoryDetailVo> taskRunDetailVos);

    /**
     * 构建待办人信息
     *
     * @param taskHistoryDetailVo 下一个节点信息
     * @return 待办人信息
     */
    List<ApproveActRunIdentityLink> buildActIdentityLink(TaskHistoryDetailVo taskHistoryDetailVo);

    /**
     * 流程开启生成待办，已办
     * 第一个节点提交的时候没有对应的已办事项
     *
     * @param processInstanceVO 实例信息
     * @param approvalInfoDTO      审批信息
     */
    void createTodoAfterProcessStart(ProcessInstanceVO processInstanceVO, ApprovalInfoDTO approvalInfoDTO);

    /**
     * 检查资源
     *
     * @param flowTypeEnum 资源类型
     * @param resourceInfoReqVO 资源信息
     * @return 状态
     */
    ResponseStatus checkResource(FlowTypeEnum flowTypeEnum, ResourceInfoReqVO resourceInfoReqVO);

    /**
     * 获取任务支持的操作对象信息
     *
     * @param approveNoList 审批单号
     * @param pageTypeEnum  页面类型
     * @return 任务支持操作对象
     */
    List<BatchTaskOperateRespVO> batchTaskOperate(List<String> approveNoList, PageTypeEnum pageTypeEnum);

    /**
     * 生成内容显示VO
     *
     * @param flowTypeEnum    流程类型
     * @param resourceInfoDTO 资源
     * @return 内容
     */
    ContentRespDTO generalContentRespVO(FlowTypeEnum flowTypeEnum, ResourceInfoDTO resourceInfoDTO);

    /**
     * 获取当前任务处理人
     *
     * @param currentTaskId 当前任务ID
     * @return 当前任务处理人
     */
    String getUnOperateUsers(String currentTaskId);

    /**
     * 判断当前登录人是否为科信人员
     *
     * @param idCard 身份证号
     * @return 是否是
     */
    Boolean isKxPeople(String idCard);

    /**
     * 设置流程信息
     *
     * @param approvalInfoDTO 审批申请内容
     */
    void setProcessInfo(ApprovalInfoDTO approvalInfoDTO);

    /**
     * 设置我的请求当前结点和未操作者信息
     *
     * @param approvalInfoDTO 审批信息
     */
    ApprovalInfoRespVO fillTaskInfoToApplyInfo(ApprovalInfoDTO approvalInfoDTO);

    /**
     * 更新当前结点任务
     *
     * @param approvalInfoDTO 审批信息
     */
    void buildTaskInfoByProcessInstanceId(ApprovalInfoDTO approvalInfoDTO, List<TaskHistoryDetailVo> list);

    /**
     * 设置资源信息
     *
     * @param approveRecordInfo 审批记录表
     * @param resourceInfoDTO   资源信息
     */
    void setResourceInfoData(ApproveRecordInfo approveRecordInfo, ResourceInfoDTO resourceInfoDTO);
}
