package com.dragonsoft.dcuc.approve.business;

import com.dragonsoft.dcuc.approve.enumresources.OperateOptionEnum;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.resp.ApproveCommentVo;
import com.dragonsoft.dcuc.approve.model.resp.ApproveWorkflowProcessTypeVo;
import com.dragonsoft.dcuc.approve.model.resp.TaskOperateRespVO;
import com.dragonsoft.dcuc.approve.model.resp.TaskOutgoingVo;
import com.dragonsoft.duceap.duwf.api.model.ActivityNodeInfoVo;
import com.dragonsoft.duceap.duwf.api.model.TraceProcessAndOverVo;
import com.dragonsoft.duceap.duwf.api.model.WorkflowProcessVo;
import com.dragonsoft.duceap.duwf.api.model.WorkflowProcesstypeVo;

import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/13
 */
public interface IWorkFlowBusiness {
    /**
     * 根据流程实例获取流程的定义任务集合
     *
     * @param processInstanceId 流程实例ID
     * @return 审批意见
     */
    List<ApproveCommentVo> getProcessDefPath(String processInstanceId);

    /**
     * 根据任务ID查看当前节点允许的操作类型
     *
     * @param taskId            任务ID
     * @param processInstanceId 实例ID
     * @param taskNode          任务节点
     * @param operateOptionEnum 操作类型
     * @return 是否允许
     */
    boolean checkOperateOptionOfCurrentStep(String taskId, String processInstanceId, String taskNode, OperateOptionEnum operateOptionEnum);

    /**
     * 获取流程信息
     *
     * @param businessCode 业务代码
     * @return 流程信息
     */
    WorkflowProcessVo queryWorkflowProcessByBusinessCode(String businessCode);

    /**
     * 获取流程类别列表
     *
     * @return 流程类别列表
     */
    List<ApproveWorkflowProcessTypeVo> queryProcessTypeList();


    /**
     * 获取流程类别详情
     *
     * @param processTypeCode 流程类别code
     * @return 程类别详情
     */
    WorkflowProcesstypeVo queryProcessType(String processTypeCode);

    /**
     * 查询任务分支（运行时任务）
     *
     * @param taskId 任务ID
     * @return 分支信息
     */
    List<TaskOutgoingVo> buildTaskOutgoingVoList(String taskId);

    /**
     * 获取任务支持操作
     *
     * @param taskId       任务ID
     * @param pageTypeEnum 页面类型
     * @return 支持类型
     */
    TaskOperateRespVO getTaskSupportOperate(String taskId, PageTypeEnum pageTypeEnum);

    /**
     * 流程图数据
     *
     * @param proInsId 实例ID
     * @return 流程图
     */
    TraceProcessAndOverVo processPic(String proInsId);

    /**
     * 获取流程图节点信息
     *
     * @param proInsId         实例id
     * @param statusType       状态类型
     * @param taskDefinitionId 任务定义ID
     * @return 节点详情
     */
    ActivityNodeInfoVo processPicDetail(String proInsId, String statusType, String taskDefinitionId);

}
