package com.dragonsoft.dcuc.approve.business.approver.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.dragonsoft.dcuc.approve.business.approver.IGetApproverBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucBusiness;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiOrgResultVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.OauthUserVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * <p>
 * 一级一级的获取机构的审批人
 * </p>
 *
 * @author huangziquan
 * @date 2024/3/4
 */
@Component(value = "nextAndNextOrgGetApproverBusiness")
public class NextAndNextOrgGetApproverBusinessImpl implements IGetApproverBusiness {
    private IDcucBusiness dcucBusiness;

    @Autowired
    public void setDcucBusiness(IDcucBusiness dcucBusiness) {
        this.dcucBusiness = dcucBusiness;
    }

    @Override
    public List<String> getApproverList(String applicantUser, String labelCode) {
        // 获取申请人对应的机构代码
        OauthUserVO handleUserInfo = dcucBusiness.getOauthUserInfoByIdcard(applicantUser);
        String currentOrgCode = handleUserInfo.getOrgCode();
        // 看有没有再获取上级机构的
        ApiOrgResultVo apiOrgResultVo = dcucBusiness.orgInfo(currentOrgCode);
        String upOrgCode = apiOrgResultVo.getUpOrgCode();
        // 获取机构代码上有这个标签的人
        return getNextAndNextApproverList(upOrgCode, labelCode);
    }

    /**
     * 一级一级的获取
     *
     * @param orgCode   机构代码
     * @param labelCode 标签代码
     * @return 审批人
     */
    protected List<String> getNextAndNextApproverList(String orgCode, String labelCode) {
        // 看有没有再获取上级机构的
        List<String> approverList = dcucBusiness.getUserByLabelCodeUserList(orgCode, labelCode);
        if (CollUtil.isEmpty(approverList)) {
            ApiOrgResultVo apiOrgResultVo = dcucBusiness.orgInfo(orgCode);
            String upOrgCode = apiOrgResultVo.getUpOrgCode();
            if (CollUtil.isEmpty(approverList)
                    && StrUtil.isNotBlank(upOrgCode)) {
                approverList = getNextAndNextApproverList(upOrgCode, labelCode);
            }
        }
        return approverList;
    }

}
