package com.dragonsoft.dcuc.approve.business.external;

import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.ListLevelEnum;
import com.dragonsoft.dcuc.approve.model.resp.OperateRespVO;
import com.dragonsoft.dcuc.approve.model.vo.AuthAppFunApiVO;

import java.util.List;
import java.util.Optional;
import java.util.Set;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/12
 */
public interface IDcucAuthBusiness {

    /**
     * 获取白名单等级
     *
     * @param idcard 身份证号
     * @return 等级
     */
    Optional<ListLevelEnum> getWhiteListLevel(String idcard);

    /**
     * 获取红名单等级
     *
     * @param flowTypeEnum 流程类型
     * @param resourceCode 资源代码
     * @return 等级
     */
    Set<ListLevelEnum> getRedListLevel(FlowTypeEnum flowTypeEnum, String resourceCode);

    /**
     * 判断是否命中红名单
     *
     * @param checkIdcard     检查身份证号
     * @param flowTypeEnum    流程类型
     * @param resourceInfoDTO 资源列表
     * @return 命中状态
     */
    OperateRespVO checkHitRedList(String checkIdcard, FlowTypeEnum flowTypeEnum, ResourceInfoDTO resourceInfoDTO);

    /**
     * 获取功能列表
     *
     * @param appCode         应用代码
     * @param functionCodList 功能列表
     * @return 功能列表
     */
    List<AuthAppFunApiVO> getFunctionList(String appCode, List<String> functionCodList);

}
