package com.dragonsoft.dcuc.approve.business.external;

import com.dragoninfo.dcuc.common.entity.ApiResultPage;
import com.dragoninfo.dcuc.common.entity.ApiSearchReq;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiOrgResultVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.ApiPoliceResultVo;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.OauthUserVO;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.UserLabel2RespVO;

import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/12
 */
public interface IDcucBusiness {

    /**
     * 获取认证用户信息
     *
     * @param searchIdcard 查询的身份证号
     * @return 认证用户信息
     */
    OauthUserVO getOauthUserInfoByIdcard(String searchIdcard);

    /**
     * 获取机构信息
     *
     * @param orgCode 机构代码
     * @return 机构信息
     */
    ApiOrgResultVo orgInfo(String orgCode);

    /**
     * 获取机构列表
     *
     * @param apiSearchReq 请求参数
     * @return 机构列表
     */
    ApiResultPage<ApiOrgResultVo> orgPage(ApiSearchReq apiSearchReq);

    /**
     * 获取警员列表
     *
     * @param apiSearchReq 请求参数
     * @return 警员列表
     */
    ApiResultPage<ApiPoliceResultVo> policePage(ApiSearchReq apiSearchReq);

    /**
     * 根据身份证号查询警员信息
     *
     * @param idcard 身份证号
     * @return 警员信息
     */
    ApiPoliceResultVo policeDetailByIdcard(String idcard);

    /**
     * 根据标签代码获取对应人员信息
     *
     * @param orgCode   机构代码
     * @param labelCode 标签代码
     * @return 人员信息
     */
    List<UserLabel2RespVO> getUserByLabelCode(String orgCode, String labelCode);

    /**
     * 根据机构代码、标签代码获取审批人员
     *
     * @param orgCode   机构代码
     * @param labelCode 标签代码
     * @return 审批人
     */
    List<String> getUserByLabelCodeUserList(String orgCode, String labelCode);

}
