package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.ObjectUtil;
import com.dragoninfo.dcuc.common.annotation.CacheLock;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.api.IApproveApiBusiness;
import com.dragonsoft.dcuc.approve.business.external.IBuSyncBusiness;
import com.dragonsoft.dcuc.approve.business.external.INoticeBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.enumresources.bu.TaskStatusEnum;
import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.ApproveMessageInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.bu.BuApproveSyncBizApproveReqVO;
import com.dragonsoft.dcuc.approve.model.req.bu.BuApproveSyncReqVO;
import com.dragonsoft.dcuc.approve.service.IApproveCirculationService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.core.persistent.transaction.TransactionSupport;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/11/7
 */
@Slf4j
@Component
public class BuSyncBusinessImpl implements IBuSyncBusiness {

    private IApproveRecordService approveRecordService;

    private IApproveCirculationService approveCirculationService;

    private INoticeBusiness noticeBusiness;

    private IApproveApiBusiness approveApiBusiness;

    @Autowired
    public void setApproveApiBusiness(IApproveApiBusiness approveApiBusiness) {
        this.approveApiBusiness = approveApiBusiness;
    }

    @Autowired
    public void setNoticeBusiness(INoticeBusiness noticeBusiness) {
        this.noticeBusiness = noticeBusiness;
    }

    @Autowired
    public void setApproveCirculationService(IApproveCirculationService approveCirculationService) {
        this.approveCirculationService = approveCirculationService;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Override
    @CacheLock
    public ResponseStatus syncTuBu() {
        TransactionSupport.execute(() -> {
            List<ApproveRecordInfo> unSyncList = approveRecordService.getUnSyncList();
            for (ApproveRecordInfo approveRecordInfo : unSyncList) {
                String id = approveRecordInfo.getId();
                String approveNo = approveRecordInfo.getApproveNo();
                Date startTime = approveRecordInfo.getStartTime();
                Date endTime = approveRecordInfo.getEndTime();

                //  检查任务编号是否正确,且开始、结束时间不为空
                boolean isNeedSync = approveApiBusiness.isStanderTaskNo(approveNo)
                        && ObjectUtil.isNotNull(startTime) && ObjectUtil.isNotNull(endTime);

                if (isNeedSync) {
                    ResponseStatus responseStatus = syncToBu(approveRecordInfo);
                    BooleanEnum booleanEnum = BooleanEnum.TRUE;
                    if (ResponseUtil.isFail(responseStatus)) {
                        log.error("同步到BU出错:{}", responseStatus.getMessage());
                        booleanEnum = BooleanEnum.FALSE;
                    }
                    approveRecordService.updateSyncFlag(id, booleanEnum);

                } else {
                    // 非标准的任务ID默认设置为成功
                    approveRecordService.updateSyncFlag(id, BooleanEnum.TRUE);
                }
            }
        });
        return ResponseStatus.success();
    }

    /**
     * 同步到BU
     *
     * @param approveRecordInfo 请求同步请求
     * @return 状态
     */
    @Transactional(rollbackFor = Throwable.class)
    @Override
    public ResponseStatus syncToBu(ApproveRecordInfo approveRecordInfo) {
        Assert.notNull(approveRecordInfo);
        ApprovalInfoDTO approvalInfoDTO = approveRecordInfo.toApprovalInfo();

        String id = approveRecordInfo.getId();

        BuApproveSyncReqVO buApproveSyncReqVO = new BuApproveSyncReqVO();

        ApproveCirculationInfo approveCirculationInfo = approveCirculationService.getListByApproveRecordLastAgree(id);
        Assert.notNull(approveCirculationInfo);
        // 设置最终审批人信息
        buApproveSyncReqVO.setLastReviewerId(approveCirculationInfo.getOperateUserId());
        buApproveSyncReqVO.setLastReviewerIdCard(approveCirculationInfo.getOperateUserId());
        buApproveSyncReqVO.setLastReviewerName(approveCirculationInfo.getOperateUserName());
        buApproveSyncReqVO.setLastReviewerUnitCode(approveCirculationInfo.getOperateUserOrgId());
        buApproveSyncReqVO.setLastReviewerUnitName(approveCirculationInfo.getOperateUserOrg());

        return syncToBu(id, buApproveSyncReqVO, approvalInfoDTO);
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public ResponseStatus syncToBu(ApproveMessageInfoDTO approveMessageInfoDTO) {
        ApprovalInfoDTO approvalInfoDTO = approveMessageInfoDTO.getApprovalInfoDTO();
        String id = approvalInfoDTO.getId();

        BuApproveSyncReqVO buApproveSyncReqVO = new BuApproveSyncReqVO();

        SecurityUser securityUser = approveMessageInfoDTO.getSecurityUser();

        Assert.notNull(securityUser);
        // 设置最终审批人信息
        buApproveSyncReqVO.setLastReviewerIdCard(securityUser.getIdcard());
        buApproveSyncReqVO.setLastReviewerName(securityUser.getName());
        buApproveSyncReqVO.setLastReviewerUnitCode(securityUser.getSecurityOrg());
        buApproveSyncReqVO.setLastReviewerUnitName(securityUser.getSecurityOrgName());
        buApproveSyncReqVO.setTaskId(approvalInfoDTO.getApproveNo());
        buApproveSyncReqVO.setCreaterId(approvalInfoDTO.getApplicantIdcard());

        return syncToBu(id, buApproveSyncReqVO, approvalInfoDTO);
    }

    /**
     * 同步到 BU
     *
     * @param id                 审批单ID
     * @param buApproveSyncReqVO 请求
     * @param approvalInfoDTO    审批单信息
     * @return 状态
     */
    protected ResponseStatus syncToBu(String id, BuApproveSyncReqVO buApproveSyncReqVO, ApprovalInfoDTO approvalInfoDTO) {
        buApproveSyncReqVO.setAppId(approvalInfoDTO.getAppCode());
        buApproveSyncReqVO.setAppName(approvalInfoDTO.getAppName());
        buApproveSyncReqVO.setCreateTime(approvalInfoDTO.getCreateTime());
        buApproveSyncReqVO.setFinishTime(approvalInfoDTO.getUpdateTime());
        buApproveSyncReqVO.setCreaterId(approvalInfoDTO.getApplicantIdcard());
        buApproveSyncReqVO.setTaskId(approvalInfoDTO.getApproveNo());
        buApproveSyncReqVO.setCreaterId(approvalInfoDTO.getApplicantIdcard());

        BuApproveSyncBizApproveReqVO buApproveSyncBizApproveReqVO = parseApprovalInfoDTO(approvalInfoDTO);

        Map<String, Object> bizData = new LinkedHashMap<>(2);
        bizData.put(ApproveConstants.TASK_INFO, buApproveSyncBizApproveReqVO);
        buApproveSyncReqVO.setBizData(bizData);

        ResponseStatus responseStatus = noticeBusiness.retryRequestSyncToBu(buApproveSyncReqVO);
        if (ResponseUtil.isSuccess(responseStatus)) {
            approveRecordService.updateSyncFlag(id, BooleanEnum.TRUE);
            return ResponseStatus.success();
        }
        approveRecordService.updateSyncFlag(id, BooleanEnum.FALSE);
        return ResponseStatus.fail();
    }

    /**
     * 转换
     *
     * @param approvalInfoDTO 审批单信息
     * @return 流程业务数据
     */
    protected BuApproveSyncBizApproveReqVO parseApprovalInfoDTO(ApprovalInfoDTO approvalInfoDTO) {
        Assert.notNull(approvalInfoDTO);
        BuApproveSyncBizApproveReqVO buApproveSyncBizApproveReqVO = new BuApproveSyncBizApproveReqVO();
        buApproveSyncBizApproveReqVO.setTitle(approvalInfoDTO.getProcessTitle());
        buApproveSyncBizApproveReqVO.setTaskClass(approvalInfoDTO.getApproveTaskClass());
        buApproveSyncBizApproveReqVO.setTaskStatus(TaskStatusEnum.ENABLE.value);
        buApproveSyncBizApproveReqVO.setTaskTermStart(approvalInfoDTO.getStartTime());
        buApproveSyncBizApproveReqVO.setTaskTermEnd(approvalInfoDTO.getEndTime());
        return buApproveSyncBizApproveReqVO;
    }


}
