package com.dragonsoft.dcuc.approve.business.external.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.auditlog.collection.qmtj.LogSendComponent;
import com.dragonsoft.auditlog.collection.qmtj.pojo.req.ApproveRegisterLog;
import com.dragonsoft.auditlog.collection.qmtj.pojo.resp.BaseResponse;
import com.dragonsoft.dcuc.approve.business.external.IQmAuditBusiness;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 启明审计业务类
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/5
 */
@Slf4j
@Service
public class QmAuditBusinessImpl implements IQmAuditBusiness {


    @Autowired
    private LogSendComponent logSendComponent;


    @Override
    public ResponseEntity<BaseResponse> registerInfo(ApproveRegisterLog approveRegisterLog) {
        String checkSum = approveRegisterLog.getCheckSum();
        if (StrUtil.isBlank(checkSum)) {
            String sysType = approveRegisterLog.getSysType();
            String sysId = approveRegisterLog.getSysId();
            String companyId = approveRegisterLog.getCompanyId();
            String orgcode = approveRegisterLog.getOrgcode();
            String sysCode = approveRegisterLog.getSysCode();
            String sysName = approveRegisterLog.getSysName();
            String sysDesc = approveRegisterLog.getSysDesc();
            String onlinetime = approveRegisterLog.getOnlinetime();
            String trustIpList = approveRegisterLog.getTrustIpList();
            List<String> targetIpList = approveRegisterLog.getTargetIpList();

            StringBuilder stringBuilder = new StringBuilder(sysType + "&");
            if (StrUtil.isNotBlank(sysId)) {
                stringBuilder.append(sysId).append("&");
            }
            stringBuilder.append(companyId).append("&").append(orgcode).append("&").append(sysCode)
                    .append("&").append(sysName).append("&").append(sysDesc).append("&").append(onlinetime)
                    .append("&").append(trustIpList).append("&").append(targetIpList);
            checkSum = stringBuilder.toString();
            log.info("register md5 source:{}", checkSum);

            checkSum = DigestUtil.md5Hex(checkSum);
            log.info("register md5 target:{}", checkSum);

            approveRegisterLog.setCheckSum(checkSum);
        }

        log.info("Qm audit log register request:{}", JsonUtils.toJSONString(approveRegisterLog));
        ResponseEntity<BaseResponse> responseEntity = logSendComponent.sendApproveRegisterLog(approveRegisterLog);
        log.info("QM audit log register response:{}", JsonUtils.toJSONString(responseEntity));
        return responseEntity;
    }

    @Override
    public ResponseDTO<String> md5(String md5source) {
        String md5Hex = DigestUtil.md5Hex(md5source);
        return ResponseUtil.newInstance(md5Hex);
    }
}
