package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.dragonsoft.dcuc.approve.business.IApproveBaseBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.constants.ApproveRedisConstants;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/6/8
 */
@Slf4j
@Component
public class ApproveBaseBusinessImpl implements IApproveBaseBusiness {

    private IApproveRecordService approveRecordService;

    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    public void setStringRedisTemplate(StringRedisTemplate stringRedisTemplate) {
        this.stringRedisTemplate = stringRedisTemplate;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Override
    public Date parseEndTime(Date endTime) {
        if (endTime != null) {
            String strEndTime = DateUtils.getDateFormat(endTime, DateConst.HYPHEN_DISPLAY_DATE);
            strEndTime += " 23:59:59";
            try {
                endTime = DateUtils.getDate(strEndTime, DateConst.HYPHEN_DISPLAY_TIME.getLabel());
            } catch (ParseException e) {
                log.error("parse", e);
            }
        }
        return endTime;
    }


    @Override
    public String buildApproveNo(String appCode, String orgCode) {
        Date date = DateUtils.getDate();
        return generalTaskNo(appCode, orgCode, date);
    }

    /**
     * 生成任务ID
     *
     * @param appCode 应用编号
     * @param orgCode 机构代码
     * @param date    日期
     * @return 任务ID
     */
    protected String generalTaskNo(String appCode, String orgCode, Date date) {
        Assert.notBlank(appCode);
        Assert.notBlank(orgCode);
        Assert.notNull(date);

        String currTimeStr = DateUtils.getTimeStr(date, DateConst.DB_STORE_TIME);
        String dateTimeStr = DateUtils.getTimeStr(date, DateConst.DB_STORE_DATE);

        // 当日的
        String key = ApproveRedisConstants.APPROVAL_TASK_ID_PREFIX + appCode + StrUtil.COLON + dateTimeStr;
        Boolean hasKey = stringRedisTemplate.hasKey(key);
        Long taskIdNo;
        // 有Key直接加1
        if (hasKey != null && hasKey) {
            taskIdNo = stringRedisTemplate.opsForValue().increment(key);
        } else {
            String existTaskId = approveRecordService.getTaskNoByDate(appCode, dateTimeStr);
            if (StringUtils.isBlank(existTaskId)) {
                taskIdNo = 1L;
            } else {
                taskIdNo = Long.valueOf(existTaskId);
                taskIdNo += 1;
            }
            // 都保留一天，存储量很小，不用担心存储问题
            Boolean setIfAbsent = stringRedisTemplate.opsForValue().setIfAbsent(key, String.valueOf(taskIdNo), 1, TimeUnit.DAYS);
            if (setIfAbsent == null || !setIfAbsent) {
                taskIdNo = stringRedisTemplate.opsForValue().increment(key);
            }
        }
        // 格式化为8位流水号
        return ApproveConstants.TASK_ID_PREFIX + orgCode + currTimeStr + String.format("%08d", taskIdNo);
    }


}
