package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.dragoninfo.dcuc.common.utils.LangUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.approve.common.ErrorCode;
import com.dragonsoft.dcuc.approve.business.IApproveBaseBusiness;
import com.dragonsoft.dcuc.approve.business.IApproveBusiness;
import com.dragonsoft.dcuc.approve.business.IWorkFlowBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucAuthBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.CandidateTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.FlowTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.ApproveActHiTask;
import com.dragonsoft.dcuc.approve.model.ApproveActRunIdentityLink;
import com.dragonsoft.dcuc.approve.model.ApproveActRunTask;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.ContentRespDTO;
import com.dragonsoft.dcuc.approve.model.dto.ResourceInfoDTO;
import com.dragonsoft.dcuc.approve.model.req.AppFunctionResourceNameReqVO;
import com.dragonsoft.dcuc.approve.model.req.AppFunctionResourceReqVO;
import com.dragonsoft.dcuc.approve.model.req.ResourceInfoReqVO;
import com.dragonsoft.dcuc.approve.model.resp.ApprovalInfoRespVO;
import com.dragonsoft.dcuc.approve.model.resp.BatchTaskOperateRespVO;
import com.dragonsoft.dcuc.approve.model.resp.TaskOperateRespVO;
import com.dragonsoft.dcuc.approve.model.vo.AuthAppFunApiVO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.service.IApproveActHiTaskService;
import com.dragonsoft.dcuc.approve.service.IApproveActRunIdentityLinkService;
import com.dragonsoft.dcuc.approve.service.IApproveActRunTaskService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import com.dragonsoft.duceap.duwf.api.model.CandidateVo;
import com.dragonsoft.duceap.duwf.api.model.ProcessInstanceVO;
import com.dragonsoft.duceap.duwf.api.model.TaskHistoryDetailVo;
import com.dragonsoft.duceap.duwf.api.model.WorkflowProcessVo;
import com.dragonsoft.duceap.duwf.api.model.WorkflowProcesstypeVo;
import com.dragonsoft.duceap.workflow.client.WorkflowClientFactory;
import com.dragonsoft.duceap.workflow.enums.WorkFlowOprationEnum;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.validation.constraints.NotBlank;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 审批业务类
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/19
 */
@Slf4j
@Service
public class ApproveBusinessImpl implements IApproveBusiness {

    private IApproveRecordService approveRecordService;

    private ApproveProperties approveProperties;

    private IApproveActRunTaskService approveActRunTaskService;

    private IApproveActHiTaskService approveActHiTaskService;

    private IApproveActRunIdentityLinkService approveActRunIdentityLinkService;

    private IWorkFlowBusiness workFlowBusiness;

    private IDcucAuthBusiness dcucAuthBusiness;

    private WorkflowClientFactory workflowClientFactory;

    private IApproveBaseBusiness approveBaseBusiness;

    @Autowired
    public void setApproveBaseBusiness(IApproveBaseBusiness approveBaseBusiness) {
        this.approveBaseBusiness = approveBaseBusiness;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Autowired
    public void setApproveActRunTaskService(IApproveActRunTaskService approveActRunTaskService) {
        this.approveActRunTaskService = approveActRunTaskService;
    }

    @Autowired
    public void setApproveActHiTaskService(IApproveActHiTaskService approveActHiTaskService) {
        this.approveActHiTaskService = approveActHiTaskService;
    }

    @Autowired
    public void setApproveActRunIdentityLinkService(IApproveActRunIdentityLinkService approveActRunIdentityLinkService) {
        this.approveActRunIdentityLinkService = approveActRunIdentityLinkService;
    }

    @Autowired
    public void setWorkFlowBusiness(IWorkFlowBusiness workFlowBusiness) {
        this.workFlowBusiness = workFlowBusiness;
    }

    @Autowired
    public void setDcucAuthBusiness(IDcucAuthBusiness dcucAuthBusiness) {
        this.dcucAuthBusiness = dcucAuthBusiness;
    }

    @Autowired
    public void setWorkflowClientFactory(WorkflowClientFactory workflowClientFactory) {
        this.workflowClientFactory = workflowClientFactory;
    }

    /**
     * 删除待办数据，新增已办数据
     *
     * @param taskId                任务id
     * @param workFlowOperationEnum 操作类型
     * @param operateUserIdcard     操作者身份证号
     * @param operateUserName       操作者身份姓名
     * @param taskRunDetailVos      流程实例对象
     */
    @Override
    public void afterTaskCommitTask(String taskId, ApproveWorkFlowOperationEnum workFlowOperationEnum, String operateUserIdcard,
                                    String operateUserName, List<TaskHistoryDetailVo> taskRunDetailVos) {
        Date now = new Date();
        ApproveActRunTask approveActRunTask = approveActRunTaskService.selectOneByTaskId(taskId);
        log.info("====获取到的待办任务为:{}====", approveActRunTask == null ? "null" : JSONObject.toJSONString(approveActRunTask));
        if (approveActRunTask != null) {
            log.info("====step1: 在待办表中删除当前结点的待办信息====");
            // step1: 删除当前结点
            approveActRunTask.setDeleted(BooleanEnum.TRUE.getValue());
            approveActRunTask.setOperateType(workFlowOperationEnum.getValue());
            // 使用@TableLogic需要使用Delete才能删除
            String approveActRunTaskId = approveActRunTask.getId();
            approveActRunTaskService.deleteById(approveActRunTaskId);
            // step2: 把当前代办写入已办
            log.info("====step2: 把当前代办写入已办====");
            ApproveActHiTask actHiTask = new ApproveActHiTask();
            BeanUtil.copyProperties(approveActRunTask, actHiTask);
            actHiTask.setDeleted(BooleanEnum.FALSE.getValue());
            actHiTask.setOperateTime(now);
            actHiTask.setOperateUserIdcard(operateUserIdcard);
            actHiTask.setOperateUserName(operateUserName);
            approveActHiTaskService.save(actHiTask);
            log.info("====step2: 生成新的待办====");
            // step3: 生成新的待办
            if (CollectionUtils.isNotEmpty(taskRunDetailVos)) {
                for (TaskHistoryDetailVo todoTask : taskRunDetailVos) {
                    ApproveActRunTask newApproveActRunTask = new ApproveActRunTask();
                    BeanUtil.copyProperties(approveActRunTask, newApproveActRunTask);
                    newApproveActRunTask.setDeleted(BooleanEnum.FALSE.getValue());
                    newApproveActRunTask.setTaskId(todoTask.getId());
                    newApproveActRunTask.setTaskName(todoTask.getName());
                    newApproveActRunTask.setTaskCode(todoTask.getTaskCode());
                    newApproveActRunTask.setId(null);
                    approveActRunTaskService.save(newApproveActRunTask);
                    // 待办人员列表
                    if (ApproveWorkFlowOperationEnum.SUSPEND.getValue().equals(newApproveActRunTask.getOperateType()) ||
                            ApproveWorkFlowOperationEnum.ACTIVATE.getValue().equals(newApproveActRunTask.getOperateType())) {
                        continue;
                    }
                    List<ApproveActRunIdentityLink> links = this.buildActIdentityLink(todoTask);
                    for (ApproveActRunIdentityLink approveActRunIdentityLink : links) {
                        approveActRunIdentityLinkService.save(approveActRunIdentityLink);
                    }
                }
            }
        }
    }

    /**
     * 构建待办人信息
     *
     * @param taskHistoryDetailVo 下一个节点信息
     * @return 待办人信息
     */
    @Override
    public List<ApproveActRunIdentityLink> buildActIdentityLink(TaskHistoryDetailVo taskHistoryDetailVo) {
        String taskHistoryDetailVoId = taskHistoryDetailVo.getId();
        String processInstanceId = taskHistoryDetailVo.getProcessInstanceId();
        List<ApproveActRunIdentityLink> links = new ArrayList<>();
        CandidateVo candidateVo = taskHistoryDetailVo.getCandidateVo();
        this.addActIdentityEntityToList(candidateVo.getCandidateGroups(), links, taskHistoryDetailVoId, processInstanceId, CandidateTypeEnum.GROUP);
        this.addActIdentityEntityToList(candidateVo.getCandidateGroupsOrg(), links, taskHistoryDetailVoId, processInstanceId, CandidateTypeEnum.ORG);
        this.addActIdentityEntityToList(candidateVo.getCandidateUsers(), links, taskHistoryDetailVoId, processInstanceId, CandidateTypeEnum.USER);
        this.addActIdentityEntityToList(candidateVo.getAssignee(), links, taskHistoryDetailVoId, processInstanceId, CandidateTypeEnum.USER);
        return links;
    }

    public void addActIdentityEntityToList(Map<String, String> userInfoMap,
                                           List<ApproveActRunIdentityLink> links,
                                           String taskId, String processInstanceId, CandidateTypeEnum candidateTypeEnum) {
        if (userInfoMap == null) {
            return;
        }
        for (Map.Entry<String, String> stringStringEntry : userInfoMap.entrySet()) {
            ApproveActRunIdentityLink approveActRunIdentityLink = new ApproveActRunIdentityLink();
            approveActRunIdentityLink.setTaskId(taskId);
            approveActRunIdentityLink.setProcessInstanceId(processInstanceId);
            String key = stringStringEntry.getKey();
            String value = stringStringEntry.getValue();
            switch (candidateTypeEnum) {
                case GROUP: {
                    approveActRunIdentityLink.setGroupId(key);
                    approveActRunIdentityLink.setGroupNameList(value);
                    break;
                }
                case ORG: {
                    approveActRunIdentityLink.setDeptId(key);
                    approveActRunIdentityLink.setDeptNameList(value);
                    break;
                }
                case USER: {
                    approveActRunIdentityLink.setUserId(key);
                    approveActRunIdentityLink.setUserNameList(value);
                    break;
                }
                default:
                    break;
            }
            links.add(approveActRunIdentityLink);
        }
    }

    /**
     * 流程开启生成待办，已办
     * 第一个节点提交的时候没有对应的已办事项
     *
     * @param processInstanceVO 实例信息
     * @param approvalInfoDTO   审批信息
     */
    @Override
    public void createTodoAfterProcessStart(ProcessInstanceVO processInstanceVO, ApprovalInfoDTO approvalInfoDTO) {
        Date now = new Date();
        List<TaskHistoryDetailVo> taskRunDetailVos = processInstanceVO.getTaskRunDetailVos();
        for (TaskHistoryDetailVo taskHistoryDetailVo : taskRunDetailVos) {
            // 待办
            ApproveActRunTask approveActRunTask = new ApproveActRunTask();
            approveActRunTask.setTaskId(taskHistoryDetailVo.getId());
            approveActRunTask.setApplyRecordId(approvalInfoDTO.getId());
            approveActRunTask.setBusinessCode(approvalInfoDTO.getBusinessCode());
            approveActRunTask.setBusinessName(approvalInfoDTO.getProcessName());
            approveActRunTask.setDeleted(BooleanEnum.FALSE.getValue());
            approveActRunTask.setGenerateTime(now);
            approveActRunTask.setOperateType(WorkFlowOprationEnum.COMPLETE_AGREE.getValue());
            approveActRunTask.setProcessInstanceId(processInstanceVO.getProcessInstanceId());
            approveActRunTask.setTaskCode(taskHistoryDetailVo.getTaskCode());
            approveActRunTask.setTaskName(taskHistoryDetailVo.getName());
            approveActRunTask.setProcessTitle(approvalInfoDTO.getProcessTitle());
            approveActRunTaskService.save(approveActRunTask);
            // 联系人
            List<ApproveActRunIdentityLink> links = this.buildActIdentityLink(taskHistoryDetailVo);
            for (ApproveActRunIdentityLink approveActRunIdentityLink : links) {
                approveActRunIdentityLinkService.save(approveActRunIdentityLink);
            }
        }
    }

    @Override
    public ResponseStatus checkResource(FlowTypeEnum flowTypeEnum, ResourceInfoReqVO resourceInfoReqVO) {
        String visitResourceCode = resourceInfoReqVO.getVisitResourceCode();
        String visitResourceName = resourceInfoReqVO.getVisitResourceName();
        String removeResourceCode = resourceInfoReqVO.getRemoveVisitResourceCode();
        String removeResourceName = resourceInfoReqVO.getRemoveVisitResourceName();

        if (StrUtil.isBlank(visitResourceCode) && StrUtil.isBlank(removeResourceCode)) {
            return ResponseUtil.dtoFail("访问资源、撤销资源必须有一个不能有空");
        }

        if (StrUtil.isNotBlank(visitResourceCode) && StrUtil.isBlank(visitResourceName)) {
            return ResponseUtil.dtoFail("有访问资源代码时，访问资源名称不能为空");
        }
        if (StrUtil.isNotBlank(removeResourceCode) && StrUtil.isBlank(removeResourceName)) {
            return ResponseUtil.dtoFail("有撤销资源代码时，撤销资源名称不能为空");
        }

        if (flowTypeEnum == FlowTypeEnum.APP_FUN_AUTH) {
            ResponseStatus checkStatus = ResponseStatus.success();
            if (StrUtil.isNotBlank(visitResourceCode)) {
                checkStatus = checkFunction(visitResourceCode, visitResourceName);
            }
            if (ResponseUtil.isFail(checkStatus)) {
                return checkStatus;
            }

            if (StrUtil.isNotBlank(removeResourceCode)) {
                return checkFunction(removeResourceCode, removeResourceName);
            }

        }
        return ResponseStatus.success();
    }

    private ResponseStatus checkFunction(String visitResourceCode, String visitResourceName) {
        ObjectMapper objectMapper = new ObjectMapper();
        List<AppFunctionResourceReqVO> appFunctionResourceReqList;
        try {
            appFunctionResourceReqList = objectMapper.readValue(visitResourceCode, new TypeReference<List<AppFunctionResourceReqVO>>() {
            });
        } catch (JsonProcessingException e) {
            log.error("json", e);
            return ResponseStatus.fail("请传入正确的应用功能资源代码格式");
        }
        if (appFunctionResourceReqList.isEmpty()) {
            return ResponseStatus.fail("请传入应用功能资源代码");
        }

        List<AppFunctionResourceNameReqVO> appFunctionResourceReqNameList;
        try {
            appFunctionResourceReqNameList = objectMapper.readValue(visitResourceName, new TypeReference<List<AppFunctionResourceNameReqVO>>() {
            });
        } catch (JsonProcessingException e) {
            log.error("json", e);
            return ResponseStatus.fail("请传入正确的应用功能资源名称格式");
        }
        if (appFunctionResourceReqNameList.isEmpty()) {
            return ResponseStatus.fail("请传入应用功能资源名称");

        }

        StringBuilder functionTipBuilder = new StringBuilder();
        for (AppFunctionResourceReqVO appFunctionResourceReqVO : appFunctionResourceReqList) {
            String appCode = appFunctionResourceReqVO.getAppCode();
            List<String> functionCode = appFunctionResourceReqVO.getFunctionCode();
            List<AuthAppFunApiVO> functionList = dcucAuthBusiness.getFunctionList(appCode, functionCode);

            Set<String> sourceSet = new HashSet<>(functionCode);

            Set<String> resultSet = functionList.stream().map(AuthAppFunApiVO::getFunCode)
                    .collect(Collectors.toSet());

            sourceSet.removeAll(resultSet);
            if (!sourceSet.isEmpty()) {
                functionTipBuilder.append("appCode:").append(appCode).append(" ")
                        .append("functionCode:");
                for (String source : sourceSet) {
                    functionTipBuilder.append(source);
                }
                functionTipBuilder.append(",");
            }
        }
        String functionTip = functionTipBuilder.toString();
        log.debug("function tip:{}", functionTip);
        if (StrUtil.isNotBlank(functionTip)) {
            functionTip = LangUtil.subLastSymbol(functionTip, ",");
            functionTip = "在权限中心无以下功能" + functionTip;
            return ResponseUtil.dtoFail(functionTip);
        }
        return ResponseStatus.success();
    }

    @Override
    public List<BatchTaskOperateRespVO> batchTaskOperate(List<String> approveNoList, PageTypeEnum pageTypeEnum) {
        List<BatchTaskOperateRespVO> taskOperateRespVOList = new ArrayList<>(approveNoList.size());

        for (String approveNo : approveNoList) {
            ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(approveNo);
            BatchTaskOperateRespVO batchTaskOperateRespVO = new BatchTaskOperateRespVO();
            batchTaskOperateRespVO.setApproveNo(approveNo);
            if (approveRecordInfo != null) {
                String taskId = approveRecordInfo.getTaskId();
                TaskOperateRespVO taskOperateRespVO = workFlowBusiness.getTaskSupportOperate(taskId, pageTypeEnum);
                BeanUtil.copyProperties(taskOperateRespVO, batchTaskOperateRespVO);
            }

            taskOperateRespVOList.add(batchTaskOperateRespVO);
        }
        return taskOperateRespVOList;
    }

    @Override
    public ContentRespDTO generalContentRespVO(FlowTypeEnum flowTypeEnum, ResourceInfoDTO resourceInfoDTO) {
        List<String> visitResourceCodes = new ArrayList<>();
        List<String> visitResourceNames = new ArrayList<>();
        List<String> removeVisitResourceCodes = new ArrayList<>();
        List<String> removeVisitResourceNames = new ArrayList<>();

        List<String> disagreeVisitResourceCodes = new ArrayList<>();
        List<String> disagreeVisitResourceNames = new ArrayList<>();

        String visitResourceName = resourceInfoDTO.getVisitResourceName();
        String visitResourceCode = resourceInfoDTO.getVisitResourceCode();

        String removeVisitResourceCode = resourceInfoDTO.getRemoveVisitResourceCode();
        String removeVisitResourceName = resourceInfoDTO.getRemoveVisitResourceName();

        String disagreeVisitResourceCode = resourceInfoDTO.getDisagreeVisitResourceCode();
        String disagreeVisitResourceName = resourceInfoDTO.getDisagreeVisitResourceName();

        if (flowTypeEnum.equals(FlowTypeEnum.APP_FUN_AUTH)) {
            ObjectMapper objectMapper = new ObjectMapper();

            if (StrUtil.isNotBlank(visitResourceCode)) {
                List<AppFunctionResourceReqVO> appFunctionResourceReqList = Collections.emptyList();
                try {
                    appFunctionResourceReqList = objectMapper.readValue(visitResourceCode, new TypeReference<List<AppFunctionResourceReqVO>>() {
                    });
                } catch (JsonProcessingException e) {
                    log.error("json", e);
                }

                for (AppFunctionResourceReqVO appFunctionResourceReqVO : appFunctionResourceReqList) {
                    String appCode = appFunctionResourceReqVO.getAppCode();
                    List<String> functionCode = appFunctionResourceReqVO.getFunctionCode();
                    for (String function : functionCode) {
                        visitResourceCodes.add(appCode + "/" + function);
                    }
                }
            }

            if (StrUtil.isNotBlank(visitResourceName)) {
                List<AppFunctionResourceNameReqVO> functionResourceNameReqList = Collections.emptyList();
                try {
                    functionResourceNameReqList = objectMapper.readValue(visitResourceName, new TypeReference<List<AppFunctionResourceNameReqVO>>() {
                    });
                } catch (JsonProcessingException e) {
                    log.error("Parse json", e);
                }

                for (AppFunctionResourceNameReqVO appFunctionResourceReqVO : functionResourceNameReqList) {
                    String appName = appFunctionResourceReqVO.getAppName();
                    List<String> functionName = appFunctionResourceReqVO.getFunctionName();
                    for (String function : functionName) {
                        visitResourceNames.add(appName + "/" + function);
                    }
                }
            }

            if (StrUtil.isNotBlank(removeVisitResourceCode)) {
                List<AppFunctionResourceReqVO> removeAppFunctionResourceReqList = Collections.emptyList();
                try {
                    removeAppFunctionResourceReqList = objectMapper.readValue(removeVisitResourceCode, new TypeReference<List<AppFunctionResourceReqVO>>() {
                    });
                } catch (JsonProcessingException e) {
                    log.error("json", e);
                }

                for (AppFunctionResourceReqVO appFunctionResourceReqVO : removeAppFunctionResourceReqList) {
                    String appCode = appFunctionResourceReqVO.getAppCode();
                    List<String> functionCode = appFunctionResourceReqVO.getFunctionCode();
                    for (String function : functionCode) {
                        removeVisitResourceCodes.add(appCode + "/" + function);
                    }
                }
            }

            if (StrUtil.isNotBlank(removeVisitResourceName)) {

                List<AppFunctionResourceNameReqVO> removeFunctionResourceNameReqList = Collections.emptyList();
                try {
                    removeFunctionResourceNameReqList = objectMapper.readValue(removeVisitResourceName, new TypeReference<List<AppFunctionResourceNameReqVO>>() {
                    });
                } catch (JsonProcessingException e) {
                    log.error("Parse json", e);
                }

                for (AppFunctionResourceNameReqVO appFunctionResourceReqVO : removeFunctionResourceNameReqList) {
                    String appName = appFunctionResourceReqVO.getAppName();
                    List<String> functionName = appFunctionResourceReqVO.getFunctionName();
                    for (String function : functionName) {
                        removeVisitResourceNames.add(appName + "/" + function);
                    }
                }
            }

        } else if (flowTypeEnum.equals(FlowTypeEnum.SERVICE_AUTH) || flowTypeEnum.equals(FlowTypeEnum.SERVICE_AUTH_CANCEL)) {
            if (StrUtil.isNotBlank(visitResourceCode)) {
                String[] resourceCode = visitResourceCode.split(StrUtil.COMMA);
                visitResourceCodes.addAll(Arrays.asList(resourceCode));
            }

            if (StrUtil.isNotBlank(visitResourceName)) {
                String[] resourceName = visitResourceName.split(StrUtil.COMMA);
                visitResourceNames.addAll(Arrays.asList(resourceName));
            }

            if (StrUtil.isNotBlank(removeVisitResourceCode)) {
                String[] removeResourceCode = removeVisitResourceCode.split(StrUtil.COMMA);
                removeVisitResourceCodes.addAll(Arrays.asList(removeResourceCode));
            }

            if (StrUtil.isNotBlank(removeVisitResourceName)) {
                String[] removeResourceName = removeVisitResourceName.split(StrUtil.COMMA);
                removeVisitResourceNames.addAll(Arrays.asList(removeResourceName));
            }

            if (StrUtil.isNotBlank(disagreeVisitResourceCode)) {
                String[] strings = disagreeVisitResourceCode.split(StrUtil.COMMA);
                disagreeVisitResourceCodes.addAll(Arrays.asList(strings));
            }

            if (StrUtil.isNotBlank(disagreeVisitResourceName)) {
                String[] strings = disagreeVisitResourceName.split(StrUtil.COMMA);
                disagreeVisitResourceNames.addAll(Arrays.asList(strings));
            }

        }

        @NotBlank String visitorName = resourceInfoDTO.getVisitorName();
        @NotBlank String visitorCode = resourceInfoDTO.getVisitorCode();
        ContentRespDTO contentRespDTO = new ContentRespDTO();
        contentRespDTO.setVisitorNames(Collections.singletonList(visitorName));
        contentRespDTO.setVisitorCodes(Collections.singletonList(visitorCode));
        contentRespDTO.setVisitResourceNames(visitResourceNames);
        contentRespDTO.setVisitResourceCodes(visitResourceCodes);
        contentRespDTO.setRemoveVisitResourceCodes(removeVisitResourceCodes);
        contentRespDTO.setRemoveVisitResourceNames(removeVisitResourceNames);
        contentRespDTO.setDisagreeVisitResourceCodes(disagreeVisitResourceCodes);
        contentRespDTO.setDisagreeVisitResourceNames(disagreeVisitResourceNames);

        return contentRespDTO;
    }

    @Override
    public String getUnOperateUsers(String currentTaskId) {
        // 设置当前任务ID
        StringBuilder unOperator = new StringBuilder();
        if (StringUtils.isNotEmpty(currentTaskId) && !ApproveConstants.END_STR.equalsIgnoreCase(currentTaskId)) {
            String[] tasks = currentTaskId.split(StrUtil.COMMA);
            for (String task : tasks) {
                List<ApproveActRunIdentityLink> taskInfo = approveActRunIdentityLinkService.findByTaskId(task);
                if (CollectionUtils.isEmpty(taskInfo)) {
                    continue;
                }
                for (ApproveActRunIdentityLink item : taskInfo) {
                    String groupNameList = item.getGroupNameList();
                    if (StrUtil.isNotBlank(groupNameList)) {
                        unOperator.append(groupNameList).append(StrUtil.COMMA);
                    }

                    String deptNameList = item.getDeptNameList();
                    if (StrUtil.isNotBlank(deptNameList)) {
                        unOperator.append(deptNameList).append(StrUtil.COMMA);
                    }
                    String userNameList = item.getUserNameList();
                    if (StrUtil.isNotBlank(userNameList)) {
                        unOperator.append(userNameList).append(StrUtil.COMMA);
                    }
                }
            }
        }
        String string = unOperator.toString();
        if (StrUtil.isNotBlank(string)) {
            string = LangUtil.subLastSymbol(string, StrUtil.COMMA);
        }
        return string;
    }

    @Override
    public Boolean isKxPeople(String idCard) {
        // 默认非科信人员
        boolean kxPeople = false;
        String kxList = approveProperties.getKxPeopleList();
        log.info("config kxList is :{}", kxList);
        if (StringUtils.isEmpty(kxList)) {
            return false;
        }
        String[] users = kxList.split(StrUtil.COMMA);
        for (String user : users) {
            // 判断当前身份证号是否为科信人员
            if (idCard.equalsIgnoreCase(user)) {
                kxPeople = true;
                break;
            }
        }
        return kxPeople;
    }

    /**
     * 设置流程信息
     *
     * @param approvalInfoDTO 审批申请内容
     */
    @Override
    public void setProcessInfo(ApprovalInfoDTO approvalInfoDTO) {
        // 处理结束时间
        approvalInfoDTO.setEndTime(approveBaseBusiness.parseEndTime(approvalInfoDTO.getEndTime()));
        // 获取流程类型详情
        WorkflowProcesstypeVo workflowProcesstypeVo;
        try {
            // 调用工作流获取流程类型详情
            workflowProcesstypeVo = workFlowBusiness.queryProcessType(approvalInfoDTO.getProcessType());
        } catch (ApplicationException e) {
            log.info("流程类型标识码-Exception：【{}】", approvalInfoDTO.getProcessType());
            throw new ApplicationException(ErrorCode.WORKFLOW_EXCEPTION.getCode(), ErrorCode.WORKFLOW_EXCEPTION.getMsg());
        }
        if (null == workflowProcesstypeVo) {
            log.info("流程类型标识码：【{}】", approvalInfoDTO.getProcessType());
            throw new ApplicationException(ErrorCode.WORKFLOW_TYPE_NOT_EXIST.getCode(), ErrorCode.WORKFLOW_TYPE_NOT_EXIST.getMsg());
        }
        approvalInfoDTO.setProcessTypeName(workflowProcesstypeVo.getName());
        // 获取流程信息
        @NotBlank String businessCode = approvalInfoDTO.getBusinessCode();
        WorkflowProcessVo workflowProcessVo = workFlowBusiness.queryWorkflowProcessByBusinessCode(businessCode);
        if (null == workflowProcessVo) {
            log.info("流程标识码：【{}】", businessCode);
            throw new ApplicationException(ErrorCode.WORKFLOW_NOT_EXIST.getCode(), ErrorCode.WORKFLOW_NOT_EXIST.getMsg());
        }
        approvalInfoDTO.setProcessName(workflowProcessVo.getName());

        String processTitle = approvalInfoDTO.getProcessTitle();
        // 判断是否有流程标题，否则生成新的标题
        if (StrUtil.isBlank(processTitle)) {
            // 流程标题
            processTitle = approvalInfoDTO.getProcessName() + "-" + approvalInfoDTO.getApplicantName() +
                    "-" + DateUtils.getTimeStr(approvalInfoDTO.getCreateTime(), DateConst.HYPHEN_DISPLAY_DATE);
            approvalInfoDTO.setProcessTitle(processTitle);
        }

    }

    /**
     * 设置我的请求当前结点和未操作者信息
     *
     * @param approvalInfoDTO 审批信息
     */
    @Override
    public ApprovalInfoRespVO fillTaskInfoToApplyInfo(ApprovalInfoDTO approvalInfoDTO) {
        ApprovalInfoRespVO approvalInfoRespVo = new ApprovalInfoRespVO();
        BeanUtil.copyProperties(approvalInfoDTO, approvalInfoRespVo);
        approvalInfoRespVo.setResourceInfoVo(approvalInfoDTO.getResourceInfoVo().toResourceInfoRespVO());

        try {
            String taskId = approvalInfoDTO.getTaskId();
            if (ApproveConstants.END_STR.equalsIgnoreCase(taskId)) {
                return approvalInfoRespVo;
            }
            // 查询当前任务详情
            TaskHistoryDetailVo taskDetailVo = workflowClientFactory.getWorkflowTaskClient().queryTaskDetail(taskId);
            if (taskDetailVo == null) {
                return approvalInfoRespVo;
            }
            approvalInfoRespVo.setTaskId(taskDetailVo.getId());
            // 重新设置候选人信息
            String unOperateUsers = this.getUnOperateUsersFromAssignerMap(taskDetailVo.getCandidateVo());
            approvalInfoRespVo.setUnOperateUsers(unOperateUsers);
        } catch (Exception e) {
            log.error("调用工作流接口报错，报错信息", e);
        }
        return approvalInfoRespVo;
    }

    /**
     * 根据未操作人map获取字符串
     *
     * @param candidateVo 条件
     * @return 字符串
     */
    private String getUnOperateUsersFromAssignerMap(CandidateVo candidateVo) {
        StringBuilder unOperateUsers = new StringBuilder();

        Map<String, String> result = new HashMap<>();
        if (candidateVo.getCandidateUsers() != null) {
            result.putAll(candidateVo.getCandidateUsers());
        }
        // 处理人为机构
        if (candidateVo.getCandidateGroupsOrg() != null) {
            result.putAll(candidateVo.getCandidateGroupsOrg());
        }
        // 处理人为角色
        if (candidateVo.getCandidateGroups() != null) {
            result.putAll(candidateVo.getCandidateGroups());
        }
        // 处理人为具体人员
        if (candidateVo.getAssignee() != null) {
            result.putAll(candidateVo.getAssignee());
        }
        for (Map.Entry<String, String> stringStringEntry : result.entrySet()) {
            unOperateUsers.append(stringStringEntry.getValue()).append(StrUtil.COMMA);
        }

        return StringUtils.isNotEmpty(unOperateUsers.toString())
                ? unOperateUsers.substring(0, unOperateUsers.length() - 1) : unOperateUsers.toString();
    }

    /**
     * 更新当前结点任务
     *
     * @param approvalInfoDTO 审批信息
     */
    @Override
    public void buildTaskInfoByProcessInstanceId(ApprovalInfoDTO approvalInfoDTO, List<TaskHistoryDetailVo> list) {
        StringBuilder taskId = new StringBuilder();
        StringBuilder taskCode = new StringBuilder();
        StringBuilder taskName = new StringBuilder();
        if (CollectionUtils.isNotEmpty(list)) {
            for (TaskHistoryDetailVo taskDetailVo : list) {
                taskId.append(taskDetailVo.getId());
                if (StringUtils.isNotEmpty(taskDetailVo.getId())) {
                    taskId.append(StrUtil.COMMA);
                }
                taskCode.append(taskDetailVo.getTaskCode());
                if (StringUtils.isNotEmpty(taskDetailVo.getTaskCode())) {
                    taskCode.append(StrUtil.COMMA);
                }
                taskName.append(taskDetailVo.getName());
                if (StringUtils.isNotEmpty(taskDetailVo.getName())) {
                    taskName.append(StrUtil.COMMA);
                }
            }
            if (taskCode.length() > 0) {
                taskCode = new StringBuilder(taskCode.substring(0, taskCode.length() - 1));
            }
            if (taskName.length() > 0) {
                taskName = new StringBuilder(taskName.substring(0, taskName.length() - 1));
            }
            if (taskId.length() > 0) {
                taskId = new StringBuilder(taskId.substring(0, taskId.length() - 1));
            }
        } else {
            taskCode = new StringBuilder(ApproveConstants.END_STR);
            taskId = new StringBuilder(ApproveConstants.END_STR);
            taskName = new StringBuilder("结束");
        }
        approvalInfoDTO.setTaskId(taskId.toString());
        approvalInfoDTO.setTaskName(taskName.toString());
        approvalInfoDTO.setApproveTaskIdentifier(taskCode.toString());
        approvalInfoDTO.setUpdateTime(new Date());
        approvalInfoDTO.setUpdateUser(ProofsUtil.getIdcard());
    }

    @Override
    public void setResourceInfoData(ApproveRecordInfo approveRecordInfo, ResourceInfoDTO resourceInfoDTO) {
        Map<String, String> accessResources = new HashMap<>(1);
        accessResources.put(resourceInfoDTO.getVisitorName(), resourceInfoDTO.getVisitResourceName());
        approveRecordInfo.setAccessResources(JSON.toJSONString(accessResources));

        Map<String, String> accessResourcesNo = new HashMap<>(1);
        accessResourcesNo.put(resourceInfoDTO.getVisitorCode(), resourceInfoDTO.getVisitResourceCode());
        approveRecordInfo.setAccessResourcesNo(JSON.toJSONString(accessResourcesNo));

        Map<String, String> removeAccessResourcesNo = new HashMap<>(1);
        removeAccessResourcesNo.put(resourceInfoDTO.getVisitorCode(), resourceInfoDTO.getRemoveVisitResourceCode());
        approveRecordInfo.setRemoveAccessResourcesNo(JSON.toJSONString(removeAccessResourcesNo));

        Map<String, String> removeAccessResources = new HashMap<>(1);
        removeAccessResources.put(resourceInfoDTO.getVisitorName(), resourceInfoDTO.getRemoveVisitResourceName());
        approveRecordInfo.setRemoveAccessResources(JSON.toJSONString(removeAccessResources));

        Map<String, String> disagreeAccessResources = new HashMap<>(1);
        disagreeAccessResources.put(resourceInfoDTO.getVisitorName(), resourceInfoDTO.getDisagreeVisitResourceName());
        approveRecordInfo.setDisagreeAccessResources(JSON.toJSONString(disagreeAccessResources));

        Map<String, String> disagreeAccessResourcesNo = new HashMap<>(1);
        disagreeAccessResourcesNo.put(resourceInfoDTO.getVisitorCode(), resourceInfoDTO.getDisagreeVisitResourceCode());
        approveRecordInfo.setDisagreeAccessResourcesNo(JSON.toJSONString(disagreeAccessResourcesNo));

    }
}
