package com.dragonsoft.dcuc.approve.business.impl;

import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dragoninfo.dcuc.common.Constants;
import com.dragoninfo.dcuc.common.enums.UserExtInfoEnum;
import com.dragoninfo.dcuc.common.utils.LangUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.duceap.enums.ManageDisposeTypeEnum;
import com.dragoninfo.dcuc.duceap.enums.OperateDisposeTypeEnum;
import com.dragoninfo.dcuc.duceap.enums.RequestDisposeTypeEnum;
import com.dragoninfo.dcuc.duceap.enums.SchemeTypeEnum;
import com.dragonsoft.dcuc.approve.business.IApproveLogBusiness;
import com.dragonsoft.dcuc.approve.business.IRiskBusiness;
import com.dragonsoft.dcuc.approve.business.external.IDcucBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.CommitTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.LogReceiverEnum;
import com.dragonsoft.dcuc.approve.enumresources.LogTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.QmApproveTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.QmApproveTypeEnumV2;
import com.dragonsoft.dcuc.approve.event.ApproveEventPublish;
import com.dragonsoft.dcuc.approve.model.FlowManageLog;
import com.dragonsoft.dcuc.approve.model.OperateLog;
import com.dragonsoft.dcuc.approve.model.RequestLog;
import com.dragonsoft.dcuc.approve.model.RiskLog;
import com.dragonsoft.dcuc.approve.model.RiskScheme;
import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.ApproveLogDTO;
import com.dragonsoft.dcuc.approve.model.dto.LogDto;
import com.dragonsoft.dcuc.approve.model.dto.LogInfoMetaDTO;
import com.dragonsoft.dcuc.approve.model.dto.ProcessInstanceInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.QmOperateLogDTO;
import com.dragonsoft.dcuc.approve.model.resp.OperateRespVO;
import com.dragonsoft.dcuc.approve.model.vo.dcuc.OauthUserVO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.service.IFlowManageLogService;
import com.dragonsoft.dcuc.approve.service.IOperateLogService;
import com.dragonsoft.dcuc.approve.service.IRequestLogService;
import com.dragonsoft.dcuc.approve.service.IRiskLogService;
import com.dragonsoft.dcuc.approve.service.IRiskSchemeService;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.commons.util.ip.IpUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.duceap.duwf.api.model.OutgoingVo;
import com.dragonsoft.duceap.duwf.api.model.ProcessInstanceVO;
import com.dragonsoft.duceap.duwf.api.model.TaskDefinitionVo;
import com.dragonsoft.duceap.duwf.api.model.TaskHistoryDetailVo;
import com.dragonsoft.duceap.workflow.client.WorkflowClientFactory;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/5/27 19:43
 */
@Service
public class ApproveLogBusinessImpl implements IApproveLogBusiness {

    private static final Logger logger = LoggerFactory.getLogger(ApproveLogBusinessImpl.class);

    private IFlowManageLogService flowManageLogService;
    private IRequestLogService requestLogService;
    private IOperateLogService operateLogService;

    private WorkflowClientFactory workflowClientFactory;

    private ApproveProperties approveProperties;

    private IRiskLogService riskLogService;

    private ApproveEventPublish approveEventPublish;

    private IRiskSchemeService riskSchemeService;

    private IDcucBusiness dcucBusiness;

    private IRiskBusiness riskBusiness;

    @Autowired
    public void setRiskBusiness(IRiskBusiness riskBusiness) {
        this.riskBusiness = riskBusiness;
    }

    @Autowired
    public void setFlowManageLogService(IFlowManageLogService flowManageLogService) {
        this.flowManageLogService = flowManageLogService;
    }

    @Autowired
    public void setRequestLogService(IRequestLogService requestLogService) {
        this.requestLogService = requestLogService;
    }

    @Autowired
    public void setOperateLogService(IOperateLogService operateLogService) {
        this.operateLogService = operateLogService;
    }

    @Autowired
    public void setWorkflowClientFactory(WorkflowClientFactory workflowClientFactory) {
        this.workflowClientFactory = workflowClientFactory;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Autowired
    public void setRiskLogService(IRiskLogService riskLogService) {
        this.riskLogService = riskLogService;
    }


    @Autowired
    public void setApproveEventPublish(ApproveEventPublish approveEventPublish) {
        this.approveEventPublish = approveEventPublish;
    }

    @Autowired
    public void setRiskSchemeService(IRiskSchemeService riskSchemeService) {
        this.riskSchemeService = riskSchemeService;
    }

    @Autowired
    public void setDcucBusiness(IDcucBusiness dcucBusiness) {
        this.dcucBusiness = dcucBusiness;
    }


    /**
     * 操作日志/请求日志构建
     *
     * @param logDto                日志Dto
     * @param workFlowOperationEnum 操作类型
     */
    @Override
    public void buildOperateLog(LogDto logDto, ApproveWorkFlowOperationEnum workFlowOperationEnum, Boolean flowFinish) {

        // TODO add wanghao   20241105 修改
        String flowNodeCode = "" ;
        String flowNodeName = "" ;
        String taskKey = "" ;
        String flowKey = "" ;
        String processInstanceId = "" ;


        if(logDto.getProcessInstanceVO() != null){
            flowNodeCode = logDto.getProcessInstanceVO().getTaskHistoryDetailVo() == null ? "" : logDto.getProcessInstanceVO().getTaskHistoryDetailVo().getTaskCode();
            flowNodeName = logDto.getProcessInstanceVO().getTaskHistoryDetailVo() == null ? "" : logDto.getProcessInstanceVO().getTaskHistoryDetailVo().getName();
            taskKey = logDto.getProcessInstanceVO().getTaskHistoryDetailVo() == null ? "" : logDto.getProcessInstanceVO().getTaskHistoryDetailVo().getTaskDefinitionKey();
            flowKey = logDto.getProcessInstanceVO().getPocessDefinitionId() == null ? "" : logDto.getProcessInstanceVO().getPocessDefinitionId();
            processInstanceId = logDto.getProcessInstanceVO().getProcessInstanceId() == null ? "" : logDto.getProcessInstanceVO().getProcessInstanceId();
        }


        /*  TODO add wanghao     原内容

        String flowNodeCode = logDto.getProcessInstanceVO() == null ? "" : logDto.getProcessInstanceVO().getTaskHistoryDetailVo().getTaskCode();
        String flowNodeName = logDto.getProcessInstanceVO() == null ? "" : logDto.getProcessInstanceVO().getTaskHistoryDetailVo().getName();
        String taskKey = logDto.getProcessInstanceVO() == null ? "" : logDto.getProcessInstanceVO().getTaskHistoryDetailVo().getTaskDefinitionKey();
        String flowKey = logDto.getProcessInstanceVO() == null ? "" : logDto.getProcessInstanceVO().getPocessDefinitionId();
        String processInstanceId = logDto.getProcessInstanceVO() == null ? "" : logDto.getProcessInstanceVO().getProcessInstanceId();
        */

        ProcessInstanceVO processInstanceVO = logDto.getProcessInstanceVO();
        ProcessInstanceInfoDTO processInstanceInfoDTO = ProcessInstanceInfoDTO.parseProcessInstanceVO(processInstanceVO);

        //根据类型判断是什么日志
        switch (workFlowOperationEnum) {
            case COMPLETE_DISAGREE:
            case BACK:
                //审批操作日志
                buildOperateLogAndSend(logDto, OperateDisposeTypeEnum.COMPLETE_DISAGREE, processInstanceInfoDTO, flowFinish);
                break;
            case COMPLETE_AGREE:
                //审批操作日志
                buildOperateLogAndSend(logDto, OperateDisposeTypeEnum.COMPLETE_AGREE, processInstanceInfoDTO, flowFinish);
                break;

            //TODO add wanghao 新增对接bu日志处理    都是作为   审批请求日志
            case SUSPEND:
            case ACTIVATE:
            case REDO:
                buildRequestLog(workFlowOperationEnum, logDto, flowNodeCode, flowNodeName, taskKey, flowKey, processInstanceId, "22");
                break;
            //TODO add wanghao 新增对接bu日志处理   end

            case RECALL:
                //审批请求日志
                buildRequestLog(RequestDisposeTypeEnum.RECALL, logDto, flowNodeCode, flowNodeName, taskKey, flowKey, processInstanceId, "22");
                break;
            default:
                break;
        }
    }

    /**
     * 构建审批请求日志对象
     *
     * @param disposeType  处理类型
     * @param logDto       日志Dto
     * @param flowNodeCode 流程节点代码
     * @param flowNodeName 流程节点名称
     * @param taskKey      任务key
     * @param flowKey      流程key
     */
    private void buildRequestLog(RequestDisposeTypeEnum disposeTypeEnum, LogDto logDto, String flowNodeCode, String flowNodeName,
                                 String taskKey, String flowKey, String processInstanceId, String businessCode) {

        ApprovalInfoDTO approvalInfoDTO = logDto.getApprovalInfoDTO();
        RequestLog requestLog = new RequestLog();
        requestLog.setIdcard(approvalInfoDTO.getApplicantIdcard());
        requestLog.setOrgName(approvalInfoDTO.getApplicantOrgName());
        requestLog.setOrgCode(approvalInfoDTO.getApplicantOrgCode());
        requestLog.setBusinessCode(approvalInfoDTO.getBusinessCode());
        requestLog.setTitle(approvalInfoDTO.getProcessTitle());
        requestLog.setDisposeType(disposeTypeEnum.getValue());
        requestLog.setResult(logDto.getResult());
        requestLog.setFailMsg(logDto.getFailMsg());
        requestLog.setCreateIp(logDto.getCreateIp());
        //操作内容策划不知道要存什么，暂时放空
//        requestLog.setOperateContent(JSON.toJSONString(logDto.getApprovalInfo()));
        requestLog.setCreateUser(logDto.getCreateUser());
        requestLog.setCreateUserName(logDto.getCreateUserName());
        Date date = new Date();
        requestLog.setCreateTime(date);
        requestLog.setFlowNodeCode(flowNodeCode);
        requestLog.setFlowNodeName(flowNodeName);
        requestLog.setApproveRecordId(approvalInfoDTO.getId());
        requestLog.setProcessInstanceId(processInstanceId);
        requestLog.setApproveNo(approvalInfoDTO.getApproveNo());
        //发送审批请求日志到消息中心
        logger.info("=====发送审批申请请求日志-发起消息,消息体为：{}====", JSON.toJSONString(requestLog));
        LogInfoMetaDTO logInfoMetaDTO = new LogInfoMetaDTO();
        logInfoMetaDTO.setLogType(LogTypeEnum.REQUEST_LOG.getValue());
        logInfoMetaDTO.setTaskKey(taskKey);
        logInfoMetaDTO.setFlowKey(flowKey);
        logInfoMetaDTO.setProcessInstanceId(processInstanceId);
        logInfoMetaDTO.setTerminalIp(logDto.getCreateIp());
        // 请求发起
        logInfoMetaDTO.setBusinessType(businessCode);

        
     // 2024-8-21 对接BU新规范
        logInfoMetaDTO.setTaskId(approvalInfoDTO.getTaskId());
        logInfoMetaDTO.setTaskName(approvalInfoDTO.getTaskName());
        logInfoMetaDTO.setAppName(approvalInfoDTO.getAppName());
        logInfoMetaDTO.setOperatorCertNo(approvalInfoDTO.getApplicantIdcard());
        logInfoMetaDTO.setOperatorName(approvalInfoDTO.getApplicantName());
        logInfoMetaDTO.setOperatorOrgCode(approvalInfoDTO.getApplicantOrgCode());
        logInfoMetaDTO.setOperatorOrgName(approvalInfoDTO.getApplicantOrgName());
        logInfoMetaDTO.setOperateTime(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss"));
        
        if (RequestDisposeTypeEnum.START.equals(disposeTypeEnum)) {
        	logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.START.getValue());
        }else if(RequestDisposeTypeEnum.RECALL.equals(disposeTypeEnum)) {
        	logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.RECALL.getValue());
        }


        System.err.println("========原有recall 撤回--bu日志requestLog Content:"+JSON.toJSONString(requestLog));
        System.err.println("========原有recall 撤回--bu日志logInfoMetaDTO :"+JSON.toJSONString(logInfoMetaDTO));


        sendInfo(JSON.toJSONString(requestLog), "审批请求日志", logInfoMetaDTO);
    }

    /**
     * 构建审批操作日志对象并发送
     *
     * @param dto                    日志Dto
     * @param operateDisposeTypeEnum 类型
     * @param processInstanceInfoDTO 流程实例信息
     */
    private void buildOperateLogAndSend(LogDto dto, OperateDisposeTypeEnum operateDisposeTypeEnum,
                                        ProcessInstanceInfoDTO processInstanceInfoDTO, Boolean flowFinish) {
        ApprovalInfoDTO approvalInfoDTO = dto.getApprovalInfoDTO();

        String failMsg = dto.getFailMsg();
        if (failMsg.length() > 1000) {
            failMsg = failMsg.substring(0, 1000);
        }

        OperateLog operateLog = new OperateLog();
        operateLog.setIdcard(approvalInfoDTO.getApplicantIdcard());
        operateLog.setOrgName(approvalInfoDTO.getApplicantOrgName());
        operateLog.setOrgCode(approvalInfoDTO.getApplicantOrgCode());
        operateLog.setBusinessCode(approvalInfoDTO.getBusinessCode());
        operateLog.setTitle(approvalInfoDTO.getProcessTitle());
        operateLog.setDisposeType(operateDisposeTypeEnum.value);
        operateLog.setResult(dto.getResult());
        operateLog.setFailMsg(failMsg);
        operateLog.setCreateIp(dto.getCreateIp());
        //操作内容策划不知道要存什么，暂时放空
//        operateLog.setOperateContent(JSON.toJSONString(dto.getApprovalInfo()));
        operateLog.setCreateUser(dto.getCreateUser());
        operateLog.setCreateUserName(dto.getCreateUserName());
        operateLog.setCreateTime(dto.getCreateTime());
        operateLog.setFlowNodeCode(processInstanceInfoDTO.getFlowNodeCode());
        operateLog.setFlowNodeName(processInstanceInfoDTO.getFlowNodeName());
        operateLog.setApproveRecordId(approvalInfoDTO.getId());
        operateLog.setProcessInstanceId(processInstanceInfoDTO.getProcessInstanceId());
        operateLog.setApproveNo(approvalInfoDTO.getApproveNo());

        LogInfoMetaDTO logInfoMetaDTO = new LogInfoMetaDTO();
        logInfoMetaDTO.setLogType(LogTypeEnum.OPERATE_LOG.getValue());
        logInfoMetaDTO.setTaskKey(processInstanceInfoDTO.getTaskKey());
        logInfoMetaDTO.setFlowKey(processInstanceInfoDTO.getFlowKey());
        logInfoMetaDTO.setProcessInstanceId(processInstanceInfoDTO.getProcessInstanceId());
        logInfoMetaDTO.setTerminalIp(dto.getCreateIp());
        logInfoMetaDTO.setFlowFinish(flowFinish);
        logInfoMetaDTO.setBusinessType(operateDisposeTypeEnum.businessTypeCode);
        logInfoMetaDTO.setNeedSendRedListRisk(dto.getNeedSendRedListRisk());
        logInfoMetaDTO.setRedListOperateRespVO(JsonUtils.toJSONString(dto.getRedListOperateRespVO()));
        // 2024-8-21 对接BU新规范
        logInfoMetaDTO.setTaskId(approvalInfoDTO.getTaskId());
        logInfoMetaDTO.setTaskName(approvalInfoDTO.getTaskName());
        logInfoMetaDTO.setAppName(approvalInfoDTO.getAppName());
        logInfoMetaDTO.setOperatorCertNo(approvalInfoDTO.getApplicantIdcard());
        logInfoMetaDTO.setOperatorName(approvalInfoDTO.getApplicantName());
        logInfoMetaDTO.setOperatorOrgCode(approvalInfoDTO.getApplicantOrgCode());
        logInfoMetaDTO.setOperatorOrgName(approvalInfoDTO.getApplicantOrgName());
        logInfoMetaDTO.setOperateTime(DateUtil.format(dto.getCreateTime(), "yyyy-MM-dd HH:mm:ss"));
        
        if (OperateDisposeTypeEnum.COMPLETE_DISAGREE.equals(operateDisposeTypeEnum)) {
        	logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.REBACK.getValue());
        }else if(OperateDisposeTypeEnum.COMPLETE_AGREE.equals(operateDisposeTypeEnum)) {
        	logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.APPROVE.getValue());
        }
		
        
        //发送审批操作日志消息
        logger.info("=====发送审批操作日志消息,消息体为：{}====", JSON.toJSONString(operateLog));
        sendInfo(JSON.toJSONString(operateLog), "审批操作日志", logInfoMetaDTO);
    }

    /**
     * 构建审批流程管理日志-删除
     *
     * @param logDto 日志Dto
     */
    @Override
    public void buildFlowManageLogDelete(LogDto logDto) {
        //构建审批流程管理日志对象
        buildFlowManageLogAndSend(logDto, ManageDisposeTypeEnum.DELETED);
    }

    @Override
    public void saveLog(ApproveLogDTO approveLogDTO) {
        String content = approveLogDTO.getContent();
        LogInfoMetaDTO logInfoMetaDTO = approveLogDTO.getLogInfoMetaDTO();
        String logType = logInfoMetaDTO.getLogType();
        switch (EnumUtils.enumOf(LogTypeEnum.class, logType)) {
            case MANAGE_LOG:
                saveManageLog(content, logInfoMetaDTO);
                break;
            case REQUEST_LOG:
                saveRequestLog(content, logInfoMetaDTO);
                break;
            case OPERATE_LOG:
                saveOperateLog(content, logInfoMetaDTO);
                break;
            default:
                break;
        }

    }

    /**
     * 构建风险日志
     *
     * @param logTypeEnum       日志类型
     * @param applyIdcard       申请人身份证号
     * @param operateIdcard     操作人身份证号
     * @param approveRecordId   表单id
     * @param businessCode      流程标识码
     * @param riskIp            风险ip
     * @param title             标题
     * @param createUserName    操作者姓名
     * @param orgName           操作者机构名称
     * @param orgCode           操作者机构code
     * @param createTime        创建时间
     * @param processInstanceId 流程实例ID
     */
    public void buildRiskLog(LogTypeEnum logTypeEnum, String applyIdcard,
                             String operateIdcard, String approveRecordId,
                             String businessCode, String riskIp,
                             String title, String createUserName,
                             String orgName, String orgCode,
                             Date createTime, String processInstanceId, String processNo,
                             String operateLogId, LogInfoMetaDTO logInfoMetaDTO) {

        String taskKey = logInfoMetaDTO.getTaskKey();
        String flowKey = logInfoMetaDTO.getFlowKey();
        try {
            //开始节点直接返回
            if (taskKey.equalsIgnoreCase(ApproveConstants.START)) {
                return;
            }

            if (StrUtil.isBlank(flowKey)) {
                throw new IllegalArgumentException("flowKey can't be blank.");
            }

            List<OperateLog> listByApproveRecordId = operateLogService.getListByApproveRecordId(approveRecordId);
            List<String> logIdList = listByApproveRecordId.stream().map(OperateLog::getId)
                    .collect(Collectors.toList());
            logIdList.add(operateLogId);
            //流程出口信息
            OutgoingVo outgoingVo = workflowClientFactory.getWorkflowTaskClient().queryStartOutgoing(flowKey);
            if (outgoingVo != null) {
                boolean needSaveRiskLog = true;
                //跳过第一个节点
                for (TaskDefinitionVo taskDefinitionVo : outgoingVo.getTaskDefinitionVoList()) {
                    if (taskKey.equals(taskDefinitionVo.getTaskDefinitionKey())) {
                        needSaveRiskLog = false;
                        break;
                    }
                }

                if (needSaveRiskLog) {
                    if (logTypeEnum.equals(LogTypeEnum.OPERATE_LOG)) {
                        // 提交人、审批人重复风险判断，跳过第一个节点,只有操作日志才分析提交、审批人风险
                        ResponseStatus commitApproveResponseStatus = riskBusiness.commitApproveRepetitionRisk(applyIdcard, operateIdcard, createUserName);
                        if (ResponseUtil.isFail(commitApproveResponseStatus)) {
                            String commitRiskContent = commitApproveResponseStatus.getMessage();

                            saveRiskLog(businessCode, riskIp, commitRiskContent,
                                    title, operateIdcard, createUserName,
                                    applyIdcard, orgName, orgCode,
                                    createTime, SchemeTypeEnum.COMMIT_APPROVE_REPETITION_RISK.value,
                                    approveRecordId, processInstanceId, processNo, Collections.singletonList(operateLogId));
                        }
                    }

                    ResponseStatus singleApproveRiskStatus = riskBusiness.singleApproveRisk(approveRecordId, operateIdcard, createUserName);
                    if (ResponseUtil.isFail(singleApproveRiskStatus)) {
                        String singleApproveRiskContent = singleApproveRiskStatus.getMessage();
                        //产生《单人连续审批风险》
                        saveRiskLog(businessCode, riskIp, singleApproveRiskContent,
                                title, operateIdcard, createUserName,
                                applyIdcard, orgName, orgCode,
                                createTime, SchemeTypeEnum.SINGLE_APPROVE_RISK.value,
                                approveRecordId, processInstanceId, processNo, logIdList);
                    }
                }
            }

            Boolean needSendRedListRisk = logInfoMetaDTO.getNeedSendRedListRisk();
            if (ObjectUtil.isNull(needSendRedListRisk)) {
                needSendRedListRisk = false;
            }
            String redListOperateRespString = logInfoMetaDTO.getRedListOperateRespVO();
            if (StrUtil.isBlank(redListOperateRespString)) {
                redListOperateRespString = "";
            }
            if (needSendRedListRisk && (StrUtil.isNotBlank(redListOperateRespString))) {
                OperateRespVO redListOperateRespVO = JsonUtils.parseObject(redListOperateRespString, OperateRespVO.class);
                RiskScheme riskScheme = riskSchemeService.selectEnableSchemeByType(SchemeTypeEnum.RED_WHITE_RISK);
                if (riskScheme != null) {

                    saveRiskLog(businessCode, riskIp, redListOperateRespVO.getRedListTip(),
                            title, operateIdcard, createUserName,
                            applyIdcard, orgName, orgCode,
                            createTime, riskScheme.getSchemeType(),
                            approveRecordId, processInstanceId, processNo, logIdList);
                }
            }

        } catch (Exception e) {
            logger.error("====================构建风险日志异常==============", e);
            throw new ApplicationException(e.getMessage());
        }
    }


    /**
     * 保存风险日志
     *
     * @param businessCode    流程标识码
     * @param riskIp          风险ip
     * @param riskContent     风险内容
     * @param title           标题
     * @param createUser      操作者身份证
     * @param createUserName  操作姓名
     * @param idcard          申请者身份证
     * @param orgName         机构名称
     * @param orgCode         机构code
     * @param createTime      创建时间
     * @param riskType        风险类型
     * @param approveRecordId 申请单id
     */
    public void saveRiskLog(String businessCode, String riskIp, String riskContent,
                            String title, String createUser, String createUserName,
                            String idcard, String orgName, String orgCode,
                            Date createTime, String riskType, String approveRecordId,
                            String processInstanceId, String approveNo, List<String> logIdList) {
        StringBuilder stringBuilder = new StringBuilder();
        Set<String> operateLogIdSet = new HashSet<>(logIdList);
        String logId = "";
        if (!operateLogIdSet.isEmpty()) {
            for (String item : operateLogIdSet) {
                stringBuilder.append(item).append(",");
            }
            logId = LangUtil.subLastSymbol(stringBuilder.toString(), ",");
        }

        RiskLog riskLog = new RiskLog();
        riskLog.setLogSender(approveProperties.getAppCode());
        riskLog.setLogReceiver(LogReceiverEnum.AUDIT_SERVICE.getValue() + "," + LogReceiverEnum.SECURITY_POLICY_SERVICE.getValue());
        riskLog.setRiskType(riskType);
        riskLog.setRiskTime(createTime);
        riskLog.setBusinessCode(businessCode);
        riskLog.setTitle(title);
        riskLog.setApproveRecordId(approveRecordId);
        riskLog.setRiskIp(riskIp);
        riskLog.setRiskContent(riskContent);
        riskLog.setCreateUser(createUser);
        riskLog.setCreateUserName(createUserName);
        riskLog.setIdcard(idcard);
        riskLog.setOrgName(orgName);
        riskLog.setOrgCode(orgCode);
        riskLog.setCreateTime(createTime);
        riskLog.setProcessInstanceId(processInstanceId);
        riskLog.setApproveNo(approveNo);
        riskLog.setLogId(logId);

        approveEventPublish.publishRiskLogEvent(riskLog);

        riskLogService.saveRiskLog(riskLog);
    }

    /**
     * 操作日志保存
     *
     * @param content        内容
     * @param logInfoMetaDTO 日志信息
     */
    private void saveOperateLog(String content, LogInfoMetaDTO logInfoMetaDTO) {
        String appToken = logInfoMetaDTO.getAppToken();
        String processInstanceId = logInfoMetaDTO.getProcessInstanceId();
        String businessType = logInfoMetaDTO.getBusinessType();
        Boolean flowFinish = logInfoMetaDTO.getFlowFinish();
        OperateLog entity = JSONObject.parseObject(content, OperateLog.class);
        entity.setProcessInstanceId(processInstanceId);
        operateLogService.saveOperateLog(entity);

        String approveRecordId = entity.getApproveRecordId();
        Date createTime = entity.getCreateTime();
        String logId = entity.getId();
        String createUser = entity.getCreateUser();
        String createUserName = entity.getCreateUserName();
        String disposeType = entity.getDisposeType();
        String approveNo = entity.getApproveNo();
        String id = entity.getId();

        OperateDisposeTypeEnum operateDisposeTypeEnum = EnumUtils.enumOf(OperateDisposeTypeEnum.class, disposeType);

        RequestLog requestLog = requestLogService.selectOneByApproveId(approveRecordId);
        String requestIdcard = "";
        String requestUsername = "";
        String requestApproveIp = "";
        if (requestLog != null) {
            requestIdcard = requestLog.getIdcard();
            requestUsername = requestLog.getCreateUserName();
            requestApproveIp = requestLog.getCreateIp();
        }

        buildRiskLog(LogTypeEnum.OPERATE_LOG, entity.getIdcard(),
                entity.getCreateUser(), entity.getApproveRecordId(),
                entity.getBusinessCode(), entity.getCreateIp(),
                entity.getTitle(), entity.getCreateUserName(),
                entity.getOrgName(), entity.getOrgCode(),
                entity.getCreateTime(), processInstanceId, approveNo, id, logInfoMetaDTO);

        QmOperateLogDTO qmOperateLogDTO = QmOperateLogDTO.builder()
                .appToken(appToken)
                .qmApproveTypeEnum(QmApproveTypeEnum.OPERATE)
                .approveId(approveRecordId)
                .approveNo(approveNo)
                .approveLogId(logId)
                .approveIdcard(createUser)
                .approveUserName(createUserName)
                .businessType(businessType)
                .approveTime(createTime)
                .operateDisposeTypeEnum(operateDisposeTypeEnum)
                .requestIdcard(requestIdcard)
                .requestUsername(requestUsername)
                .requestApproveIp(requestApproveIp)
                .flowFinish(flowFinish).logInfoMetaDTO(logInfoMetaDTO)
                .build();

        approveEventPublish.publishOperateEvent(qmOperateLogDTO);
    }

    /**
     * 请求日志保存
     *
     * @param content        内容
     * @param logInfoMetaDTO 日志信息
     */
    private void saveRequestLog(String content, LogInfoMetaDTO logInfoMetaDTO) {
        String processInstanceId = logInfoMetaDTO.getProcessInstanceId();
        String businessType = logInfoMetaDTO.getBusinessType();

        RequestLog entity = JSONObject.parseObject(content, RequestLog.class);
        entity.setProcessInstanceId(processInstanceId);
        String approveNo = entity.getApproveNo();
        requestLogService.saveRequestLog(entity);
        String id = entity.getId();

        buildRiskLog(LogTypeEnum.REQUEST_LOG, entity.getIdcard(),
                entity.getCreateUser(), entity.getApproveRecordId(),
                entity.getBusinessCode(), entity.getCreateIp(),
                entity.getTitle(), entity.getCreateUserName(),
                entity.getOrgName(), entity.getOrgCode(),
                entity.getCreateTime(), processInstanceId, approveNo, id, logInfoMetaDTO);

        String approveRecordId = entity.getApproveRecordId();
        String applicantIdcard = entity.getIdcard();
        Date createTime = entity.getCreateTime();
        String logId = entity.getId();
        String createUserName = entity.getCreateUserName();
        String createIp = entity.getCreateIp();

        QmOperateLogDTO qmOperateLogDTO = QmOperateLogDTO.builder()
                .qmApproveTypeEnum(QmApproveTypeEnum.REQUEST)
                .approveId(approveRecordId)
                .approveNo(approveNo)
                .approveLogId(logId)
                .requestIdcard(applicantIdcard)
                .requestUsername(createUserName)
                .requestApproveIp(createIp)
                .approveIdcard(applicantIdcard)
                .businessType(businessType)
                .approveTime(createTime).logInfoMetaDTO(logInfoMetaDTO)
                .build();

        approveEventPublish.publishOperateEvent(qmOperateLogDTO);
    }

    /**
     * 流程管理日志保存
     *
     * @param content        内容
     * @param logInfoMetaDTO 日志信息
     */
    private void saveManageLog(String content, LogInfoMetaDTO logInfoMetaDTO) {
        String processInstanceId = logInfoMetaDTO.getProcessInstanceId();
        String businessType = logInfoMetaDTO.getBusinessType();
        FlowManageLog entity = JSONObject.parseObject(content, FlowManageLog.class);
        entity.setProcessInstanceId(processInstanceId);
        flowManageLogService.saveFlowManageLog(entity);

        String disposeType = entity.getDisposeType();
        ManageDisposeTypeEnum manageDisposeTypeEnum = EnumUtils.enumOf(ManageDisposeTypeEnum.class, disposeType);

        String approveRecordId = entity.getApproveRecordId();
        Date createTime = entity.getCreateTime();
        String applicantIdcard = entity.getIdcard();
        String logId = entity.getId();
        String approveNo = entity.getApproveNo();

        String requestIdcard = "";
        String requestUsername = "";
        String requestApproveIp = "";

        if (manageDisposeTypeEnum.equals(ManageDisposeTypeEnum.INSERT)) {
            requestIdcard = entity.getIdcard();
            requestUsername = entity.getCreateUserName();
            requestApproveIp = entity.getCreateIp();
        } else {
            RequestLog requestLog = requestLogService.selectOneByApproveId(approveRecordId);
            if (requestLog != null) {
                requestIdcard = requestLog.getIdcard();
                requestUsername = requestLog.getCreateUserName();
                requestApproveIp = requestLog.getCreateIp();
            }
        }

        QmOperateLogDTO qmOperateLogDTO = QmOperateLogDTO.builder()
                .qmApproveTypeEnum(QmApproveTypeEnum.FLOW)
                .approveId(approveRecordId)
                .approveNo(approveNo)
                .approveLogId(logId)
                .requestIdcard(applicantIdcard)
                .businessType(businessType)
                .approveTime(createTime)
                .requestIdcard(requestIdcard)
                .requestUsername(requestUsername)
                .requestApproveIp(requestApproveIp).logInfoMetaDTO(logInfoMetaDTO)
                .build();


        approveEventPublish.publishOperateEvent(qmOperateLogDTO);
    }


    /**
     * 构建审批流程管理日志-新建/修改
     *
     * @param logDto         日志Dto
     * @param commitTypeEnum 类型（保存/保存并提交）
     */
    @Override
    public void buildFlowManageLogInsert(LogDto logDto, CommitTypeEnum commitTypeEnum, ManageDisposeTypeEnum manageDisposeTypeEnum, Boolean flowFinish) {
        buildFlowManageLogAndSend(logDto, manageDisposeTypeEnum);
        if (CommitTypeEnum.APPLY_COMMIT.equals(commitTypeEnum)) {
            ProcessInstanceVO processInstanceVO = logDto.getProcessInstanceVO();
            ProcessInstanceInfoDTO processInstanceInfoDTO = ProcessInstanceInfoDTO.parseProcessInstanceVO(processInstanceVO);
            String flowKey = processInstanceInfoDTO.getFlowKey();
            String processInstanceId = processInstanceInfoDTO.getProcessInstanceId();
            //如果是保存并提交，则需要构建（申请请求日志-发起）对象
            buildRequestLog(RequestDisposeTypeEnum.START, logDto, "start", "开始", "start", flowKey, processInstanceId, "21");
        }
    }

    /**
     * 构建审批流程管理日志-修改
     *
     * @param logDto             日志Dto
     * @param needSaveOperateLog 是否保存操作日志
     */
    @Override
    public void buildFlowManageLogUpdate(LogDto logDto, Boolean needSaveOperateLog, Boolean flowFinish) {
        buildFlowManageLogAndSend(logDto, ManageDisposeTypeEnum.UPDATE);
        if (needSaveOperateLog) {
            ProcessInstanceVO processInstanceVO = logDto.getProcessInstanceVO();
            ProcessInstanceInfoDTO processInstanceInfoDTO = ProcessInstanceInfoDTO.parseProcessInstanceVO(processInstanceVO);

            //实例id不为空，说明为流程的的修改并提交，所以需要构建审批操作日志（通过）
            buildOperateLogAndSend(logDto, OperateDisposeTypeEnum.COMPLETE_AGREE, processInstanceInfoDTO, flowFinish);
        }
    }


    /**
     * 构建审批流程管理日志
     *
     * @param logDto                日志Dto
     * @param manageDisposeTypeEnum 审批流程管理日志类型（DisposeTypeEnum）
     */
    private void buildFlowManageLogAndSend(LogDto logDto, ManageDisposeTypeEnum manageDisposeTypeEnum) {

        String processInstanceId = logDto.getProcessInstanceVO() == null ? "" : logDto.getProcessInstanceVO().getProcessInstanceId();

        ProcessInstanceVO processInstanceVO = logDto.getProcessInstanceVO();
        String pocessDefinitionId = "";
        String taskDefinitionKey = "";
        if (processInstanceVO != null) {
            TaskHistoryDetailVo taskHistoryDetailVo = processInstanceVO.getTaskHistoryDetailVo();
            if (ObjectUtil.isNotNull(taskHistoryDetailVo)) {
                pocessDefinitionId = taskHistoryDetailVo.getProcessDefinitionId();
                taskDefinitionKey = taskHistoryDetailVo.getTaskDefinitionKey();
            }
        }

        ApprovalInfoDTO approvalInfoDTO = logDto.getApprovalInfoDTO();

        FlowManageLog flowManageLog = new FlowManageLog();
        BeanUtils.copyProperties(logDto, flowManageLog);
        flowManageLog.setIdcard(approvalInfoDTO.getApplicantIdcard());
        flowManageLog.setOrgName(approvalInfoDTO.getApplicantOrgName());
        flowManageLog.setOrgCode(approvalInfoDTO.getApplicantOrgCode());
        flowManageLog.setBusinessCode(approvalInfoDTO.getBusinessCode());
        flowManageLog.setTitle(approvalInfoDTO.getProcessTitle());
        flowManageLog.setDisposeType(manageDisposeTypeEnum.value);
        //操作内容策划不知道要存什么，暂时放空
//        flowManageLog.setOperateContent(JSON.toJSONString(logDto.getApprovalInfo()));
        flowManageLog.setApproveRecordId(approvalInfoDTO.getId());
        flowManageLog.setApproveNo(approvalInfoDTO.getApproveNo());
        flowManageLog.setProcessInstanceId(processInstanceId);
        //发送审批流程管理日志到消息中心
        String flowManageLogJsonString = JSON.toJSONString(flowManageLog);
        logger.info("=====发送审批流程管理日志消息,消息体为：{}====", flowManageLogJsonString);
        //审批流程管理日志类型不会涉及到审批风险，故taskKey/flowKey可以为空
        LogInfoMetaDTO logInfoMetaDTO = new LogInfoMetaDTO();
        logInfoMetaDTO.setLogType(LogTypeEnum.MANAGE_LOG.getValue());
        logInfoMetaDTO.setTaskKey(taskDefinitionKey);
        logInfoMetaDTO.setFlowKey(pocessDefinitionId);
        logInfoMetaDTO.setProcessInstanceId(processInstanceId);
        logInfoMetaDTO.setTerminalIp(logDto.getCreateIp());
        logInfoMetaDTO.setBusinessType(manageDisposeTypeEnum.auditCode);
        
     // 2024-8-21 对接BU新规范
        logInfoMetaDTO.setTaskId(approvalInfoDTO.getTaskId());
        logInfoMetaDTO.setTaskName(approvalInfoDTO.getTaskName());
        logInfoMetaDTO.setAppName(approvalInfoDTO.getAppName());
        logInfoMetaDTO.setOperatorCertNo(approvalInfoDTO.getApplicantIdcard());
        logInfoMetaDTO.setOperatorName(approvalInfoDTO.getApplicantName());
        logInfoMetaDTO.setOperatorOrgCode(approvalInfoDTO.getApplicantOrgCode());
        logInfoMetaDTO.setOperatorOrgName(approvalInfoDTO.getApplicantOrgName());
        logInfoMetaDTO.setOperateTime(DateUtil.format(new Date(), "yyyy-MM-dd HH:mm:ss"));
        
        if (ManageDisposeTypeEnum.UPDATE.equals(manageDisposeTypeEnum)) {
        	logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.START.getValue());
        }else if (ManageDisposeTypeEnum.DELETED.equals(manageDisposeTypeEnum)) {
        	logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.RECALL.getValue());
        }
        
        sendInfo(flowManageLogJsonString, "审批流程管理日志", logInfoMetaDTO);
    }


    /**
     * 发送日志消息
     *
     * @param content        消息体对象json字符串
     * @param title          标题
     * @param logInfoMetaDTO 日志元信息
     */
    public void sendInfo(String content, String title, LogInfoMetaDTO logInfoMetaDTO) {
        String appToken = logInfoMetaDTO.getAppToken();
        if (StrUtil.isBlank(appToken)) {
            ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
            if (servletRequestAttributes != null) {
                HttpServletRequest request = servletRequestAttributes.getRequest();
                String appTokenId = request.getHeader(Constants.RZZX_APPTOKEN);
                logger.info("从请求头获取到应用令牌信息:{}", appTokenId);
                logInfoMetaDTO.setAppToken(appTokenId);
                
                
                
            }
        }
        
        System.out.println("======================================20240909=========================");
        appToken = logInfoMetaDTO.getAppToken();
        if (StrUtil.isBlank(appToken)) {
	        SecurityUser securityUser = (SecurityUser)UserContextUtils.getCurrentUser();
	        String appTokenId = (String)securityUser.getExtendtions().get(UserExtInfoEnum.APP_TOKEN_ID.getValue());
	        logger.info("=================20240909==================" + appTokenId);
	        logInfoMetaDTO.setAppToken(appTokenId);
        }
        System.out.println("======================================20240909=========================");

        ApproveLogDTO approveLogDTO = ApproveLogDTO.builder()
                .content(content)
                .title(title)
                .logInfoMetaDTO(logInfoMetaDTO).build();

        approveEventPublish.publishApproveLogEvent(approveLogDTO);
    }

    /**
     * 构建日志Dto
     *
     * @param approvalInfoDTO   审批信息
     * @param vo                实例信息
     * @param approveResultEnum 返回信息
     * @param failMsg           错误消息
     * @return 日志信息
     */
    @Override
    public LogDto buildLogDto(ApprovalInfoDTO approvalInfoDTO, ProcessInstanceVO vo, ApproveResultEnum approveResultEnum,
                              String failMsg, boolean needSendRedListRisk, OperateRespVO operateRespVO) {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        assert servletRequestAttributes != null;
        HttpServletRequest request = servletRequestAttributes.getRequest();
        String realIpAdrress = IpUtils.getRealIpAdrress(request);

        String createUser;
        String createUserName = "";
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        if (currentUser != null) {
            createUser = currentUser.getIdcard();
            createUserName = currentUser.getName();
        } else {
            // 没有用户登陆的话使用凭据作为操作人
            String idcard = ProofsUtil.getIdcard();
            createUser = idcard;
            OauthUserVO oauthUserInfoByIdcard = dcucBusiness.getOauthUserInfoByIdcard(idcard);
            if (ObjectUtil.isNotNull(oauthUserInfoByIdcard)) {
                createUserName = oauthUserInfoByIdcard.getName();
            }
        }

        LogDto logDto = new LogDto();
        logDto.setCreateIp(realIpAdrress);
        logDto.setOperateContent("");
        logDto.setCreateUser(createUser);
        logDto.setCreateUserName(createUserName);
        logDto.setApprovalInfoDTO(approvalInfoDTO);
        logDto.setProcessInstanceVO(vo);
        logDto.setFailMsg(failMsg);
        logDto.setResult(approveResultEnum.value);
        logDto.setCreateTime(new Date());
        logDto.setNeedSendRedListRisk(needSendRedListRisk);
        logDto.setRedListOperateRespVO(operateRespVO);
        return logDto;
    }



    // TODO add wanghao  对接bu日志
    /**
     * 构建审批请求日志对象
     *
     * @param workFlowOperationEnum  处理类型
     * @param logDto       日志Dto
     * @param flowNodeCode 流程节点代码
     * @param flowNodeName 流程节点名称
     * @param taskKey      任务key
     * @param flowKey      流程key
     */
    private void buildRequestLog(ApproveWorkFlowOperationEnum  workFlowOperationEnum, LogDto logDto, String flowNodeCode, String flowNodeName,
                                 String taskKey, String flowKey, String processInstanceId, String businessCode) {

        ApprovalInfoDTO approvalInfoDTO = logDto.getApprovalInfoDTO();
        RequestLog requestLog = new RequestLog();
        requestLog.setIdcard(approvalInfoDTO.getApplicantIdcard());
        requestLog.setOrgName(approvalInfoDTO.getApplicantOrgName());
        requestLog.setOrgCode(approvalInfoDTO.getApplicantOrgCode());
        requestLog.setBusinessCode(approvalInfoDTO.getBusinessCode());
        requestLog.setTitle(approvalInfoDTO.getProcessTitle());

        requestLog.setDisposeType(workFlowOperationEnum.getValue());        //  字典包，没有我们新 审批动作？？？,直接存其他字符？  bu标有标准吗   日志内容里面的 处理类型

        requestLog.setResult(logDto.getResult());
        requestLog.setFailMsg(logDto.getFailMsg());
        requestLog.setCreateIp(logDto.getCreateIp());
        //操作内容策划不知道要存什么，暂时放空
//        requestLog.setOperateContent(JSON.toJSONString(logDto.getApprovalInfo()));
        requestLog.setCreateUser(logDto.getCreateUser());
        requestLog.setCreateUserName(logDto.getCreateUserName());
        Date date = new Date();
        requestLog.setCreateTime(date);
        requestLog.setFlowNodeCode(flowNodeCode);
        requestLog.setFlowNodeName(flowNodeName);
        requestLog.setApproveRecordId(approvalInfoDTO.getId());
        requestLog.setProcessInstanceId(processInstanceId);
        requestLog.setApproveNo(approvalInfoDTO.getApproveNo());
        //发送审批请求日志到消息中心
        logger.info("=====发送审批申请请求日志-发起消息,消息体为：{}====", JSON.toJSONString(requestLog));
        LogInfoMetaDTO logInfoMetaDTO = new LogInfoMetaDTO();
        logInfoMetaDTO.setLogType(LogTypeEnum.REQUEST_LOG.getValue());
        logInfoMetaDTO.setTaskKey(taskKey);
        logInfoMetaDTO.setFlowKey(flowKey);
        logInfoMetaDTO.setProcessInstanceId(processInstanceId);
        logInfoMetaDTO.setTerminalIp(logDto.getCreateIp());
        // 请求发起
        logInfoMetaDTO.setBusinessType(businessCode);


        // 2024-8-21 对接BU新规范
        logInfoMetaDTO.setTaskId(approvalInfoDTO.getTaskId());
        logInfoMetaDTO.setTaskName(approvalInfoDTO.getTaskName());
        logInfoMetaDTO.setAppName(approvalInfoDTO.getAppName());
        logInfoMetaDTO.setOperatorCertNo(approvalInfoDTO.getApplicantIdcard());
        logInfoMetaDTO.setOperatorName(approvalInfoDTO.getApplicantName());
        logInfoMetaDTO.setOperatorOrgCode(approvalInfoDTO.getApplicantOrgCode());
        logInfoMetaDTO.setOperatorOrgName(approvalInfoDTO.getApplicantOrgName());
        logInfoMetaDTO.setOperateTime(DateUtil.format(date, "yyyy-MM-dd HH:mm:ss"));


        // 缺少 bu标 日志元数据  字典编码 QmApproveTypeEnumV2
        if (ApproveWorkFlowOperationEnum.SUSPEND.equals(workFlowOperationEnum)) {
//            logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.START.getValue());
            logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.SUSPEND.getValue());
        }else if(ApproveWorkFlowOperationEnum.ACTIVATE.equals(workFlowOperationEnum)) {
            logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.ACTIVATE.getValue());
        }else if(ApproveWorkFlowOperationEnum.REDO.equals(workFlowOperationEnum)){
            logInfoMetaDTO.setOperateType(QmApproveTypeEnumV2.REDO.getValue());
        }

        System.err.println("========新增挂起、激活、终止--bu日志requestLog Content:"+JSON.toJSONString(requestLog));
        System.err.println("========新增挂起、激活、终止--bu日志logInfoMetaDTO :"+JSON.toJSONString(logInfoMetaDTO));

        sendInfo(JSON.toJSONString(requestLog), "审批请求日志", logInfoMetaDTO);
    }


}
