package com.dragonsoft.dcuc.approve.business.impl;

import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.dragonsoft.approve.common.ErrorCode;
import com.dragonsoft.dcuc.approve.business.IApproveUploadBusiness;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveWorkFlowOperationEnum;
import com.dragonsoft.dcuc.approve.enumresources.PermissionValidTypeEnum;
import com.dragonsoft.dcuc.approve.model.ApproveActHiTask;
import com.dragonsoft.dcuc.approve.model.ApproveCirculationInfo;
import com.dragonsoft.dcuc.approve.model.ApproveQueryRelation;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.req.ApproveCirculationReqVO;
import com.dragonsoft.dcuc.approve.model.req.ApproveRecordReportReqVO;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.service.IApproveActHiTaskService;
import com.dragonsoft.dcuc.approve.service.IApproveCirculationService;
import com.dragonsoft.dcuc.approve.service.IApproveQueryRelationService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author wangzw
 * @date 2021/5/26
 * @desc 审批信息保存
 */
@Component
public class ApproveUploadBusinessImpl implements IApproveUploadBusiness {

    private static final Logger logger = LoggerFactory.getLogger(ApproveUploadBusinessImpl.class);

    private IApproveRecordService approveRecordService;

    private IApproveQueryRelationService approveQueryRelationService;

    private IApproveCirculationService approveCirculationService;

    private IApproveActHiTaskService approveActHiTaskService;

    private ApproveProperties approveProperties;

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Autowired
    public void setApproveQueryRelationService(IApproveQueryRelationService approveQueryRelationService) {
        this.approveQueryRelationService = approveQueryRelationService;
    }

    @Autowired
    public void setApproveCirculationService(IApproveCirculationService approveCirculationService) {
        this.approveCirculationService = approveCirculationService;
    }

    @Autowired
    public void setApproveActHiTaskService(IApproveActHiTaskService approveActHiTaskService) {
        this.approveActHiTaskService = approveActHiTaskService;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseStatus approveUpload(List<ApproveRecordReportReqVO> approveRecordList) {
        logger.info("审批单信息保存开始，数据推送条数：【{}】", approveRecordList.size());
        try {
            for (ApproveRecordReportReqVO approveRecord : approveRecordList) {
                String oldApproveNo = approveRecord.getApproveNo();
                String appCode = approveRecord.getAppCode();
                String newApproveNo = buildNewApproveNo(appCode, oldApproveNo);
                ApproveRecordInfo approveRecordInfo = approveRecordService.selectByApproveNo(newApproveNo);
                boolean isSave = false;
                if (approveRecordInfo == null) {
                    approveRecordInfo = new ApproveRecordInfo();
                    isSave = true;
                }

                String applicantIdcard = approveRecord.getApplicantIdcard();
                Date applyTimeDate = DateUtils.parseDate(approveRecord.getApplyTime(), DateUtils.DATE_FORMAT_19);

                BeanUtils.copyProperties(approveRecord, approveRecordInfo);
                approveRecordInfo.setBusinessCode(approveRecord.getProcessNo());
                approveRecordInfo.setIsDeleted(BooleanEnum.FALSE.getValue());
                approveRecordInfo.setCreateUser(applicantIdcard);
                approveRecordInfo.setCreateTime(applyTimeDate);
                approveRecordInfo.setProcessTitle(approveRecord.getApproveTitle());
                approveRecordInfo.setTaskName("结束");
                approveRecordInfo.setTaskCode("end");

                approveRecordInfo.setApproveNo(newApproveNo);
                approveRecordInfo.setCreateUser(applicantIdcard);

                if (isSave) {
                    approveRecordService.save(approveRecordInfo);
                } else {
                    approveRecordService.updateById(approveRecordInfo);
                }
                String approveRecordId = approveRecordInfo.getId();
                // 如果是更新删除原有数据
                if (!isSave) {
                    approveQueryRelationService.deleteByApproveRecordId(approveRecordId);
                    approveCirculationService.deleteByApproveRecordId(approveRecordId);
                    approveActHiTaskService.deleteByApplyRecordId(approveRecordId);
                }

                ApproveQueryRelation approveQueryRelation = new ApproveQueryRelation();
                BeanUtils.copyProperties(approveRecord, approveQueryRelation);
                approveQueryRelation.setIsDeleted(BooleanEnum.FALSE.getValue());
                approveQueryRelation.setCreateUser(applicantIdcard);
                approveQueryRelation.setCreateTime(applyTimeDate);
                //保存关联关系
                approveQueryRelation.setApproveRecordId(approveRecordId);
                approveQueryRelationService.save(approveQueryRelation);
                //审批单包含流转记录处理
                List<ApproveCirculationReqVO> approveCirculationList = approveRecord.getApproveCirculationList();
                for (ApproveCirculationReqVO approveCirculation : approveCirculationList) {
                    ApproveCirculationInfo approveCirculationInfo = new ApproveCirculationInfo();
                    BeanUtils.copyProperties(approveCirculation, approveCirculationInfo);
                    approveCirculationInfo.setApproveNo(newApproveNo);
                    approveCirculationInfo.setIsDeleted(BooleanEnum.FALSE.getValue());
                    approveCirculationInfo.setCreateUser(approveCirculation.getOperateUserId());
                    approveCirculationInfo.setCreateTime(Timestamp.valueOf(approveCirculation.getOperateTime()));
                    //保存关联关系
                    approveCirculationInfo.setApproveRecordId(approveRecordId);
                    approveCirculationService.save(approveCirculationInfo);
                    //将审批记录转换成已办数据
                    ApproveActHiTask actHiTask = new ApproveActHiTask();
                    actHiTask.setApplyRecordId(approveRecordId);
                    actHiTask.setBusinessCode(approveRecordInfo.getBusinessCode());
                    actHiTask.setBusinessName(approveRecordInfo.getProcessName());
                    actHiTask.setTaskId(approveCirculationInfo.getTaskId());
                    actHiTask.setTaskCode(approveCirculationInfo.getNodeNo());
                    actHiTask.setTaskName(approveCirculationInfo.getNodeName());
                    actHiTask.setOperateUserIdcard(approveCirculationInfo.getOperateUserId());
                    actHiTask.setOperateUserName(approveCirculationInfo.getOperateUserName());
                    actHiTask.setOperateType(approveCirculationInfo.getOperateType());
                    actHiTask.setProcessInstanceId(approveRecordInfo.getProcessInstanceId());
                    actHiTask.setProcessTitle(approveRecordInfo.getProcessTitle());
                    try {
                        actHiTask.setOperateTime(DateUtils.parseDate(approveCirculationInfo.getOperateTime(), DateUtils.DATE_FORMAT_19));
                    } catch (ParseException ignored) {

                    }
                    actHiTask.setDeleted(BooleanEnum.FALSE.getValue());
                    approveActHiTaskService.save(actHiTask);
                }

            }
        } catch (Exception e) {
            logger.error("保存审批单失败！", e);
            throw new ApplicationException(ErrorCode.APPROVAL_UPLOAD_EXCEPTION.getCode(), ErrorCode.APPROVAL_UPLOAD_EXCEPTION.getMsg());
        }
        return ResponseStatus.success();
    }

    /**
     * 生成新审批单号
     *
     * @param appCode   应用代码
     * @param approveNo 原审批单号
     * @return 新审批单号
     */
    private String buildNewApproveNo(String appCode, String approveNo) {
        return appCode + ":" + approveNo;
    }

    /**
     * 检查参数
     *
     * @param approveRecordList 审批列表
     * @return 错误信息
     */
    @Override
    public String checkParam(List<ApproveRecordReportReqVO> approveRecordList) {
        StringBuilder msg = new StringBuilder();
        for (int i = 0; i < approveRecordList.size(); i++) {
            ApproveRecordReportReqVO approveRecordReqVO = approveRecordList.get(i);
            if (approveRecordReqVO.getAppCode().equals(approveProperties.getAppCode())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的appCode参数不允许与审批中心一致,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApproveType())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的approveType参数不允许为空,");
            } else {
                //判断值是否为正确的码值
                boolean contains = EnumUtils.contains(ApproveTypeEnum.class, approveRecordReqVO.getApproveType());
                if (!contains) {
                    msg.append("ApproveRecordDTO数组[").append(i).append("]的approveType参数输入不正确,");
                }
            }
            if (StringUtils.isNotEmpty(approveRecordReqVO.getPermissionValidType())) {
                String permissionValidType = approveRecordReqVO.getPermissionValidType();
                PermissionValidTypeEnum permissionValidTypeEnum = EnumUtils.enumOf(PermissionValidTypeEnum.class, permissionValidType);
                if (permissionValidTypeEnum == null) {
                    msg.append("ApproveRecordDTO数组[").append(i).append("]的permissionValidType参数输入不正确,");
                }
            }


            if (StringUtils.isNotEmpty(approveRecordReqVO.getApplyTime())) {
                try {
                    DateUtils.parseDate(approveRecordReqVO.getApplyTime(), DateUtils.DATE_FORMAT_19);
                } catch (Exception e) {
                    msg.append("ApproveRecordDTO数组[").append(i).append("]的applyTime参数输入不正确,");
                }
            }

            if (StringUtils.isNotEmpty(approveRecordReqVO.getAccessResources())) {
                try {
                    Map<String, String> codeMap = (Map) JSON.parse(approveRecordReqVO.getAccessResources());
                } catch (Exception e) {
                    msg.append("ApproveRecordDTO数组[").append(i).append("]的accessResources参数输入不正确,");
                }
            }
            if (StringUtils.isNotEmpty(approveRecordReqVO.getAccessResourcesNo())) {
                try {
                    Map<String, String> codeMap = (Map) JSON.parse(approveRecordReqVO.getAccessResourcesNo());
                } catch (Exception e) {
                    msg.append("ApproveRecordDTO数组[").append(i).append("]的accessResourcesNo参数输入不正确,");
                }
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApproveNo())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的approveNo参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApproveTitle())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的approveTitle参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplicantIdcard())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applicantIdcard参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplicantName())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applicantName参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplicantName())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applicantName参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplicantPhoneNo())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applicantPhoneNo参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplicantOrgCode())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applicantOrgCode参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplicantOrgName())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applicantOrgName参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplyTime())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applyTime参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getAppName())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的appName参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getAppCode())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的appCode参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getAccessResources())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的accessResources参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getAccessResourcesNo())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的accessResourcesNo参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getApplyReason())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的applyReason参数不允许为空,");
            }
            if (StringUtils.isEmpty(approveRecordReqVO.getFinalApprovalResult())) {
                msg.append("ApproveRecordDTO数组[").append(i).append("]的finalApprovalResult参数不允许为空,");
            } else {
                //判断值是否为正确的码值
                ApproveResultEnum resultEnum = EnumUtils.enumOf(ApproveResultEnum.class, approveRecordReqVO.getFinalApprovalResult());
                if (resultEnum == null) {
                    msg.append("ApproveRecordDTO数组[").append(i).append("]的finalApprovalResult参数输入不正确,");
                }
            }
            if (com.baomidou.mybatisplus.core.toolkit.CollectionUtils.isNotEmpty(approveRecordReqVO.getApproveCirculationList())) {
                for (int j = 0; j < approveRecordReqVO.getApproveCirculationList().size(); j++) {
                    ApproveCirculationReqVO approveCirculationReqVO = approveRecordReqVO.getApproveCirculationList().get(j);
                    if (StringUtils.isEmpty(approveCirculationReqVO.getNodeName())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的nodeName()参数不允许为空,");
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getNodeNo())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的nodeNo参数不允许为空,");
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getOperateType())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的operatorType参数不允许为空,");
                    } else {
                        //判断值是否为正确的枚举
                        ApproveWorkFlowOperationEnum resultEnum = EnumUtils.enumOf(ApproveWorkFlowOperationEnum.class, approveCirculationReqVO.getOperateType());
                        if (resultEnum == null) {
                            msg.append("ApproveRecordDTO数组[").append(i).append("]的operatorType参数输入不正确,");
                        }
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getOperateTime())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的operateTime参数不允许为空,");
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getOperateUserName())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的operateUserName()参数不允许为空,");
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getOperateUserId())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的operateUserId参数不允许为空,");
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getOperateUserOrg())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的operateUserOrg参数不允许为空,");
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getOperateUserOrgId())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的operateUserOrgId参数不允许为空,");
                    }
                    if (StringUtils.isEmpty(approveCirculationReqVO.getApproveOpinion())) {
                        msg.append("数组[").append(i).append("]下的approveCirculationDTO数组[").append(j).append("]的approveOpinion参数不允许为空,");
                    }

                    if (StringUtils.isNotEmpty(approveCirculationReqVO.getOperateTime())) {
                        try {
                            DateUtils.parseDate(approveCirculationReqVO.getOperateTime(), DateUtils.DATE_FORMAT_19);
                        } catch (Exception e) {
                            msg.append("ApproveRecordDTO数组[").append(i).append("]的operateTime参数输入不正确,");
                        }
                    }
                }
            }
        }
        return msg.toString();
    }


}
