package com.dragonsoft.dcuc.approve.business.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.EnumUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.dragonsoft.approve.common.ErrorCode;
import com.dragonsoft.dcuc.approve.business.IWorkFlowBusiness;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.enumresources.OperateOptionEnum;
import com.dragonsoft.dcuc.approve.enumresources.PageTypeEnum;
import com.dragonsoft.dcuc.approve.model.resp.ApproveCommentVo;
import com.dragonsoft.dcuc.approve.model.resp.ApproveWorkflowProcessTypeVo;
import com.dragonsoft.dcuc.approve.model.resp.TaskOperateRespVO;
import com.dragonsoft.dcuc.approve.model.resp.TaskOutgoingVo;
import com.dragonsoft.dcuc.approve.properties.WorkFlowProperties;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.duceap.core.entity.page.PageRequest;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.duwf.api.enums.StateTypeEnum;
import com.dragonsoft.duceap.duwf.api.model.*;
import com.dragonsoft.duceap.workflow.client.WorkflowClientFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 工作流业务
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/13
 */
@Slf4j
@Service
public class WorkFlowBusinessImpl implements IWorkFlowBusiness {

    private WorkflowClientFactory workflowClientFactory;

    private WorkFlowProperties workFlowProperties;

    @Autowired
    public void setWorkflowClientFactory(WorkflowClientFactory workflowClientFactory) {
        this.workflowClientFactory = workflowClientFactory;
    }

    @Autowired
    public void setWorkFlowProperties(WorkFlowProperties workFlowProperties) {
        this.workFlowProperties = workFlowProperties;
    }

    @Override
    public List<ApproveCommentVo> getProcessDefPath(String processInstanceId) {
        //获取流程图对象
        TraceProcessAndOverVo traceProcessAndOverVo = workflowClientFactory.getWorkflowProcessClient().queryTraceProcessAndOver(processInstanceId);
        //解析流程图json
        JSONObject data = JSONObject.parseObject(traceProcessAndOverVo.getJson());
        JSONArray jsonArray = JSONArray.parseArray(data.getString("nodeDataArray"));
        List<ApproveCommentVo> approveCommentVos = new ArrayList<>();
        for (int m = 0; m < jsonArray.size(); m++) {
            JSONObject nodeData = jsonArray.getJSONObject(m);
            if ("activity".equalsIgnoreCase(nodeData.getString("category"))) {
                //获取节点信息
                ApproveCommentVo approveCommentVo = new ApproveCommentVo();
                approveCommentVo.setNodeName(nodeData.getString("text"));
                String assignee = nodeData.getString("assignee");
                approveCommentVo.setAssignee(assignee);
                approveCommentVo.setId(nodeData.getString("id"));
                approveCommentVo.setSort(m);
                approveCommentVos.add(approveCommentVo);
            }
        }
        return approveCommentVos;
    }

    /**
     * 根据任务ID查看当前节点允许的操作类型
     *
     * @param taskId            任务ID
     * @param operateOptionEnum 操作类型
     * @return 是否允许
     */
    @Override
    public boolean checkOperateOptionOfCurrentStep(String taskId, String processInstanceId, String taskNode, OperateOptionEnum operateOptionEnum) {
        //查询任务详情
        TaskHistoryDetailVo taskDetailVo = workflowClientFactory.getWorkflowTaskClient().queryTaskDetail(taskId);
        String taskDefinitionKey = taskDetailVo.getTaskDefinitionKey();
        //根据流程实例ID和任务定义ID返回表单操作类型
        FromAndOperationVo fromAndOperationVo = workflowClientFactory.getWorkflowProcessClient().queryFormMetadataByProInsId(processInstanceId, taskDefinitionKey, taskNode);
        List<OperationVo> operation = fromAndOperationVo.getOperation();
        if (CollectionUtils.isNotEmpty(operation)) {
            for (OperationVo operationVo : operation) {
                if (operationVo.getBtnId().equalsIgnoreCase(operateOptionEnum.getValue())) {
                    return true;
                }
            }
        }
        return false;
    }

    @Override
    public WorkflowProcessVo queryWorkflowProcessByBusinessCode(String businessCode) {
        WorkflowProcessVo workflowProcessVo;
        try {
            //调用工作流获取流程信息
            workflowProcessVo = workflowClientFactory.getWorkflowProcessClient().queryWorkflowProcessByBusinessCode(businessCode);
        } catch (ApplicationException e) {
            log.error("流程类型标识码-Exception：【{}】", businessCode, e);
            throw new ApplicationException(ErrorCode.WORKFLOW_EXCEPTION.getCode(), ErrorCode.WORKFLOW_EXCEPTION.getMsg());
        }
        return workflowProcessVo;
    }

    @Override
    public WorkflowProcesstypeVo queryProcessType(String processTypeCode) {
        //获取流程类别详情
        return workflowClientFactory.getWorkflowProcessDefTypeClient().findOneByCode(processTypeCode);
    }

    @Override
    public List<ApproveWorkflowProcessTypeVo> queryProcessTypeList() {

        String processTypeCode = workFlowProperties.getProcessTypeCode();

        //根据流程类别code获取流程类别信息
        WorkflowProcesstypeVo workflowProcesstypeVo = queryProcessType(processTypeCode);
        List<ApproveWorkflowProcessTypeVo> result = new ArrayList<>();
        if (null != workflowProcesstypeVo) {
            //根据流程类别id查询子类别
            Searchable searchable = Searchable.newSearchable();
            searchable.addSearchFilter("upProcess", SearchOperator.eq, workflowProcesstypeVo.getId());
            List<WorkflowProcesstypeVo> list = workflowClientFactory.getWorkflowProcessDefTypeClient().list(searchable);
            //循环获取每个子类别下的所有定义流程
            for (WorkflowProcesstypeVo processTypeVo : list) {
                ApproveWorkflowProcessTypeVo approveWorkflowProcessTypeVo = new ApproveWorkflowProcessTypeVo();
                BeanUtils.copyProperties(processTypeVo, approveWorkflowProcessTypeVo);
                searchable = Searchable.newSearchable();
                searchable.setPage(new PageRequest(0, 999));
                searchable.addSearchFilter("processType", SearchOperator.eq, processTypeVo.getId());
                //获取流程类型下的流程信息
                Page<WorkflowProcessVo> processVoPage = workflowClientFactory.getWorkflowProcessClient().queryWorkflowProcessPage(searchable);
                List<WorkflowProcessVo> activeDataList = new ArrayList<>();
                for (WorkflowProcessVo workFlowApproveVo : processVoPage.getContent()) {
                    if ("1".equalsIgnoreCase(workFlowApproveVo.getStatus())) {
                        activeDataList.add(workFlowApproveVo);
                    }
                }
                approveWorkflowProcessTypeVo.setWorkflowProcessVoList(activeDataList);
                result.add(approveWorkflowProcessTypeVo);
            }
        }

        return result;
    }

    /**
     * 查询任务分支（运行时任务）
     *
     * @param taskId 任务ID
     * @return 分支信息
     */
    @Override
    public List<TaskOutgoingVo> buildTaskOutgoingVoList(String taskId) {
        //调用工作流查询任务分支（运行时任务）
        OutgoingVo outgoingVo = workflowClientFactory.getWorkflowTaskClient().queryTaskOutgoing(taskId);
        List<TaskOutgoingVo> list = new ArrayList<>();
        log.info("工作流返回对象：【{}】", JSON.toJSONString(outgoingVo));
        if (outgoingVo != null && CollectionUtils.isNotEmpty(outgoingVo.getTaskDefinitionVoList())) {
            for (TaskDefinitionVo taskDefinitionVo : outgoingVo.getTaskDefinitionVoList()) {
                if (!ApproveConstants.END_STR.equalsIgnoreCase(taskDefinitionVo.getTaskDefinitionKey())) {
                    TaskOutgoingVo taskOutgoingVo = new TaskOutgoingVo();
                    taskOutgoingVo.setTaskDefinitionKey(taskDefinitionVo.getTaskDefinitionKey());
                    taskOutgoingVo.setName(taskDefinitionVo.getName());
                    taskOutgoingVo.setTaskCode(taskDefinitionVo.getTaskCode());
                    list.add(taskOutgoingVo);
                }
            }
        }
        return list;
    }

    @Override
    public TaskOperateRespVO getTaskSupportOperate(String taskId, PageTypeEnum pageTypeEnum) {
        // 判断是否是结束节点
        TaskOperateRespVO taskOperateRespVO = new TaskOperateRespVO();
        boolean showBack = false;
        boolean showEdit = false;
        boolean showRecall = false;
        if (!ApproveConstants.END_STR.equals(taskId)) {
            TaskHistoryDetailVo taskHistoryDetailVo = workflowClientFactory.getWorkflowTaskClient().queryTaskDetail(taskId);
            String processInstanceId = taskHistoryDetailVo.getProcessInstanceId();
            String taskCode = taskHistoryDetailVo.getTaskCode();
            String processDefinitionId = taskHistoryDetailVo.getProcessDefinitionId();
            String taskDefinitionKey = taskHistoryDetailVo.getTaskDefinitionKey();

            //退回
            showRecall = checkOperateOptionOfCurrentStep(taskId, processInstanceId, taskCode, OperateOptionEnum._BTN_BACK_COMMIT);
            //流程出口信息
            OutgoingVo outgoingVo = workflowClientFactory.getWorkflowTaskClient().queryStartOutgoing(processDefinitionId);
            //设置是否显示撤回
            showBack = appendShowBack(taskId, processInstanceId, outgoingVo, pageTypeEnum);
            //设置是否显示编辑
            showEdit = appendShowEdit(outgoingVo, taskDefinitionKey);
        }

        taskOperateRespVO.setShowBack(showBack);
        taskOperateRespVO.setShowEdit(showEdit);
        taskOperateRespVO.setShowRecall(showRecall);


        System.err.println(JSONObject.toJSONString(taskOperateRespVO));
        return taskOperateRespVO;
    }

    /**
     * 设置是否显示撤回按钮
     *
     * @param taskId            任务id
     * @param processInstanceId 实例ID
     */
    private boolean appendShowBack(String taskId, String processInstanceId,
                                   OutgoingVo outgoingVo, PageTypeEnum pageTypeEnum) {

        boolean showBack = false;
        List<String> taskDefinitionKeyList = new ArrayList<>();
        List<TaskDefinitionVo> taskDefinitionVoList = new ArrayList<>();
        if (outgoingVo != null) {
            taskDefinitionVoList = outgoingVo.getTaskDefinitionVoList();
            if (CollUtil.isNotEmpty(taskDefinitionVoList)) {    //TODO add  wanghao
                for (TaskDefinitionVo taskDefinitionVo : taskDefinitionVoList) {
                    if (ObjectUtil.isNotNull(taskDefinitionVo)) {
                        taskDefinitionKeyList.add(taskDefinitionVo.getTaskDefinitionKey());
                    }
                }
            }
        }


        if (!taskId.equalsIgnoreCase(ApproveConstants.END_STR)) {
            String historyTaskDefinitionKey = "";
            if (pageTypeEnum != null && pageTypeEnum.equals(PageTypeEnum.REQUEST)) {
                if (!taskDefinitionVoList.isEmpty()) {
                    historyTaskDefinitionKey = taskDefinitionVoList.get(0).getTaskDefinitionKey();
                }

            } else {
                TaskHistoryDetailVo currentTask = workflowClientFactory.getWorkflowTaskClient().queryTaskDetail(taskId);
                historyTaskDefinitionKey = currentTask.getTaskDefinitionKey();
            }
            // 1. 判断是否为第一个节点，是则不能撤回
            // 2. 不是第一个节点，且当前流程的上一个节点为撤回人的处理节点，a->b->c->d 只有c能撤回节点才能撤回
            // 3. 判断当前节点是否有配置撤回的功能
            TaskHistoryDetailVo taskHistoryDetailVo = workflowClientFactory.getWorkflowTaskClient().queryTaskDetail(taskId);
            String taskDefinitionKey = taskHistoryDetailVo.getTaskDefinitionKey();
            if (taskDefinitionKeyList.contains(taskDefinitionKey)) {
                return false;
            }
            List<ApproveCommentVo> processDefPath = getProcessDefPath(processInstanceId);
            //当前活动节点任务在流程中的位置
            int currentTaskIndexPosition = 0;
            //已完成的节点在流程中的位置
            int finishTaskIndexPosition = 0;
            //游标
            int index = 0;
            for (ApproveCommentVo approveCommentVo : processDefPath) {
                if (approveCommentVo.getId().equalsIgnoreCase(taskDefinitionKey)) {
                    finishTaskIndexPosition = index;
                }
                if (approveCommentVo.getId().equalsIgnoreCase(historyTaskDefinitionKey)) {
                    currentTaskIndexPosition = index;
                }
                index++;
            }
            //已完成的任务节点位置在当前任务节点位置的上一个位置，则可以撤回
            showBack = (finishTaskIndexPosition) == (currentTaskIndexPosition + 1);
        }

        return showBack;
    }

    /**
     * 判断是否为第一个节点，是则允许编辑
     *
     * @param outgoingVo        任务分支
     * @param taskDefinitionKey 任务定义key
     */
    private boolean appendShowEdit(OutgoingVo outgoingVo, String taskDefinitionKey) {
        List<String> taskDefinitionKeyList = new ArrayList<>();
        //获取流程出口的key对象
        for (TaskDefinitionVo taskDefinitionVo : outgoingVo.getTaskDefinitionVoList()) {
            taskDefinitionKeyList.add(taskDefinitionVo.getTaskDefinitionKey());
        }
        //判断是否为第2个节点，若是，则为true
        return taskDefinitionKeyList.contains(taskDefinitionKey);
    }

    @Override
    public TraceProcessAndOverVo processPic(String proInsId) {
        //调用工作流 流程图信息接口
        return workflowClientFactory.getWorkflowProcessClient().queryTraceProcessAndOver(proInsId);
    }

    @Override
    public ActivityNodeInfoVo processPicDetail(String proInsId, String statusType, String taskDefinitionId) {
        //调用工作流 流程图节点详情
        return workflowClientFactory.getWorkflowTaskClient().historyDiagramDetail(proInsId, taskDefinitionId,
                EnumUtil.likeValueOf(StateTypeEnum.class, statusType));
    }

}
