package com.dragonsoft.dcuc.approve.config;

import com.dragoninfo.dcuc.common.http.SkipSslVerificationHttpRequestFactory;
import com.dragonsoft.auditlog.collection.qmtj.provider.IRestTemplateProvider;
import com.dragonsoft.dcuc.approve.constants.ApproveBeanConstants;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.duceap.base.exception.ApplicationException;
import com.dragonsoft.smtools.loader.SMFactory;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.client.RestTemplate;

import java.lang.reflect.InvocationTargetException;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/6/30
 */
@Slf4j
@Configuration
public class ApproveConfig {

    private ApproveProperties approveProperties;

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Bean
    public IRestTemplateProvider restTemplateProvider() {
        return new SkipSslRestTemplateProvider();

    }

    static class SkipSslRestTemplateProvider implements IRestTemplateProvider {
        @Override
        public RestTemplate getRestTemplate() {
            SkipSslVerificationHttpRequestFactory skipSslVerificationHttpRequestFactory
                    = new SkipSslVerificationHttpRequestFactory();

            return new RestTemplate(skipSslVerificationHttpRequestFactory);
        }
    }


    @Bean(name = ApproveBeanConstants.SKIP_SSL_REST_TEMPLATE)
    public RestTemplate restTemplate() {
        SkipSslVerificationHttpRequestFactory skipSslVerificationHttpRequestFactory
                = new SkipSslVerificationHttpRequestFactory();
        skipSslVerificationHttpRequestFactory.setReadTimeout(approveProperties.getRequestTimeout());
        skipSslVerificationHttpRequestFactory.setConnectTimeout(approveProperties.getRequestTimeout());
        return new RestTemplate(skipSslVerificationHttpRequestFactory);
    }

    @Bean(name = ApproveBeanConstants.DRAGON_OBJECT_MAPPER)
    public ObjectMapper dragonObjectMapper() {
        ObjectMapper objectMapper = new ObjectMapper();
        // 忽略不对应的字段
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        return objectMapper;
    }

    @Bean
    public SMFactory smFactory() {
        try {
            return SMFactory.init(approveProperties.getSmTypeEnum());
        } catch (ClassNotFoundException | InvocationTargetException | InstantiationException | IllegalAccessException | NoSuchMethodException e) {
            log.error("国密初始化错误", e);
            throw new ApplicationException("国密初始化错误");
        }
    }

}
