package com.dragonsoft.dcuc.approve.event;

import com.dragonsoft.dcuc.approve.model.dto.ApproveLogDTO;
import com.dragonsoft.dcuc.approve.event.event.ApproveFinalResultEvent;
import com.dragonsoft.dcuc.approve.event.event.ApproveLogEvent;
import com.dragonsoft.dcuc.approve.event.event.ApproveRiskLogEvent;
import com.dragonsoft.dcuc.approve.event.event.AuditApproveOperateLogEvent;
import com.dragonsoft.dcuc.approve.model.RiskLog;
import com.dragonsoft.dcuc.approve.model.dto.ApproveMessageInfoDTO;
import com.dragonsoft.dcuc.approve.model.dto.QmOperateLogDTO;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

/**
 * @author wangrs
 * @date 2021/01/22
 * @desc 审批服务事件发布
 */
@Component
public class ApproveEventPublish implements ApplicationContextAware {

    private ApplicationContext applicationContext;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    /**
     * 异步发布风险日志事件
     *
     * @param riskLog 风险日志
     */
    @Async
    public void publishRiskLogEvent(RiskLog riskLog) {
        ApproveRiskLogEvent approveRiskLogEvent = new ApproveRiskLogEvent(riskLog);
        applicationContext.publishEvent(approveRiskLogEvent);
    }

    /**
     * 异步发布审批操作日志事件
     *
     * @param qmOperateLogDTO 操作日志
     */
    @Async
    public void publishOperateEvent(QmOperateLogDTO qmOperateLogDTO) {
        AuditApproveOperateLogEvent auditApproveOperateLogEvent = new AuditApproveOperateLogEvent(qmOperateLogDTO);
        applicationContext.publishEvent(auditApproveOperateLogEvent);
    }

    /**
     * 异步发布审批最终结果事件
     *
     * @param approveMessageInfoDTO 审批最终结果事件
     */
    @Async
    public void publishApproveFinalResultEvent(ApproveMessageInfoDTO approveMessageInfoDTO) {
        ApproveFinalResultEvent approveFinalResultEvent = new ApproveFinalResultEvent(approveMessageInfoDTO);
        applicationContext.publishEvent(approveFinalResultEvent);
    }

    /**
     * 审批日志事件
     *
     * @param approveLogDTO 审批日志事件
     */
    @Async
    public void publishApproveLogEvent(ApproveLogDTO approveLogDTO) {
        ApproveLogEvent approveLogEvent = new ApproveLogEvent(approveLogDTO);
        applicationContext.publishEvent(approveLogEvent);
    }
}
