package com.dragonsoft.dcuc.approve.event.listener;

import com.dragoninfo.dcuc.duceap.enums.SchemeTypeEnum;
import com.dragonsoft.auditlog.collection.qmtj.LogSendComponent;
import com.dragonsoft.auditlog.collection.qmtj.pojo.req.ApproveRiskBusLog;
import com.dragonsoft.dcuc.approve.event.event.ApproveRiskLogEvent;
import com.dragonsoft.dcuc.approve.model.RiskLog;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * @author wangrs
 * @date 2021/01/22
 * @desc 事件监听类
 */
@Component
public class ApproveRiskLogEventListener implements ApplicationListener<ApproveRiskLogEvent> {

    private LogSendComponent logSendComponent;

    private ApproveProperties approveProperties;


    /**
     * 审批风险日志
     */
    public static final String LOG_TYPE = "012";

    /**
     * 风险类型
     */
    public static final String MAIN_TYPE = "02";

    @Autowired
    public void setLogSendComponent(LogSendComponent logSendComponent) {
        this.logSendComponent = logSendComponent;
    }

    @Autowired
    public void setApproveProperties(ApproveProperties approveProperties) {
        this.approveProperties = approveProperties;
    }

    @Override
    public void onApplicationEvent(ApproveRiskLogEvent event) {
        if (!approveProperties.getSendToAudit()) {
            return;
        }
        RiskLog riskLog = event.getRiskLog();

        String riskType = riskLog.getRiskType();
        String riskContent = riskLog.getRiskContent();
        Date riskTime = riskLog.getRiskTime();
        String logId = riskLog.getLogId();
        // 文档有误，实际需使用毫秒
        String time = riskTime.getTime() + "";

        String auditSysId = approveProperties.getAuditSysId();
        SchemeTypeEnum schemeTypeEnum = EnumUtils.enumOf(SchemeTypeEnum.class, riskType);

        ApproveRiskBusLog approveRiskBusLog = new ApproveRiskBusLog();
        approveRiskBusLog.setType(schemeTypeEnum.auditType);
        approveRiskBusLog.setMainType(MAIN_TYPE);
        approveRiskBusLog.setContent(riskContent);
        approveRiskBusLog.setTime(time);
        approveRiskBusLog.setTaskId(logId);
        approveRiskBusLog.setLevel("低");

        List<ApproveRiskBusLog> approveBusLogList = Collections.singletonList(approveRiskBusLog);


        //这里调用启明天津的包进行日志请求
        logSendComponent.sendApproveRiskBusLog(auditSysId, LOG_TYPE, approveBusLogList);

    }
}
