package com.dragonsoft.dcuc.approve.event.listener;

import com.dragonsoft.dcuc.approve.constants.ApproveBeanConstants;
import com.dragonsoft.dcuc.approve.constants.ApproveConstants;
import com.dragonsoft.dcuc.approve.model.dto.RzyRiskLogDTO;
import com.dragonsoft.dcuc.approve.event.event.ApproveRiskLogEvent;
import com.dragonsoft.dcuc.approve.model.RiskLog;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/6/30
 */
@Slf4j
@Component
public class RzySyslogSendRiskInfoEventListener implements ApplicationListener<ApproveRiskLogEvent> {

    protected final Logger sysLogLogger = LoggerFactory.getLogger(ApproveConstants.SYS_LOG_NAME);

    private ObjectMapper objectMapper;

    @Qualifier(ApproveBeanConstants.DRAGON_OBJECT_MAPPER)
    @Autowired
    public void setObjectMapper(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Override
    public void onApplicationEvent(ApproveRiskLogEvent event) {
        RiskLog riskLog = event.getRiskLog();
        sendRiskLog(riskLog);
    }

    /**
     * 发送安全策略控制服务
     *
     * @param riskLog 风险信息
     */
    private void sendRiskLog(RiskLog riskLog) {
        String riskType = riskLog.getRiskType();
        String createUser = riskLog.getCreateUser();
        String riskIp = riskLog.getRiskIp();
        Date riskTime = riskLog.getRiskTime();
        String riskContent = riskLog.getRiskContent();

        RzyRiskLogDTO authRiskDTO = RzyRiskLogDTO.builder()
                .riskType(riskType)
                .userName(createUser)
                .authIp(riskIp)
                .riskAt(riskTime)
                .detail(riskContent)
                .build();
        try {
            String writeValueAsString = objectMapper.writeValueAsString(authRiskDTO);
            log.info("向安全策略控制服务发送风险：{}", writeValueAsString);
            sysLogLogger.info(writeValueAsString);
        } catch (JsonProcessingException e) {
            log.error("parse error", e);
        }

    }
}
