package com.dragonsoft.dcuc.approve.facade.api.v3;

import com.dragonsoft.dcuc.approve.constants.ApproveBeanConstants;
import com.dragonsoft.dcuc.approve.enumresources.ZeroTrustBusinessRespEnum;
import com.dragonsoft.dcuc.approve.model.req.v3.rzy.RzyRiskInfoDetailReqVO;
import com.dragonsoft.dcuc.approve.model.req.v3.rzy.RzyRiskInfoReqVO;
import com.dragonsoft.dcuc.approve.model.resp.RzyRiskInfoRespVO;
import com.dragonsoft.dcuc.approve.service.IApproveSecurityWarningInfoService;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import com.dragonsoft.smtools.loader.SMFactory;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/13
 */
@Slf4j
@RestController
public class ApiV3SecurityPolicyFacade implements IApiV3SecurityPolicyFacade {

    private ObjectMapper objectMapper;

    private SMFactory smFactory;

    private IApproveSecurityWarningInfoService approveSecurityWarningInfoService;

    @Autowired
    public void setApproveSecurityWarningInfoService(IApproveSecurityWarningInfoService approveSecurityWarningInfoService) {
        this.approveSecurityWarningInfoService = approveSecurityWarningInfoService;
    }

    @Autowired
    public void setSmFactory(SMFactory smFactory) {
        this.smFactory = smFactory;
    }

    @Qualifier(ApproveBeanConstants.DRAGON_OBJECT_MAPPER)
    @Autowired
    public void setObjectMapper(ObjectMapper objectMapper) {
        this.objectMapper = objectMapper;
    }

    @Override
    public RzyRiskInfoRespVO rzyRiskInfo(String riskInfoString) throws JsonProcessingException {

        log.info("接收到安全策略控制服务下发的预警信息：{}", JsonUtils.toJSONString(riskInfoString));

        JsonNode jsonNode = objectMapper.readTree(riskInfoString);
        JsonNode content = jsonNode.get("content");
        String contentString = objectMapper.writeValueAsString(content);
        String checksum = jsonNode.get("checksum").asText();
        log.info("接收的日志易预警信息校验原文:{}", contentString);
        log.info("接收的日志易预警信息校验码:{}", checksum);
        String generalCheckSum = smFactory.getSM3().summary(contentString).toString();
        log.info("接收的日志易预警信息生成的校验码:{}", generalCheckSum);

        if (!checksum.equalsIgnoreCase(generalCheckSum)) {
            RzyRiskInfoRespVO zeroTrustMessageRespVO = new RzyRiskInfoRespVO();
            zeroTrustMessageRespVO.setStatusCode(ZeroTrustBusinessRespEnum.OPERATE_FAIL.getValue());
            zeroTrustMessageRespVO.setMessage("检验码错误");
            zeroTrustMessageRespVO.setCheckSum(generalCheckSum);
            return zeroTrustMessageRespVO;
        }

        TypeReference<RzyRiskInfoReqVO> typeReference = new TypeReference<RzyRiskInfoReqVO>() {
        };
        RzyRiskInfoReqVO rzyRiskInfoReqVO;
        try {
            rzyRiskInfoReqVO = objectMapper.readValue(riskInfoString, typeReference);
        } catch (JsonProcessingException e) {
            log.error("请求内容解析失败", e);
            RzyRiskInfoRespVO zeroTrustMessageRespVO = new RzyRiskInfoRespVO();
            zeroTrustMessageRespVO.setStatusCode(ZeroTrustBusinessRespEnum.OPERATE_FAIL.getValue());
            zeroTrustMessageRespVO.setMessage("请求内容解析失败");
            return zeroTrustMessageRespVO;
        }

        List<RzyRiskInfoDetailReqVO> contentList = rzyRiskInfoReqVO.getContent();
        for (RzyRiskInfoDetailReqVO rzyRiskInfoDetailReqVO : contentList) {
            approveSecurityWarningInfoService.save(rzyRiskInfoDetailReqVO);
        }

        RzyRiskInfoRespVO zeroTrustMessageRespVO = new RzyRiskInfoRespVO();
        zeroTrustMessageRespVO.setStatusCode(ZeroTrustBusinessRespEnum.SUCCESS.getValue());
        zeroTrustMessageRespVO.setMessage(ZeroTrustBusinessRespEnum.SUCCESS.getLabel());
        zeroTrustMessageRespVO.setCheckSum(generalCheckSum);
        return zeroTrustMessageRespVO;
    }
}
