package com.dragonsoft.dcuc.approve.filter;

import cn.hutool.core.util.StrUtil;
import com.dragoninfo.dcuc.common.entity.Proofs;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.business.ISecurityUserBusiness;
import com.dragonsoft.dcuc.approve.properties.ApproveProperties;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.core.spring.SpringUtils;
import com.dragonsoft.duceap.web.utils.ResponseUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/8/18
 */
@Slf4j
public class CheckProofsFilter extends OncePerRequestFilter {

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        ApproveProperties approveProperties = SpringUtils.getBean(ApproveProperties.class);

        String defaultAppCode = approveProperties.getAppCode();
        Proofs proofs = ProofsUtil.generalProofs(request, defaultAppCode);

        String idcard = proofs.getIdcard();
        String appCode = proofs.getAppCode();

        if (!(StrUtil.isNotBlank(idcard) && StrUtil.isNotBlank(appCode))) {
            ResponseStatus responseStatus = ResponseStatus.fail("请传入凭据");
            ResponseUtils.outJson(responseStatus, response);
            return;
        }
        ISecurityUserBusiness securityUserBusiness = SpringUtils.getBean(ISecurityUserBusiness.class);

        ResponseDTO<SecurityUser> securityUserResp = securityUserBusiness.getSecurityUser(idcard, appCode);
        if (ResponseUtil.isFail(securityUserResp)) {
            ResponseStatus responseStatus = ResponseStatus.fail(securityUserResp.getMessage());
            ResponseUtils.outJson(responseStatus, response);
            return;
        }
        SecurityUser securityUser = ResponseUtil.getResult(securityUserResp);

        securityUserBusiness.setUserExtInfo(securityUser, request);

        UserContextUtils.setCurrentUser(securityUser);

        filterChain.doFilter(request, response);
    }

}
