package com.dragonsoft.dcuc.approve.filter.cache;

import com.dragoninfo.dcuc.common.entity.SecurityApp;
import com.dragonsoft.dcuc.approve.properties.ProofStarterProperties;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;

import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 应用缓存
 * </p>
 *
 * @author huangzqa
 * @date 2021/9/3
 */
public class SecurityAppCache {

    private final Cache<String, SecurityApp> caffeineCache;

    public SecurityAppCache(ProofStarterProperties proofStarterProperties) {
        ProofStarterProperties.AppCache appCache = proofStarterProperties.getAppCache();
        this.caffeineCache = Caffeine.newBuilder()
                .expireAfterWrite(appCache.getExpireAfterWrite(), TimeUnit.SECONDS)
                .initialCapacity(appCache.getInitialCapacity())
                .maximumSize(appCache.getMaximumSize())
                .build();
    }

    private String cacheKey(String... names) {
        return "cacheAppKey_" + StringUtils.join(names, "&");
    }

    public SecurityApp getIfPresent(String... key) {
        String cacheKey = cacheKey(key);
        return this.caffeineCache.getIfPresent(cacheKey);
    }

    public void put(String key, SecurityApp value) {
        String cacheKey = cacheKey(key);
        this.caffeineCache.put(cacheKey, value);
    }

    public void remove(String... keys) {
        String cacheKey = cacheKey(keys);
        this.caffeineCache.invalidate(cacheKey);
    }
}
