package com.dragonsoft.dcuc.approve.filter.cache;

import com.dragonsoft.dcuc.approve.properties.ProofStarterProperties;
import com.dragonsoft.duceap.base.entity.security.SecurityUser;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;

import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 用户信息缓存
 * </p>
 *
 * @author huangzqa
 * @date 2021/9/3
 */
public class SecurityUserCache {

    private final Cache<String, SecurityUser> caffeineCache;

    public SecurityUserCache(ProofStarterProperties proofStarterProperties) {
        ProofStarterProperties.UserCache userCache = proofStarterProperties.getUserCache();
        this.caffeineCache = Caffeine.newBuilder()
                .expireAfterWrite(userCache.getExpireAfterWrite(), TimeUnit.SECONDS)
                .initialCapacity(userCache.getInitialCapacity())
                .maximumSize(userCache.getMaximumSize())
                .build();
    }

    private String cacheKey(String... names) {
        return "cacheUserKey_" + StringUtils.join(names, "&");
    }

    public SecurityUser getIfPresent(String... key) {
        String cacheKey = cacheKey(key);
        return this.caffeineCache.getIfPresent(cacheKey);
    }

    public void put(String key, SecurityUser value) {
        String cacheKey = cacheKey(key);
        this.caffeineCache.put(cacheKey, value);
    }

    public void remove(String... keys) {
        String cacheKey = cacheKey(keys);
        this.caffeineCache.invalidate(cacheKey);
    }
}
