package com.dragonsoft.dcuc.approve.model.vo.dcuc;

import com.dragonsoft.dcuc.approve.model.ApproveOrgInfo;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.commons.util.date.DateConst;
import com.dragonsoft.duceap.commons.util.date.DateUtils;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.SneakyThrows;

import java.io.Serializable;
import java.text.ParseException;
import java.util.Date;

/**
 * 接口信息
 *
 * @author huangzqa
 * @date 2021/1/21
 **/
@ApiModel(value = "机构信息")
@Data
public class ApiOrgResultVo implements Serializable {

    @ApiModelProperty(value = "机构代码")
    private String orgCode;

    @ApiModelProperty(value = "上级机构代码")
    private String upOrgCode;

    @ApiModelProperty(value = "机构类型 1:正式 2：临时")
    private String orgKind;

    @ApiModelProperty(value = "机构全称")
    private String fullName;

    @ApiModelProperty(value = "机构简称")
    private String shortName;

    @ApiModelProperty(value = "行政区划")
    private String areaCode;

    @ApiModelProperty(value = "标准代码")
    private String standCode;

    @ApiModelProperty(value = "机构类别 1，行政单位，2 业务单位")
    private String orgType;

    @ApiModelProperty(value = "单位层级；1：部级单位 2：厅级单位 3：局级单位 4：处级单位 5：科级单位 6：股级单位")
    private String orgLevel;

    @ApiModelProperty(value = "单位级别；01：正部级单位 02：副部级单位 11：正厅级单位 12：副厅级单位 21:正处级单位 22:副处级单位 31：正科级单位 32：副科级单位 41：正股级单位 42：副股级单位")
    private String orgRank;

    @ApiModelProperty(value = "单位类别")
    private String unitClass;

    @ApiModelProperty(value = "邮箱地址")
    private String email;

    @ApiModelProperty(value = "联系人")
    private String linkMan;

    @ApiModelProperty(value = "电话")
    private String phone;

    @ApiModelProperty(value = "更新时间")
    private String updateTime;

    @ApiModelProperty(value = "删除标识")
    private String deleted;

    @SneakyThrows(ParseException.class)
    public ApproveOrgInfo toApproveOrgInfo() {
        ApproveOrgInfo approveOrgInfo = new ApproveOrgInfo();
        approveOrgInfo.setOrgCode(this.orgCode);
        approveOrgInfo.setOrgName(this.fullName);
        approveOrgInfo.setUpOrgCode(this.upOrgCode);
        approveOrgInfo.setOrgKind(this.orgKind);
        approveOrgInfo.setAreaCode(this.areaCode);

        Date updateTimeDate = DateUtils.getDate(this.updateTime, DateConst.HYPHEN_DISPLAY_TIME);

        approveOrgInfo.setRemoteUpdateTime(updateTimeDate);
        approveOrgInfo.setDeleted(BooleanEnum.FALSE.value);
        Date date = DateUtils.getDate();
        approveOrgInfo.setCreateTime(date);
        approveOrgInfo.setUpdateTime(date);

        return approveOrgInfo;
    }


}
