package com.dragonsoft.dcuc.approve.service;

import com.dragonsoft.dcuc.approve.model.ApproveActRunTask;
import com.dragonsoft.dcuc.approve.model.resp.ProcessStatusVo;
import com.dragonsoft.dcuc.approve.model.resp.TaskVo;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.data.domain.Page;

import java.util.List;

/**
 * @author wangrs
 * @date 2021/03/23
 */
public interface IApproveActRunTaskService {

    /**
     * 保存
     *
     * @param approveActRunTask 待办
     */
    void save(ApproveActRunTask approveActRunTask);

    /**
     * 根据审批单号删除待办
     *
     * @param applyRecordId 审批单号
     */
    void deleteByApplyRecordId(String applyRecordId);

    /**
     * 根据ID删除
     *
     * @param id 待办
     */
    void deleteById(String id);

    /**
     * 根据任务ID查询待办
     *
     * @param taskId 任务ID
     * @return 待办
     */
    ApproveActRunTask selectOneByTaskId(String taskId);

    /**
     * 查询待办事宜
     *
     * @param searchable 查询条件
     * @return 分页
     */
    Page<ApproveActRunTask> page(Searchable searchable);

    /**
     * 总条数
     *
     * @param countSearchable 条件
     * @return 条数
     */
    Integer totalCounts(Searchable countSearchable);

    /**
     * 根据查询条件获取待办列表
     *
     * @param searchable 查询条件
     * @return 待办列表
     */
    List<ApproveActRunTask> idListBySearchable(Searchable searchable);

    /**
     * 根据审批记录ID查询
     *
     * @param approveRecordId 审批记录ID
     * @return 待办列表
     */
    List<ApproveActRunTask> findByApproveRecordId(String approveRecordId);

    /**
     * 生成待办状态
     *
     * @param approveRecordId 审批记录ID
     * @return 待办状态
     */
    List<ProcessStatusVo> buildProcessStatusVo(String approveRecordId);


    /**
     * 构建待办列表
     *
     * @param searchable 查询条件
     * @return 分页
     */
    Page<TaskVo> buildApproveActRunTask(Searchable searchable);

}
