package com.dragonsoft.dcuc.approve.service;

import com.dragonsoft.dcuc.approve.model.dto.ApprovalInfoDTO;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.resp.ApprovalInfoRespVO;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.data.domain.Page;

import java.util.Date;
import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2021/7/1
 */
public interface IApproveRecordService {

    /**
     * 保存
     *
     * @param approveRecordInfo 审批信息
     */
    void save(ApproveRecordInfo approveRecordInfo);

    /**
     * 根据ID删除
     *
     * @param id id
     */
    void deleteById(String id);

    /**
     * 根据id更新
     *
     * @param approveRecordInfo 审批信息
     */
    void updateById(ApproveRecordInfo approveRecordInfo);

    /**
     * 获取审批信息
     *
     * @param approveRecordId 审批记录单号
     * @return 审批信息
     */
    ApproveRecordInfo selectOne(String approveRecordId);

    /**
     * 根据申请单号查询审批单信息
     *
     * @param approveNo 审批单号
     * @return 审批信息
     */
    ApproveRecordInfo selectByApproveNo(String approveNo);

    /**
     * 根据实例id、应用代码 获取审批信息
     *
     * @param processInstanceId 实例id
     * @return 审批信息
     */
    ApproveRecordInfo findByProcessInstanceId(String processInstanceId);

    /**
     * 分页
     *
     * @param searchable 分页条件
     * @return 分页结果
     */
    Page<ApproveRecordInfo> paging(Searchable searchable);

    /**
     * 获取用户请求数量
     *
     * @param userId 用户ID
     * @return 数量
     */
    Integer getUserRequestCount(String userId);

    /**
     * 获取最新的审批信息
     *
     * @param date 时间
     * @return 审批信息
     */
    ApproveRecordInfo findByCreateTime(Date date);

    /**
     * 将旧表单表对象构建成模式3新表对象
     *
     * @param approvalInfoDTO 审批信息
     * @param approveNo    审批单号
     * @return 审批记录信息
     */
    ApproveRecordInfo buildApproveRecordInfo(ApprovalInfoDTO approvalInfoDTO, String approveNo);

    /**
     * 我的请求列表
     *
     * @param searchable 搜索条件
     * @return 分页
     */
    Page<ApprovalInfoRespVO> queryUserApproveList(Searchable searchable);

    /**
     * 根据任务Id获取审批信息
     *
     * @param taskId 任务ID
     * @return 审批信息
     */
    ApproveRecordInfo getOneByTaskId(String taskId);

    /**
     * 根据身份证号应用代码获取对应审批单信息
     *
     * @param idcard  身份证号
     * @param appCode 应用代码
     * @return 审批单信息
     */
    List<ApproveRecordInfo> getRecordByIdcardAppCode(String idcard, String appCode);

    /**
     * 获取taskNo
     *
     * @param appCode     应用代码
     * @param dateTimeStr 日期时间
     * @return taskNo
     */
    String getTaskNoByDate(String appCode, String dateTimeStr);

    /**
     * 获取超期的审批
     *
     * @param startExpirationDateTime 开始超期的时间
     * @return 超期的审批单
     */
    List<ApproveRecordInfo> findExpirationTimeRecord(Date startExpirationDateTime);

    /**
     * 更新同步标志位
     *
     * @param id          审批记录ID
     * @param booleanEnum 是否成功
     */
    void updateSyncFlag(String id, BooleanEnum booleanEnum);

    /**
     * 获取未同步列表
     *
     * @return 未同步列表
     */
    List<ApproveRecordInfo> getUnSyncList();
}
