package com.dragonsoft.dcuc.approve.service.impl;

import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.dragonsoft.dcuc.approve.business.IApproveBusiness;
import com.dragonsoft.dcuc.approve.mapper.ApproveActHiTaskMapper;
import com.dragonsoft.dcuc.approve.model.ApproveActHiTask;
import com.dragonsoft.dcuc.approve.model.ApproveRecordInfo;
import com.dragonsoft.dcuc.approve.model.resp.TaskVo;
import com.dragonsoft.dcuc.approve.service.IApproveActHiTaskService;
import com.dragonsoft.dcuc.approve.service.IApproveRecordService;
import com.dragonsoft.dcuc.approve.utils.ProofsUtil;
import com.dragonsoft.duceap.base.enums.BooleanEnum;
import com.dragonsoft.duceap.core.entity.page.PageImpl;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.enums.SearchOperator;
import com.dragonsoft.duceap.core.search.filter.Condition;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @author wangrs huangzqa
 * @date 2021-03-23
 */
@Service
public class ApproveActHiTaskServiceImpl implements IApproveActHiTaskService {

    private ApproveActHiTaskMapper approveActHiTaskMapper;

    private IApproveBusiness approveBusiness;

    private IApproveRecordService approveRecordService;

    @Autowired
    public void setApproveActHiTaskMapper(ApproveActHiTaskMapper approveActHiTaskMapper) {
        this.approveActHiTaskMapper = approveActHiTaskMapper;
    }

    @Autowired
    public void setApproveBusiness(IApproveBusiness approveBusiness) {
        this.approveBusiness = approveBusiness;
    }

    @Autowired
    public void setApproveRecordService(IApproveRecordService approveRecordService) {
        this.approveRecordService = approveRecordService;
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void save(ApproveActHiTask approveActHiTask) {
        approveActHiTaskMapper.insert(approveActHiTask);
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void update(ApproveActHiTask approveActHiTask) {
        approveActHiTaskMapper.updateById(approveActHiTask);
    }

    @Override
    public Page<ApproveActHiTask> page(Searchable searchable) {
        return approveActHiTaskMapper.pagingBySearchable(searchable);
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void deleteByApplyRecordId(String applyRecordId) {
        //删除所有已办
        LambdaUpdateWrapper<ApproveActHiTask> hiTaskLambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        hiTaskLambdaUpdateWrapper.eq(ApproveActHiTask::getApplyRecordId, applyRecordId);
        approveActHiTaskMapper.delete(hiTaskLambdaUpdateWrapper);
    }

    /**
     * 构建已办列表
     *
     * @param searchable 查询条件
     * @return 分页
     */
    @Override
    public Page<TaskVo> buildApproveActHiTask(Searchable searchable) {
        searchable.addSearchFilter("operateUserIdcard", SearchOperator.eq, ProofsUtil.getIdcard());
        searchable.addSort(Sort.Direction.DESC, "operateTime");
        Condition processDefinitionNameCondition = searchable.getSearchFilter("processDefinitionName", SearchOperator.like);

        if (processDefinitionNameCondition != null) {
            searchable.addSearchFilter("businessName", SearchOperator.like, processDefinitionNameCondition.getValue());
            searchable.removeSearchFilter("processDefinitionName", SearchOperator.like);
        }
        searchable.addSearchFilter("deleted", SearchOperator.eq, BooleanEnum.FALSE.getValue());
        //获取已办列表
        Page<ApproveActHiTask> page = approveActHiTaskMapper.pagingBySearchable(searchable);
        List<TaskVo> list = new ArrayList<>();
        for (ApproveActHiTask actHiTask : page.getContent()) {
            String applyRecordId = actHiTask.getApplyRecordId();
            TaskVo taskVo = new TaskVo();
            BeanUtils.copyProperties(actHiTask, taskVo);
            //获取表单信息对象
            ApproveRecordInfo approveRecordInfo = approveRecordService.selectOne(applyRecordId);

            if (approveRecordInfo != null) {
                //设置流程名称
                taskVo.setProcessName(approveRecordInfo.getProcessName());
                //设置流程标题
                taskVo.setProcessTitle(approveRecordInfo.getProcessTitle());
                taskVo.setApplicantName(approveRecordInfo.getApplicantName());
                taskVo.setProcessType(approveRecordInfo.getProcessType());
                taskVo.setBusinessId(approveRecordInfo.getId());
                taskVo.setStartUserName(approveRecordInfo.getApplicantName());
                taskVo.setStartUserId(approveRecordInfo.getApplicantIdcard());
                //设置当前待办任务处理人
                String unOperateUsers = approveBusiness.getUnOperateUsers(approveRecordInfo.getTaskId());
                taskVo.setUnOperateUsers(unOperateUsers);
                taskVo.setId(approveRecordInfo.getTaskId());
                taskVo.setTaskName(approveRecordInfo.getTaskName());
                taskVo.setProcessTypeName(approveRecordInfo.getProcessTypeName());
                taskVo.setAppCode(approveRecordInfo.getAppCode());
                taskVo.setApproveNo(approveRecordInfo.getApproveNo());
            }
            taskVo.setCreateTime(actHiTask.getOperateTime());
            taskVo.setEndTime(actHiTask.getOperateTime());
            taskVo.setProcessInstanceId(actHiTask.getProcessInstanceId());
            list.add(taskVo);
        }
        return new PageImpl<>(list, page.getPageable(), page.getTotalElements());
    }

    @Transactional(rollbackFor = Throwable.class)
    @Override
    public void deleteAndSaveList(String approveId, List<ApproveActHiTask> approveActHiTaskList) {
        deleteByApplyRecordId(approveId);
        for (ApproveActHiTask approveActHiTask : approveActHiTaskList) {
            this.save(approveActHiTask);
        }
    }

}
