package com.dragonsoft.dcuc.approve.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dragoninfo.dcuc.common.utils.DcucBeanUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.mapper.ApproveOrgInfoMapper;
import com.dragonsoft.dcuc.approve.model.ApproveOrgInfo;
import com.dragonsoft.dcuc.approve.model.resp.ApproveOrgInfoRespVO;
import com.dragonsoft.dcuc.approve.service.IApproveOrgInfoService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.Date;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/12
 */
@Service
public class ApproveOrgInfoServiceImpl extends ServiceImpl<ApproveOrgInfoMapper, ApproveOrgInfo> implements IApproveOrgInfoService {

    private ApproveOrgInfoMapper approveOrgInfoMapper;

    @Autowired
    public void setApproveOrgInfoMapper(ApproveOrgInfoMapper approveOrgInfoMapper) {
        this.approveOrgInfoMapper = approveOrgInfoMapper;
    }

    @Override
    public Page<ApproveOrgInfoRespVO> page(Searchable searchable) {
        Page<ApproveOrgInfo> page = approveOrgInfoMapper.pagingBySearchable(searchable);
        return DcucBeanUtil.createCopyToObjectPage(page, ApproveOrgInfoRespVO.class);
    }

    @Override
    public ResponseDTO<ApproveOrgInfoRespVO> detail(String id) {
        ApproveOrgInfo approveUserInfo = approveOrgInfoMapper.selectById(id);
        if (ObjectUtil.isNotNull(approveUserInfo)) {
            return ResponseUtil.dtoFail("机构不存在");
        }
        ApproveOrgInfoRespVO approveUserInfoRespVO = DcucBeanUtil.createCopyToObject(approveUserInfo, ApproveOrgInfoRespVO.class);
        return ResponseUtil.newInstance(approveUserInfoRespVO);
    }

    @Override
    public ApproveOrgInfo selectByOrgCode(String orgCode) {
        LambdaQueryWrapper<ApproveOrgInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(ApproveOrgInfo::getOrgCode, orgCode)
                .select(ApproveOrgInfo::getId, ApproveOrgInfo::getCreateTime);
        return approveOrgInfoMapper.selectOne(lambdaQueryWrapper);
    }

    @Transactional(
            rollbackFor = {Exception.class}
    )
    @Override
    public void saveAllList(Collection<ApproveOrgInfo> saveList) {
        if (CollUtil.isEmpty(saveList)) {
            return;
        }
        this.saveBatch(saveList);
    }

    @Override
    public void updateAllList(Collection<ApproveOrgInfo> updateList) {
        if (CollUtil.isEmpty(updateList)) {
            return;
        }
        for (ApproveOrgInfo approveUserInfo : updateList) {
            approveOrgInfoMapper.updateById(approveUserInfo);
        }
    }

    @Override
    public Date getLastUpdateTime() {
        LambdaQueryWrapper<ApproveOrgInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.orderByDesc(ApproveOrgInfo::getUpdateTime)
                .last("LIMIT 1");
        ApproveOrgInfo approveUserInfo = approveOrgInfoMapper.selectOne(lambdaQueryWrapper);
        if (approveUserInfo != null) {
            return approveUserInfo.getUpdateTime();
        }
        return new Date();
    }

    @Override
    public void updateUpOrgId(Date updateTime) {
        approveOrgInfoMapper.updateUpOrgId(updateTime);
    }
}
