package com.dragonsoft.dcuc.approve.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dragoninfo.dcuc.common.utils.DcucBeanUtil;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragonsoft.dcuc.approve.mapper.ApproveUserInfoMapper;
import com.dragonsoft.dcuc.approve.model.ApproveUserInfo;
import com.dragonsoft.dcuc.approve.model.resp.ApproveUserInfoRespVO;
import com.dragonsoft.dcuc.approve.service.IApproveUserInfoService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 *
 * </p>
 *
 * @author huangzqa
 * @date 2023/7/12
 */
@Service
public class ApproveUserInfoServiceImpl extends ServiceImpl<ApproveUserInfoMapper, ApproveUserInfo> implements IApproveUserInfoService {

    private ApproveUserInfoMapper approveUserInfoMapper;

    @Autowired
    public void setApproveUserInfoMapper(ApproveUserInfoMapper approveUserInfoMapper) {
        this.approveUserInfoMapper = approveUserInfoMapper;
    }

    @Override
    public Page<ApproveUserInfoRespVO> page(Searchable searchable) {
        Page<ApproveUserInfo> page = approveUserInfoMapper.pagingBySearchable(searchable);
        return DcucBeanUtil.createCopyToObjectPage(page, ApproveUserInfoRespVO.class);
    }

    @Override
    public ResponseDTO<ApproveUserInfoRespVO> detail(String id) {
        ApproveUserInfo approveUserInfo = approveUserInfoMapper.selectById(id);
        if (ObjectUtil.isNotNull(approveUserInfo)) {
            return ResponseUtil.dtoFail("用户不存在");
        }
        ApproveUserInfoRespVO approveUserInfoRespVO = DcucBeanUtil.createCopyToObject(approveUserInfo, ApproveUserInfoRespVO.class);
        return ResponseUtil.newInstance(approveUserInfoRespVO);
    }

    @Override
    public ApproveUserInfo selectByIdcard(String idcard) {
        LambdaQueryWrapper<ApproveUserInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(ApproveUserInfo::getIdcard, idcard)
                .select(ApproveUserInfo::getId, ApproveUserInfo::getCreateTime);
        return approveUserInfoMapper.selectOne(lambdaQueryWrapper);
    }

    @Transactional(
            rollbackFor = {Exception.class}
    )
    @Override
    public void saveAllList(Collection<ApproveUserInfo> saveList) {
        if (CollUtil.isEmpty(saveList)) {
            return;
        }

        Map<String, List<ApproveUserInfo>> stringApproveUserInfoMap = saveList.stream().collect(Collectors.groupingBy(ApproveUserInfo::getIdcard));
        List<ApproveUserInfo> distantAddList = new ArrayList<>(stringApproveUserInfoMap.size());
        for (String idcard : stringApproveUserInfoMap.keySet()) {
            List<ApproveUserInfo> approveUserInfoList = stringApproveUserInfoMap.get(idcard);
            distantAddList.add(approveUserInfoList.get(0));
        }
        this.saveBatch(distantAddList);
    }

    @Override
    public void updateAllList(Collection<ApproveUserInfo> updateList) {
        if (CollUtil.isEmpty(updateList)) {
            return;
        }
        for (ApproveUserInfo approveUserInfo : updateList) {
            approveUserInfoMapper.updateById(approveUserInfo);
        }
    }

    @Override
    public void deleteByIdcard(String idcard) {
        LambdaQueryWrapper<ApproveUserInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(ApproveUserInfo::getIdcard, idcard);
        approveUserInfoMapper.delete(lambdaQueryWrapper);
    }

    @Override
    public Date getLastUpdateTime() {
        LambdaQueryWrapper<ApproveUserInfo> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.orderByDesc(ApproveUserInfo::getUpdateTime)
                .last("LIMIT 1");
        ApproveUserInfo approveUserInfo = approveUserInfoMapper.selectOne(lambdaQueryWrapper);
        if (approveUserInfo != null) {
            return approveUserInfo.getUpdateTime();
        }
        return new Date();
    }
}
