package com.dragonsoft.dcuc.approve.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.duceap.enums.RequestDisposeTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.ApproveResultEnum;
import com.dragonsoft.dcuc.approve.mapper.RequestLogMapper;
import com.dragonsoft.dcuc.approve.model.RequestLog;
import com.dragonsoft.dcuc.approve.model.resp.RequestLogVo;
import com.dragonsoft.dcuc.approve.service.IRequestLogService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.core.entity.page.PageImpl;
import com.dragonsoft.duceap.core.search.Searchable;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/5/27 19:43
 */
@Service
public class RequestLogServiceImpl implements IRequestLogService {

    @Autowired
    private RequestLogMapper requestLogMapper;

    @Override
    public RequestLog selectOneByApproveId(String approveId) {
        LambdaQueryWrapper<RequestLog> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(RequestLog::getApproveRecordId, approveId);
        lambdaQueryWrapper.last(" LIMIT 1");

        return requestLogMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public Page<RequestLogVo> queryRequestLogList(Searchable searchable) {
        searchable.addSort(Sort.Direction.DESC, "createTime");
        Page<RequestLog> logPage = requestLogMapper.pagingBySearchable(searchable);
        List<RequestLogVo> logVoList = new ArrayList<>();
        for (RequestLog log : logPage.getContent()) {
            RequestLogVo logVo = new RequestLogVo();
            BeanUtils.copyProperties(log, logVo);
            logVo.setResultStr(EnumUtils.enumOf(ApproveResultEnum.class, log.getResult()).getLabel());
            logVo.setDisposeTypeStr(EnumUtils.enumOf(RequestDisposeTypeEnum.class, log.getDisposeType()).getLabel());
            logVoList.add(logVo);
        }
        return new PageImpl<>(logVoList, logPage.getPageable(), logPage.getTotalElements());
    }

    @Override
    public ResponseDTO<RequestLogVo> requestLogDetail(String id) {
        RequestLog log = requestLogMapper.selectById(id);
        RequestLogVo logVo = new RequestLogVo();
        BeanUtil.copyProperties(log, logVo);
        logVo.setResultStr(EnumUtils.enumOf(ApproveResultEnum.class, log.getResult()).getLabel());
        logVo.setDisposeTypeStr(EnumUtils.enumOf(RequestDisposeTypeEnum.class, log.getDisposeType()).getLabel());
        return ResponseUtil.newInstance(logVo);
    }

    @Override
    public void saveRequestLog(RequestLog entity) {
        requestLogMapper.insert(entity);
    }
}
