package com.dragonsoft.dcuc.approve.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.common.utils.ResponseUtil;
import com.dragoninfo.dcuc.duceap.enums.SchemeTypeEnum;
import com.dragonsoft.dcuc.approve.enumresources.LogReceiverEnum;
import com.dragonsoft.dcuc.approve.mapper.RiskLogMapper;
import com.dragonsoft.dcuc.approve.model.RiskLog;
import com.dragonsoft.dcuc.approve.model.resp.RiskLogVo;
import com.dragonsoft.dcuc.approve.service.IRiskLogService;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.commons.util.enums.EnumUtils;
import com.dragonsoft.duceap.core.entity.page.PageImpl;
import com.dragonsoft.duceap.core.search.Searchable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * 代码千万行,注释第一行,编码不规范,同事两行泪
 *
 * @author huang(jy)
 * @version 1.0
 * @date 2021/5/27 19:43
 */
@Service
public class RiskLogServiceImpl implements IRiskLogService {

    private static final Logger logger = LoggerFactory.getLogger(RiskLogServiceImpl.class);

    @Autowired
    private RiskLogMapper riskLogMapper;

    @Autowired
    private IApplyInfoFacade applyInfoFacade;

    @Override
    public Page<RiskLogVo> queryRequestLogList(Searchable searchable) {
        searchable.addSort(Sort.Direction.DESC, "createTime");
        Page<RiskLog> logPage = riskLogMapper.pagingBySearchable(searchable);
        List<RiskLogVo> logVoList = new ArrayList<>();
        for (RiskLog log : logPage.getContent()) {
            RiskLogVo logVo = new RiskLogVo();
            BeanUtils.copyProperties(log, logVo);
            logVo.setRiskTypeStr(EnumUtils.enumOf(SchemeTypeEnum.class, log.getRiskType()).getLabel());
            try {
                ApplyInfo applyInfo = applyInfoFacade.getAppByCode(log.getLogSender());
                if (applyInfo != null) {
                    logVo.setLogSenderStr(applyInfo.getApplyName());
                }

                String[] arr = logVo.getLogReceiver().split(",");
                StringBuilder logReceiverStr = new StringBuilder();
                for (String s : arr) {
                    LogReceiverEnum receiverEnum = EnumUtils.enumOf(LogReceiverEnum.class, s);
                    logReceiverStr.append(receiverEnum.getLabel()).append(",");
                }
                logVo.setLogReceiverStr(logReceiverStr.substring(0, logReceiverStr.length() - 1));
            } catch (Exception e) {
                logger.error("日志发送方、接收方转换异常", e);
            }
            logVoList.add(logVo);
        }
        return new PageImpl<>(logVoList, logPage.getPageable(), logPage.getTotalElements());
    }

    @Override
    public ResponseDTO<RiskLogVo> requestLogDetail(String id) {
        RiskLog log = riskLogMapper.selectById(id);
        RiskLogVo logVo = new RiskLogVo();
        BeanUtil.copyProperties(log, logVo);
        logVo.setRiskTypeStr(EnumUtils.enumOf(SchemeTypeEnum.class, log.getRiskType()).getLabel());
        return ResponseUtil.newInstance(logVo);
    }

    @Override
    public Integer selectCountBySchemeType(String schemeType) {
        LambdaQueryWrapper<RiskLog> where = Wrappers.lambdaQuery();
        where.eq(RiskLog::getRiskType, schemeType);
        return riskLogMapper.selectCount(where);
    }

    @Override
    public void saveRiskLog(RiskLog riskLog) {
        riskLogMapper.insert(riskLog);
    }
}
