ALTER TABLE `t_task_assign_record`
    ADD COLUMN `approve_record_id` varchar(32) NOT NULL COMMENT '申请单id' AFTER `id`;


-- 审批单信息表(接口文档)
CREATE TABLE `t_approve_record_info`
(
    `id`                    VARCHAR(32)   NOT NULL COMMENT '主键',
    `process_no`            VARCHAR(32)   NULL DEFAULT NULL COMMENT '流程编号',
    `process_name`          VARCHAR(100)  NULL DEFAULT NULL COMMENT '流程名称',
    `process_type`          VARCHAR(100)  NULL DEFAULT NULL COMMENT '流程类型',
    `process_type_name`     VARCHAR(100)  NULL DEFAULT NULL COMMENT '流程类型名称',
    `approve_type`          VARCHAR(2)    NULL DEFAULT NULL COMMENT '审批类型',
    `approve_no`            VARCHAR(64)   NULL DEFAULT NULL COMMENT '审批单编号',
    `process_title`         VARCHAR(100)  NULL DEFAULT NULL COMMENT '审批单标题',
    `applicant_idcard`      VARCHAR(18)   NULL DEFAULT NULL COMMENT '申请人身份证号码',
    `applicant_name`        VARCHAR(100)  NULL DEFAULT NULL COMMENT '申请人姓名',
    `applicant_phone_no`    VARCHAR(20)   NULL DEFAULT NULL COMMENT '申请人手机号码',
    `applicant_org_code`    VARCHAR(32)   NULL DEFAULT NULL COMMENT '申请人单位代码',
    `applicant_org_name`    VARCHAR(200)  NULL DEFAULT NULL COMMENT '申请人单位名称',
    `apply_time`            VARCHAR(30)   NULL DEFAULT NULL COMMENT '申请时间',
    `app_name`              VARCHAR(100)  NULL DEFAULT NULL COMMENT '应用名称',
    `app_code`              VARCHAR(32)   NULL DEFAULT NULL COMMENT '应用编号',
    `approve_content`       VARCHAR(2000) NULL DEFAULT NULL COMMENT '审批内容',
    `access_resources`      VARCHAR(2000) NULL DEFAULT NULL COMMENT '访问资源',
    `access_resources_no`   VARCHAR(2000) NULL DEFAULT NULL COMMENT '访问资源编号',
    `valid_period_time`     VARCHAR(30)   NULL DEFAULT NULL COMMENT '有效期',
    `permission_valid_type` varchar(2)         DEFAULT '02' COMMENT '有效期类别（自定义：01；长期：02；）',
    `start_time`            datetime      NULL DEFAULT NULL COMMENT '有效开始时间',
    `end_time`              datetime      NULL DEFAULT NULL COMMENT '有效结束时间',
    `apply_reason`          VARCHAR(300)  NULL DEFAULT NULL COMMENT '申请事由',
    `final_approval_result` VARCHAR(2)    NULL DEFAULT NULL COMMENT '最终审批结果,详见ApproveResultEnum枚举',
    `attachment_url`        VARCHAR(100)  NULL DEFAULT NULL COMMENT '附件地址',
    `message_id`            VARCHAR(100)  NULL DEFAULT NULL COMMENT '第三方消息id',
    `process_instance_id`   VARCHAR(100)  NULL DEFAULT NULL COMMENT '实例id',
    `task_code`             VARCHAR(100)  NULL DEFAULT NULL COMMENT '任务标识码',
    `task_id`               VARCHAR(100)  NULL DEFAULT NULL COMMENT '任务id',
    `task_name`             VARCHAR(100)  NULL DEFAULT NULL COMMENT '任务名称',
    `other_apply_context`   VARCHAR(2000) NULL DEFAULT NULL COMMENT '其他申请信息',
    `is_deleted`            CHAR(1)            DEFAULT NULL COMMENT '是否删除',
    `create_user`           VARCHAR(32)        DEFAULT NULL COMMENT '创建人',
    `create_time`           datetime      NULL DEFAULT NULL COMMENT '创建时间',
    `update_user`           VARCHAR(32)        DEFAULT NULL COMMENT '更新人',
    `update_time`           datetime      NULL DEFAULT NULL COMMENT '更新时间',
    `delete_user`           VARCHAR(32)        DEFAULT NULL COMMENT '删除人',
    `delete_time`           datetime      NULL DEFAULT NULL COMMENT '删除时间',
    PRIMARY KEY (`id`)
) ENGINE = INNODB
  DEFAULT CHARSET = utf8 COMMENT ='审批单信息表';

-- 审批单关联查询记录表
CREATE TABLE `t_approve_query_relation`
(
    `id`                VARCHAR(32)  NOT NULL COMMENT '主键',
    `approve_record_id` VARCHAR(32)  NULL DEFAULT NULL COMMENT '审批单信息记录id',
    `query_user_type`   VARCHAR(20)  NULL DEFAULT NULL COMMENT '案件/事件/线索/被查询人员类别',
    `query_user_name`   VARCHAR(100) NULL DEFAULT NULL COMMENT '案件/事件/线索/被查询人员名称',
    `query_user_id`     VARCHAR(32)  NULL DEFAULT NULL COMMENT '案件/事件/线索/被查询人员编号',
    `is_deleted`        CHAR(1)           DEFAULT NULL COMMENT '是否删除',
    `create_user`       VARCHAR(32)       DEFAULT NULL COMMENT '创建人',
    `create_time`       datetime     NULL DEFAULT NULL COMMENT '创建时间',
    `update_user`       VARCHAR(32)       DEFAULT NULL COMMENT '更新人',
    `update_time`       datetime     NULL DEFAULT NULL COMMENT '更新时间',
    `delete_user`       VARCHAR(32)       DEFAULT NULL COMMENT '删除人',
    `delete_time`       datetime     NULL DEFAULT NULL COMMENT '删除时间',
    PRIMARY KEY (`id`)
) ENGINE = INNODB
  DEFAULT CHARSET = utf8 COMMENT ='审批单信息关联表';


-- 审批流转记录表(接口文档)
CREATE TABLE `t_approve_circulation_info`
(
    `id`                     VARCHAR(32)   NOT NULL COMMENT '主键',
    `approve_record_id`      VARCHAR(32)   NULL DEFAULT NULL COMMENT '审批单信息记录id',
    `approve_no`             VARCHAR(64)   NULL DEFAULT NULL COMMENT '审批单编号',
    `process_no`             VARCHAR(32)   NULL DEFAULT NULL COMMENT '流程编号',
    `node_name`              VARCHAR(100)  NULL DEFAULT NULL COMMENT '节点名称',
    `node_no`                VARCHAR(32)   NULL DEFAULT NULL COMMENT '节点标识码',
    `task_key`               VARCHAR(100)  NULL DEFAULT NULL COMMENT '节点定义id',
    `task_id`                VARCHAR(100)  NULL DEFAULT NULL COMMENT '任务id',
    `operate_type`           VARCHAR(50)   NULL DEFAULT NULL COMMENT '操作类型,详见ApproveWorkFlowOprationEnum枚举',
    `operate_time`           VARCHAR(30)   NULL DEFAULT NULL COMMENT '操作时间',
    `operate_user_name`      VARCHAR(100)  NULL DEFAULT NULL COMMENT '操作人姓名',
    `operate_user_id`        VARCHAR(18)   NULL DEFAULT NULL COMMENT '操作人身份证号码',
    `operate_user_org`       VARCHAR(200)  NULL DEFAULT NULL COMMENT '操作人单位名称',
    `operate_user_org_id`    VARCHAR(32)   NULL DEFAULT NULL COMMENT '操作人单位编号',
    `approve_opinion`        VARCHAR(1000) NULL DEFAULT NULL COMMENT '审批意见',
    `operate_time_consuming` VARCHAR(50)   NULL DEFAULT NULL COMMENT '操作耗时',
    `electronic_signature`   VARCHAR(100)  NULL DEFAULT NULL COMMENT '电子签名',
    `is_deleted`             CHAR(1)            DEFAULT NULL COMMENT '是否删除',
    `create_user`            VARCHAR(32)        DEFAULT NULL COMMENT '创建人',
    `create_time`            datetime      NULL DEFAULT NULL COMMENT '创建时间',
    `update_user`            VARCHAR(32)        DEFAULT NULL COMMENT '更新人',
    `update_time`            datetime      NULL DEFAULT NULL COMMENT '更新时间',
    `delete_user`            VARCHAR(32)        DEFAULT NULL COMMENT '删除人',
    `delete_time`            datetime      NULL DEFAULT NULL COMMENT '删除时间',
    PRIMARY KEY (`id`)
) ENGINE = INNODB
  DEFAULT CHARSET = utf8 COMMENT = '审批流转记录表';


/*==============================================================*/
/* Table: T_APPROVE_ACT_HI_TASK                                 */
/*==============================================================*/
CREATE TABLE `t_approve_act_hi_task`
(
    `ID`                  varchar(32) NOT NULL COMMENT '主键',
    `APPLY_RECORD_ID`     varchar(32) DEFAULT NULL COMMENT '申请单ID',
    `BUSINESS_CODE`       varchar(40) DEFAULT NULL COMMENT '流程标识码',
    `BUSINESS_NAME`       varchar(50) DEFAULT NULL COMMENT '流程名称',
    `TASK_ID`             varchar(32) DEFAULT NULL COMMENT '任务ID',
    `TASK_CODE`           varchar(60) DEFAULT NULL COMMENT '任务代码',
    `TASK_NAME`           varchar(60) DEFAULT NULL COMMENT '任务名称',
    `OPERATE_USER_IDCARD` varchar(32) DEFAULT NULL COMMENT '处理人身份证号',
    `OPERATE_USER_NAME`   varchar(30) DEFAULT NULL COMMENT '处理人姓名',
    `OPERATE_TIME`        datetime    DEFAULT NULL COMMENT '操作时间',
    `OPERATE_TYPE`        varchar(40) DEFAULT NULL COMMENT '操作类型（详见ApproveWorkFlowOprationEnum）',
    `PROCESS_INSTANCE_ID` varchar(10) DEFAULT NULL COMMENT '流程实例ID',
    `DELETED`             varchar(1)  DEFAULT NULL COMMENT '是否删除标识',
    PRIMARY KEY (`ID`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8 COMMENT ='已办表';

/*==============================================================*/
/* Table: T_APPROVE_ACT_RUN_TASK                                */
/*==============================================================*/
CREATE TABLE `t_approve_act_run_task`
(
    `ID`                  varchar(32) NOT NULL COMMENT '主键',
    `APPLY_RECORD_ID`     varchar(32) DEFAULT NULL COMMENT '申请单ID',
    `BUSINESS_CODE`       varchar(32) DEFAULT NULL COMMENT '流程标识码',
    `BUSINESS_NAME`       varchar(50) DEFAULT NULL COMMENT '流程名称',
    `TASK_ID`             varchar(32) DEFAULT NULL COMMENT '任务ID',
    `TASK_CODE`           varchar(60) DEFAULT NULL COMMENT '任务代码',
    `TASK_NAME`           varchar(60) DEFAULT NULL COMMENT '任务名称',
    `OPERATE_TYPE`        varchar(40) DEFAULT NULL COMMENT '操作类型（详见ApproveWorkFlowOprationEnum）',
    `PROCESS_INSTANCE_ID` varchar(10) DEFAULT NULL COMMENT '流程实例ID',
    `GENERATE_TIME`       datetime    DEFAULT NULL COMMENT '生成时间',
    `DELETED`             varchar(1)  DEFAULT NULL COMMENT '删除标识',
    PRIMARY KEY (`ID`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8 COMMENT ='待办表';

/*==============================================================*/
/* Table: T_APPROVE_ACT_RUN_IDENTITY_LINK                       */
/*==============================================================*/
CREATE TABLE `t_approve_act_run_identity_link`
(
    `ID`                  varchar(32) NOT NULL COMMENT '主键',
    `APPLY_RECORD_ID`     varchar(32)  DEFAULT NULL COMMENT '申请单ID',
    `PROCESS_INSTANCE_ID` varchar(32)  DEFAULT NULL COMMENT '流程实例ID',
    `TASK_ID`             varchar(150) DEFAULT NULL COMMENT '任务ID',
    `GROUP_ID`            varchar(64)  DEFAULT NULL COMMENT '群组ID',
    `USER_ID`             varchar(64)  DEFAULT NULL COMMENT '用户ID',
    `DEPT_ID`             varchar(64)  DEFAULT NULL COMMENT '部门ID',
    `GROUP_NAME_LIST`     varchar(800) DEFAULT NULL COMMENT '群组名称列表',
    `USER_NAME_LIST`      varchar(800) DEFAULT NULL COMMENT '用户名称列表',
    `DEPT_NAME_LIST`      varchar(800) DEFAULT NULL COMMENT '用户部门列表',
    PRIMARY KEY (`ID`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8 COMMENT ='任务关联人员表';